###
# Copyright (c) 2015-2021, Antoine Le Gonidec <vv221@dotslashplay.it>
# Copyright (c) 2016-2021, Mopi
# Copyright (c) 2017-2021, Phil Morrell
# Copyright (c) 2017-2021, Jacek Szafarkiewicz
# Copyright (c) 2018-2021, VA
# Copyright (c) 2018-2021, Janeene "dawnmist" Beeforth
# Copyright (c) 2018-2021, BetaRays
# Copyright (c) 2018-2021, Andrey
# Copyright (c) 2019-2021, Emmanuel Gil Peyrot <linkmauve@linkmauve.fr>
# Copyright (c) 2020-2021, macaron
# Copyright (c) 2020-2021, Hoël Bézier
# Copyright (c) 2020-2021, quinao
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# This software is provided by the copyright holders and contributors "as is"
# and any express or implied warranties, including, but not limited to, the
# implied warranties of merchantability and fitness for a particular purpose
# are disclaimed. In no event shall the copyright holder or contributors be
# liable for any direct, indirect, incidental, special, exemplary, or
# consequential damages (including, but not limited to, procurement of
# substitute goods or services; loss of use, data, or profits; or business
# interruption) however caused and on any theory of liability, whether in
# contract, strict liability, or tort (including negligence or otherwise)
# arising in any way out of the use of this software, even if advised of the
# possibility of such damage.
###

###
# common functions for ./play.it scripts
# send your bug reports to contact@dotslashplay.it
###

LIBRARY_VERSION=2.13.3
# shellcheck disable=SC2034
library_revision=20210706.1

# test the validity of the argument given to parent function
# USAGE: testvar $var_name $pattern
testvar() {
	test "${1%%_*}" = "$2"
}

# set defaults rights on files (755 for dirs & 644 for regular files)
# USAGE: set_standard_permissions $dir[…]
set_standard_permissions() {
	[ "$DRY_RUN" -eq 1 ] && return 0
	for dir in "$@"; do
		[  -d "$dir" ] || return 1
		find "$dir" -type d -exec chmod 755 '{}' +
		find "$dir" -type f -exec chmod 644 '{}' +
	done
}

# convert files name to lower case
# USAGE: tolower $dir[…]
# CALLS: tolower_convmv tolower_shell
tolower() {
	[ "$DRY_RUN" -eq 1 ] && return 0
	for dir in "$@"; do
		[ -d "$dir" ] || return 1
		if command -v convmv > /dev/null; then
			tolower_convmv "$dir"
		else
			tolower_shell "$dir"
		fi
	done
}

# convert files name to lower case using convmv
# USAGE: tolower_convmv $directory
# RETURN: nothing
# SIDE EFFECT: convert all file names in a given path to lowercase
tolower_convmv() {
	# shellcheck disable=SC2039
	local directory
	directory="$1"

	###
	# TODO
	# Check that $directory is a writable directory
	###

	# shellcheck disable=SC2039
	local convmv_options
	convmv_options='-f utf8 --notest --lower -r'

	# shellcheck disable=SC2086
	find "$directory" -mindepth 1 -maxdepth 1 -exec \
		convmv $convmv_options {} + >/dev/null 2>&1
}

# convert files name to lower case using pure shell
# USAGE: tolower_shell $dir
# CALLED BY: tolower
tolower_shell() {
	local dir="$1"

	find "$dir" -depth -mindepth 1 | while read -r file; do
		newfile=$(dirname "$file")/$(basename "$file" | tr '[:upper:]' '[:lower:]')
		[ -e "$newfile" ] || mv "$file" "$newfile"
	done
}

# convert files name to upper case
# USAGE: toupper $dir[…]
# CALLS: toupper_convmv toupper_shell
toupper() {
	[ "$DRY_RUN" = '1' ] && return 0
	for dir in "$@"; do
		[ -d "$dir" ] || return 1
		if command -v convmv > /dev/null; then
			toupper_convmv "$dir"
		else
			toupper_shell "$dir"
		fi
	done
}

# convert files name to upper case using convmv
# USAGE: toupper_convmv $directory
# RETURN: nothing
# SIDE EFFECT: convert all file names in a given path to uppercase
toupper_convmv() {
	# shellcheck disable=SC2039
	local directory
	directory="$1"

	###
	# TODO
	# Check that $directory is a writable directory
	###

	# shellcheck disable=SC2039
	local convmv_options
	convmv_options='-f utf8 --notest --upper -r'

	# shellcheck disable=SC2086
	find "$directory" -mindepth 1 -maxdepth 1 -exec \
		convmv $convmv_options {} + >/dev/null 2>&1
}

# convert files name to upper case using pure shell
# USAGE: toupper_shell $dir
# CALLED BY: toupper
toupper_shell() {
	local dir="$1"

	find "$dir" -depth -mindepth 1 | while read -r file; do
		newfile="$(dirname "$file")/$(basename "$file" | tr '[:lower:]' '[:upper:]')"
		[ -e "$newfile" ] || mv "$file" "$newfile"
	done
}

# get archive-specific value for a given variable name, or use default value
# USAGE: use_archive_specific_value $variable_name
# RETURN: nothing
# SIDE EFFECT: update the value of the variables using the given name
use_archive_specific_value() {
	###
	# TODO
	# We should check that a valid shell variable name has been passed
	###
	# shellcheck disable=SC2039
	local variable_name variable_value
	variable_name="$1"

	# Try to get an archive-specific value for the given variable
	variable_value=$(get_archive_specific_value "$variable_name")

	# If no archive-specific value exists, fall back on the default value
	if [ -z "$variable_value" ]; then
		variable_value=$(get_value "$variable_name")
	fi

	# Update the variable value
	export "$variable_name"="$variable_value"
	return 0
}

# return an archive-specific value for a given variable name, if one exists
# USAGE: get_archive_specific_value $variable_name
# RETURN: an archive-specific value, or nothing
get_archive_specific_value() {
	# If $ARCHIVE is not set, return early
	if [ -z "$ARCHIVE" ]; then
		return 0
	fi

	# Try to find a variable using the base name + the current archive identifier suffix
	# shellcheck disable=SC2039
	local variable_base_name variable_name_with_suffix archive_suffix variable_value
	variable_base_name="$1"

	# Try first with "ARCHIVE_BASE_" as the base archive identifier
	# This step should be skipped with game scripts targeting a library version older than 2.13
	if \
		# shellcheck disable=SC2154
		version_is_at_least '2.13' "$target_version" && \
		[ "${ARCHIVE#ARCHIVE_BASE_}" != "$ARCHIVE" ]
	then
		archive_suffix="${ARCHIVE#ARCHIVE_BASE_}"
		variable_name_with_suffix="${variable_base_name}_${archive_suffix}"
		while [ "$variable_name_with_suffix" != "$variable_base_name" ]; do
			variable_value=$(get_value "$variable_name_with_suffix")
			if [ -n "$variable_value" ]; then
				printf '%s' "$variable_value"
				return 0
			fi
			variable_name_with_suffix="${variable_name_with_suffix%_*}"
		done
	fi

	# If no value has been found using "ARCHIVE_BASE_" as the base archive identifier, try again using "ARCHIVE_"
	if [ "${ARCHIVE#ARCHIVE_}" != "$ARCHIVE" ]; then
		archive_suffix="${ARCHIVE#ARCHIVE_}"
		variable_name_with_suffix="${variable_base_name}_${archive_suffix}"
		while [ "$variable_name_with_suffix" != "$variable_base_name" ]; do
			variable_value=$(get_value "$variable_name_with_suffix")
			if [ -n "$variable_value" ]; then
				printf '%s' "$variable_value"
				return 0
			fi
			variable_name_with_suffix="${variable_name_with_suffix%_*}"
		done
	fi

	# No archive-specific value has been found
	# This should not trigger an error
	return 0
}

# get package-specific value for a given variable name, or use default value
# USAGE: use_package_specific_value $var_name
use_package_specific_value() {
	# get the current package
	local package
	package=$(package_get_current)

	local name_real
	name_real="$1"
	local name
	name="${name_real}_${package#PKG_}"
	local value
	while [ "$name" != "$name_real" ]; do
		value="$(get_value "$name")"
		if [ -n "$value" ]; then
			export ${name_real?}="$value"
			return 0
		fi
		name="${name%_*}"
	done
}

# get the value of a variable and print it
# USAGE: get_value $variable_name
get_value() {
	local name
	local value
	name="$1"
	value="$(eval printf -- '%b' \"\$$name\")"
	printf '%s' "$value"
}

# check that the value assigned to a given option is valid
# USAGE: check_option_validity $option_name
# CALLS: error_option_invalid
check_option_validity() {
	local option_name option_value allowed_values
	option_name="$1"
	option_value=$(get_value "OPTION_${option_name}")
	allowed_values=$(get_value "ALLOWED_VALUES_${option_name}")
	for allowed_value in $allowed_values; do
		if [ "$option_value" = "$allowed_value" ]; then
			# leaves the function with a success code if the tested value is valid
			return 0
		fi
	done
	# if we did not leave the function before this point, the tested value is not valid
	option_name=$(printf '%s' "$option_name" | tr '[:upper:]' '[:lower:]')
	if [ "$option_name" = 'compression' ]; then
		error_compression_invalid
	else
		error_option_invalid "$option_name" "$option_value"
	fi
}

# try to guess the tar implementation used for `tar` on the current system
# USAGE: guess_tar_implementation
guess_tar_implementation() {
	case "$(tar --version | head --lines 1)" in
		(*'GNU tar'*)
			PLAYIT_TAR_IMPLEMENTATION='gnutar'
		;;
		(*'libarchive'*)
			PLAYIT_TAR_IMPLEMENTATION='bsdtar'
		;;
		(*)
			error_unknown_tar_implementation
		;;
	esac
}

# set up a required archive
# USAGE: archive_initialize_required $archive_name $archive_candidate[…]
# RETURNS: nothing
archive_initialize_required() {
	local archive_name archive_candidate

	debug_entering_function 'archive_initialize_required'

	archive_name="$1"
	shift 1

	archive_candidate=$(archive_find_from_candidates "$archive_name" "$@")

	# Throw an error if no archive candidate has been found
	if [ -z "$archive_candidate" ]; then
		error_archive_not_found "$@"
	fi

	# Call common part of archive initialization
	archive_initialize "$archive_name" "$archive_candidate"

	debug_leaving_function 'archive_initialize_required'

	return 0
}

# set up an optional archive
# USAGE: archive_initialize_optional $archive_name $archive_candidate[…]
# RETURNS: nothing
archive_initialize_optional() {
	local archive_name archive_candidate

	debug_entering_function 'archive_initialize_optional'

	archive_name="$1"
	shift 1

	archive_candidate=$(archive_find_from_candidates "$archive_name" "$@")

	# Return early if no archive candidate has been found
	if [ -z "$archive_candidate" ]; then
		return 0
	fi

	# Call common part of archive initialization
	archive_initialize "$archive_name" "$archive_candidate"

	debug_leaving_function 'archive_initialize_optional'

	return 0
}

# common part of an archive initialization
# USAGE: archive_initialize $archive_name $archive_candidate
# RETURNS: nothing
archive_initialize() {
	local archive_name archive_candidate archive_path archive_part_name archive_part_candidate
	archive_name="$1"
	archive_candidate="$2"

	archive_path=$(archive_find_path "$archive_candidate")

	###
	# TODO
	# Check that the archive path is not empty
	###

	# Set the current archive properties from the candidate one
	archive_set_properties_from_candidate "$archive_name" "$archive_candidate"

	# Check archive integrity if it comes with a MD5 hash
	if [ -n "$(get_value "${archive_name}_MD5")" ]; then
		archive_integrity_check "$archive_candidate" "$archive_path" "$archive_name"
	fi

	# Check dependencies
	check_deps

	# Update total size of all archives
	archive_add_size_to_total "$archive_name"

	# Check for archive extra parts
	for i in $(seq 1 9); do
		archive_part_name="${archive_name}_PART${i}"
		archive_part_candidate="${archive_candidate}_PART${i}"
		test -z "$(get_value "$archive_part_candidate")" && break
		archive_initialize_required "$archive_part_name" "$archive_part_candidate"
	done

	return 0
}

# find a single archive from a list of candidates
# USAGE: archive_find_from_candidates $archive_name $archive_candidate[…]
# RETURNS: an archive identifier, or nothing
archive_find_from_candidates() {
	local archive_name archive_candidate file_name file_path current_archive_path
	archive_name="$1"
	shift 1

	# An archive path might already be set, if it has been passed on the command line
	if [ -n "$(get_value $archive_name)" ]; then
		current_archive_path="$(get_value $archive_name)"
	fi

	# Loop around archive candidates, stopping on the first one found
	for archive_candidate in "$@"; do
		if [ -n "$current_archive_path" ]; then
			file_name=$(get_value "$archive_candidate")
			if [ "$(basename "$current_archive_path")" = "$file_name" ]; then
				file_path=$(realpath "$current_archive_path")
			fi

		else
			file_path=$(archive_find_path "$archive_candidate")
		fi
		test -n "$file_path" && break
	done

	# Return early if no archive candidate has been found
	if [ -z "$file_path" ]; then
		return 0
	fi

	# Return the identifier of the found archive candidate
	printf '%s' "$archive_candidate"
	return 0
}

# return the absolute path to a given archive
# USAGE: archive_find_path $archive_name
# RETURNS: an absolute file path, or nothing
archive_find_path() {
	local archive_name archive_filename archive_path
	archive_name="$1"

	# Right now, an archive can only be found using its file name
	archive_filename=$(get_value "$archive_name")
	archive_path=$(archive_find_path_from_name "$archive_filename")

	printf '%s' "$archive_path"
	return 0
}

# find an archive from its file name
# USAGE: archive_find_path_from_name $file_name
# RETURNS: an absolute file path, or nothing
archive_find_path_from_name() {
	local file_name
	file_name="$1"

	###
	# TODO
	# Check that the provided file name is not empty
	###

	# Look for the archive in current directory
	if [ -f "$PWD/$file_name" ]; then
		file_path=$(realpath "$PWD/$file_name")
		printf '%s' "$file_name"
		return 0
	fi

	# Look for the archive in the same directory than the main archive
	if \
		[ -n "$SOURCE_ARCHIVE" ] && \
		[ -f "$(dirname "$SOURCE_ARCHIVE")/$file_name" ]
	then
		file_path=$(realpath "$(dirname "$SOURCE_ARCHIVE")/$file_name")
		printf '%s' "$file_path"
		return 0
	fi

	# No archive found, return nothing
	return 0
}

# set an archive properties from a candidate informations
# USAGE: archive_set_properties_from_candidate $archive_name $archive_candidate
# RETURNS: nothing
archive_set_properties_from_candidate() {
	local archive_name archive_candidate archive_path property
	archive_name="$1"
	archive_candidate="$2"

	###
	# TODO
	# Check that the provided archive name is not empty
	###

	###
	# TODO
	# Check that the provided archive candidate is not empty
	###

	# Set archive path
	archive_path=$(archive_find_path "$archive_candidate")

	###
	# TODO
	# Check that the archive path is not empty
	###

	export "${archive_name}=$archive_path"

	# Print information message
	information_file_in_use "$archive_path"

	# Set list of extra archive parts
	for i in $(seq 1 9); do
		export "${archive_name}_PART${i}=$(get_value "${archive_candidate}_PART${i}")"
	done

	# Set other archive properties
	for property in \
		'MD5' \
		'TYPE' \
		'SIZE' \
		'VERSION'
	do
		export "${archive_name}_${property}=$(get_value "${archive_candidate}_${property}")"
	done

	return 0
}

# add the size of given archive to the total size of all archives in use
# USAGE: archive_add_size_to_total $archive
# RETURNS: nothing
archive_add_size_to_total() {
	local archive archive_size
	archive="$1"

	###
	# TODO
	# Check that the provided archive name is not empty
	###

	# Get the given archive size, defaults to a size of 0
	archive_size=$(get_value "${archive}_SIZE")
	: "${archive_size:=0}"

	# Update the total size of all archives in use
	: "${ARCHIVE_SIZE:=0}"
	ARCHIVE_SIZE=$((ARCHIVE_SIZE + archive_size))
	export ARCHIVE_SIZE

	return 0
}

# get the type of a given archive
# USAGE: archive_get_type $archive_identifier
# RETURNS: an archive type
archive_get_type() {
	# Get the archive identifier, check that it is not empty
	# shellcheck disable=SC2039
	local archive_identifier
	archive_identifier="$1"
	if [ -z "$archive_identifier" ]; then
		error_empty_string 'archive_get_type' 'archive_identifier'
		return 1
	fi

	# Return archive type early if it is already set
	# shellcheck disable=SC2039
	local archive_type
	archive_type=$(get_value "${archive_identifier}_TYPE")
	if [ -n "$archive_type" ]; then
		printf '%s' "$archive_type"
		return 0
	fi

	# Guess archive type from its file name
	# shellcheck disable=SC2039
	local archive_file
	archive_file=$(get_value "$archive_identifier")
	case "$archive_file" in
		(*'.cab')
			archive_type='cabinet'
		;;
		(*'.deb')
			archive_type='debian'
		;;
		('setup_'*'.exe'|'patch_'*'.exe')
			archive_type='innosetup'
		;;
		('gog_'*'.sh')
			archive_type='mojosetup'
		;;
		(*'.iso')
			archive_type='iso'
		;;
		(*'.msi')
			archive_type='msi'
		;;
		(*'.rar')
			archive_type='rar'
		;;
		(*'.tar')
			archive_type='tar'
		;;
		(*'.tar.gz'|*'.tgz')
			archive_type='tar.gz'
		;;
		(*'.zip')
			archive_type='zip'
		;;
		(*'.7z')
			archive_type='7z'
		;;
		(*)
			error_archive_type_not_set "$archive_identifier"
		;;
	esac

	# Return guessed type
	printf '%s' "$archive_type"
	return 0
}

# check integrity of target file
# USAGE: archive_integrity_check $archive $file ($name)
# CALLS: archive_integrity_check_md5
archive_integrity_check() {
	local archive
	local file
	local name
	archive="$1"
	file="$2"
	name="$3"
	case "$OPTION_CHECKSUM" in
		('md5')
			archive_integrity_check_md5 "$archive" "$file" "$name"
		;;
		('none')
			return 0
		;;
		(*)
			error_invalid_argument 'OPTION_CHECKSUM' 'archive_integrity_check'
		;;
	esac
}

# return the list of supported archives for the current script
# USAGE: archives_return_list
# RETURNS: the list of identifiers of the supported archives,
#          as a list of strings separated by spaces or line breaks
archives_return_list() {
	# If a list is already explicitely set, return early
	# shellcheck disable=SC2153
	if [ -n "$ARCHIVES_LIST" ]; then
		printf '%s' "$ARCHIVES_LIST"
		return 0
	fi

	# Parse the calling script to guess the identifiers of the archives it supports
	# shellcheck disable=SC2039
	local script pattern
	script="$0"

	# Try to find archives using the ARCHIVE_BASE_xxx_[0-9]+ naming scheme
	# Fall back to the older naming scheme for scripts targeting a library older than 2.13
	# shellcheck disable=SC2039
	local archives_list pattern
	# shellcheck disable=SC2154
	if version_is_at_least '2.13' "$target_version"; then
		pattern='^ARCHIVE_BASE\(_[0-9A-Z]\+\)*_[0-9]\+='
	else
		pattern='^ARCHIVE_[0-9A-Z]\+\(_OLD[0-9A-Z]*\)*='
	fi
	archives_list=$(grep \
		--regexp="$pattern" "$script" | \
		cut --delimiter='=' --fields=1)

	# Returns the list of found archives
	if [ -n "$archives_list" ]; then
		printf '%s' "$archives_list"
		return 0
	fi

	# Fall back on trying to find archives using the old naming scheme
	# This will be deprecated in some future release
	pattern='^ARCHIVE_[0-9A-Z]\+\(_OLD[0-9A-Z]*\)*='
	archives_list=$(grep \
		--regexp="$pattern" "$script" | \
		cut --delimiter='=' --fields=1)

	# Returns the list of found archives
	if [ -n "$archives_list" ]; then
		printf '%s' "$archives_list"
		return 0
	fi

	# The current script does not seem to support any archive
	# This should not happen
	return 1
}

# get cached md5sum identified by a name
# USAGE: archive_get_md5sum_cached $name
# CALLS: error_empty_string
archive_get_md5sum_cached() {
	local name md5sum
	name="$1"
	if [ -z "$name" ]; then
		error_empty_string 'archive_get_md5sum_cached' 'name'
	fi
	get_value "${name}_CACHED_MD5SUM"
}

# compute md5sum for a given file, store it in a cached variable
# USAGE: archive_get_md5sum_computed $name $file
# CALLS: error_empty_string error_not_a_file error_unavailable_command
archive_get_md5sum_computed() {
	local name file md5sum
	name="$1"
	file="$2"
	if [ -z "$name" ]; then
		error_empty_string 'archive_get_md5sum_computed' 'name'
	fi
	if [ -z "$file" ]; then
		error_empty_string 'archive_get_md5sum_computed' 'file'
	fi
	if [ ! -f "$file" ]; then
		error_not_a_file "$file"
	fi
	if ! command -v 'md5sum' 'awk' >/dev/null 2>&1; then
		error_unavailable_command 'archive_get_md5sum_computed' 'md5sum'
	fi
	md5sum=$(md5sum "$file" | awk '{print $1}')
	export "${name?}_CACHED_MD5SUM=$md5sum"
}

# check if there is a cached md5sum value for the given name
# USAGE: archive_has_md5sum_cached $name
# CALLS: archive_get_md5sum_cached
archive_has_md5sum_cached() {
	local name
	name="$1"
	if [ -z "$name" ]; then
		error_empty_string 'archive_has_md5sum_cached' 'name'
	fi
	test -n "$(archive_get_md5sum_cached "$name")"
}

# check integrity of target file against MD5 control sum
# USAGE: archive_integrity_check_md5 $archive $file ($name)
# CALLS: info_archive_integrity_check error_hashsum_mismatch
# CALLED BY: archive_integrity_check
archive_integrity_check_md5() {
	local archive file name
	archive="$1"
	file="$2"
	name="$3"
	if [ -z "$archive" ]; then
		error_empty_string 'archive_integrity_check_md5' 'archive'
	fi
	if [ -z "$file" ]; then
		error_empty_string 'archive_integrity_check_md5' 'file'
	fi
	if [ -z "$name" ]; then
		error_empty_string 'archive_integrity_check_md5' 'name'
	fi
	if [ ! -f "$file" ]; then
		error_not_a_file "$file"
	fi
	# Cache MD5 hash here to prevent it from getting ignored in a subshell
	if ! archive_has_md5sum_cached "$name"; then
		info_archive_hash_computation "$file"
		archive_get_md5sum_computed "$name" "$file"
		info_archive_hash_computation_done
	fi
	archive_sum="$(get_value "${archive}_MD5")"
	file_sum="$(archive_get_md5sum_cached "$name")"
	if [ "$file_sum" != "$archive_sum" ]; then
		error_hashsum_mismatch "$file"
	fi
}

# check script dependencies
# USAGE: check_deps
# NEEDED VARS: (ARCHIVE) (OPTION_CHECKSUM) (OPTION_PACKAGE) (SCRIPT_DEPS)
# CALLS: check_deps_7z error_dependency_not_found icons_list_dependencies
check_deps() {
	local archive_type

	if [ "$ARCHIVE" ]; then
		archive_type=$(archive_get_type "$ARCHIVE")
		case "$archive_type" in
			('cabinet')
				SCRIPT_DEPS="$SCRIPT_DEPS cabextract"
			;;
			('debian')
				SCRIPT_DEPS="$SCRIPT_DEPS debian"
			;;
			('innosetup1.7'*)
				SCRIPT_DEPS="$SCRIPT_DEPS innoextract1.7"
			;;
			('innosetup'*)
				SCRIPT_DEPS="$SCRIPT_DEPS innoextract"
			;;
			('installshield')
				SCRIPT_DEPS="$SCRIPT_DEPS unshield"
			;;
			('lha')
				SCRIPT_DEPS="$SCRIPT_DEPS lha"
			;;
			('nixstaller')
				SCRIPT_DEPS="$SCRIPT_DEPS gzip tar unxz"
			;;
			('msi')
				SCRIPT_DEPS="$SCRIPT_DEPS msiextract"
			;;
			('mojosetup'|'iso')
				SCRIPT_DEPS="$SCRIPT_DEPS bsdtar"
			;;
			('rar'|'nullsoft-installer')
				SCRIPT_DEPS="$SCRIPT_DEPS unar"
			;;
			('tar')
				SCRIPT_DEPS="$SCRIPT_DEPS tar"
			;;
			('tar.gz')
				SCRIPT_DEPS="$SCRIPT_DEPS gzip tar"
			;;
			('zip'|'zip_unclean'|'mojosetup_unzip')
				SCRIPT_DEPS="$SCRIPT_DEPS unzip"
			;;
		esac
	fi
	case "$OPTION_COMPRESSION" in
		('gzip')
			SCRIPT_DEPS="$SCRIPT_DEPS gzip"
		;;
		('xz')
			SCRIPT_DEPS="$SCRIPT_DEPS xz"
		;;
		('bzip2')
			SCRIPT_DEPS="$SCRIPT_DEPS bzip2"
		;;
		('zstd')
			SCRIPT_DEPS="$SCRIPT_DEPS zstd"
		;;
		('lz4')
			SCRIPT_DEPS="$SCRIPT_DEPS lz4"
		;;
		('lzip')
			SCRIPT_DEPS="$SCRIPT_DEPS lzip"
		;;
		('lzop')
			SCRIPT_DEPS="$SCRIPT_DEPS lzop"
		;;
	esac
	if [ "$OPTION_CHECKSUM" = 'md5sum' ]; then
		SCRIPT_DEPS="$SCRIPT_DEPS md5sum"
	fi
	if [ "$OPTION_PACKAGE" = 'deb' ]; then
		SCRIPT_DEPS="$SCRIPT_DEPS fakeroot dpkg"
	fi
	if [ "$OPTION_PACKAGE" = 'gentoo' ]; then
		# fakeroot-ng doesn't work anymore, fakeroot >=1.25.1 does
		SCRIPT_DEPS="$SCRIPT_DEPS fakeroot:>=1.25.1 ebuild"
	fi
	for dep in $SCRIPT_DEPS; do
		case $dep in
			('7z')
				check_deps_7z
			;;
			('innoextract'*)
				check_deps_innoextract "$dep"
			;;
			('lha')
				check_deps_lha
			;;
			('debian')
				check_deps_debian
			;;
			('fakeroot'*)
				check_deps_fakeroot "$dep"
			;;
			(*)
				if ! command -v "$dep" >/dev/null 2>&1; then
					error_dependency_not_found "$dep"
				fi
			;;
		esac
	done

	# Check for the dependencies required to extract the icons
	unset ICONS_DEPS
	icons_list_dependencies
	for dep in $ICONS_DEPS; do
		if ! command -v "$dep" >/dev/null 2>&1; then
			case "$OPTION_ICONS" in
				('yes')
					error_icon_dependency_not_found "$dep"
				;;
				('auto')
					warning_icon_dependency_not_found "$dep"
					export SKIP_ICONS=1
					break
				;;
			esac
		fi
	done
}

# check presence of a software to handle .7z archives
# USAGE: check_deps_7z
# CALLS: error_dependency_not_found
# CALLED BY: check_deps
check_deps_7z() {
	for command in '7zr' '7za' 'unar'; do
		if command -v "$command" >/dev/null 2>&1; then
			return 0
		fi
	done
	error_dependency_not_found '7zr'
}

# check presence of a software to handle LHA (.lzh) archives
# USAGE: check_deps_lha
# CALLS: error_dependency_not_found
# CALLED BY: check_deps
check_deps_lha() {
	for command in 'lha' 'bsdtar'; do
		if command -v "$command" >/dev/null 2>&1; then
			return 0
		fi
	done
	error_dependency_not_found 'lha'
}

# check presence of a software to handle .deb packages
# USAGE: check_deps_debian
# CALLS: error_dependency_not_found
# CALLED BY: check_deps
check_deps_debian() {
	for command in 'dpkg-deb' 'bsdtar' 'unar'; do
		if command -v "$command" >/dev/null 2>&1; then
			return 0
		fi
	done
	if command -v 'tar' >/dev/null 2>&1; then
		for command in '7z' '7zr' 'ar'; do
			if command -v "$command" >/dev/null 2>&1; then
				return 0
			fi
		done
	fi
	error_dependency_not_found 'dpkg-deb'
}

# check innoextract presence, optionally in a given minimum version
# USAGE: check_deps_innoextract $keyword
# CALLS: error_dependency_not_found
# CALLED BYD: check_deps
check_deps_innoextract() {
	local keyword
	local name
	keyword="$1"
	case "$keyword" in
		('innoextract1.7')
			name='innoextract (>= 1.7)'
		;;
		(*)
			name='innoextract'
		;;
	esac
	if ! command -v 'innoextract' >/dev/null 2>&1; then
		error_dependency_not_found "$name"
	fi

	# Check innoextract version
	# shellcheck disable=SC2039
	local innoextract_version
	innoextract_version=$(LANG=C innoextract --version | head --lines=1 | cut --delimiter=' ' --fields=2)
	case "$keyword" in
		('innoextract1.7')
			if ! version_is_at_least '1.7' "$innoextract_version"; then
				error_dependency_not_found "$name"
			fi
		;;
	esac

	return 0
}

check_deps_fakeroot() {
	local keyword
	local name
	keyword="$1"
	case "$keyword" in
		('fakeroot:>=1.25.1')
			name='fakeroot (>=1.25.1)'
		;;
		(*)
			name='fakeroot'
		;;
	esac
	if ! command -v 'fakeroot' >/dev/null 2>&1; then
		error_dependency_not_found "$name"
	fi

	# Check fakeroot version
	# shellcheck disable=SC2039
	local fakeroot_version
	fakeroot_version="$(LANG=C fakeroot --version | cut --delimiter=' ' --fields=3)"
	case "$keyword" in
		('fakeroot:>=1.25.1')
			if ! version_is_at_least '1.25.1' "$fakeroot_version"; then
				error_dependency_not_found "$name"
			fi
		;;
	esac

	return 0
}

# output what a command is provided by
# USAGE: dependency_provided_by $command
# CALLED BY: error_dependency_not_found
dependency_provided_by() {
	local command provider
	command="$1"
	case "$command" in
		('7zr')
			provider='p7zip'
		;;
		('bsdtar')
			provider='libarchive'
		;;
		('convert'|'identify')
			provider='imagemagick'
		;;
		('lha')
			provider='lhasa'
		;;
		('icotool'|'wrestool')
			provider='icoutils'
		;;
		('dpkg-deb')
			provider='dpkg'
		;;
		(*)
			provider="$command"
		;;
	esac
	printf '%s' "$provider"
	return 0
}

# select package architecture to build
# USAGE: select_package_architecture
# NEEDED_VARS: OPTION_ARCHITECTURE
select_package_architecture() {
	[ "$OPTION_ARCHITECTURE" = 'all' ] && return 0
	if ! version_is_at_least '2.6' "$target_version"; then
		warning_option_not_supported '--architecture'
		OPTION_ARCHITECTURE='all'
		export OPTION_ARCHITECTURE
		return 0
	fi
	if [ "$OPTION_ARCHITECTURE" = 'auto' ]; then
		case "$(uname --machine)" in
			('i686')
				OPTION_ARCHITECTURE='32'
			;;
			('x86_64')
				OPTION_ARCHITECTURE='64'
			;;
			(*)
				select_package_architecture_warning_unknown
				OPTION_ARCHITECTURE='all'
				export OPTION_ARCHITECTURE
				return 0
			;;
		esac
		export OPTION_ARCHITECTURE
		select_package_architecture
		return 0
	fi
	local package_arch
	local packages_list_32
	local packages_list_64
	local packages_list_all

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	for package in $packages_list; do
		package_arch="$(get_value "${package}_ARCH")"
		case "$package_arch" in
			('32')
				packages_list_32="$packages_list_32 $package"
			;;
			('64')
				packages_list_64="$packages_list_64 $package"
			;;
			(*)
				packages_list_all="$packages_list_all $package"
			;;
		esac
	done
	case "$OPTION_ARCHITECTURE" in
		('32')
			if [ -z "$packages_list_32" ]; then
				warning_architecture_not_available "$OPTION_ARCHITECTURE"
				OPTION_ARCHITECTURE='all'
				return 0
			fi
			PACKAGES_LIST="$packages_list_32 $packages_list_all"
		;;
		('64')
			if [ -z "$packages_list_64" ]; then
				warning_architecture_not_available "$OPTION_ARCHITECTURE"
				OPTION_ARCHITECTURE='all'
				return 0
			fi
			PACKAGES_LIST="$packages_list_64 $packages_list_all"
		;;
		(*)
			error_architecture_not_supported "$OPTION_ARCHITECTURE"
		;;
	esac
	export PACKAGES_LIST
}

# set temporary directories
# USAGE: set_temp_directories $pkg[…]
# NEEDED VARS: (ARCHIVE_SIZE) GAME_ID (LANG) (PWD) (XDG_CACHE_HOME) (XDG_RUNTIME_DIR)
# CALLS: testvar temporary_directories_find_base check_directory_is_case_sensitive
#	check_directory_supports_unix_permissions
# SETS: PLAYIT_WORKDIR postinst prerm
set_temp_directories() {
	local base_directory

	debug_entering_function 'set_temp_directories'

	# If $PLAYIT_WORKDIR is already set, delete it before setting a new one
	[ "$PLAYIT_WORKDIR" ] && rm --force --recursive "$PLAYIT_WORKDIR"

	# Look for a directory with enough free space to work in
	# shellcheck disable=SC2039
	base_directory=$(temporary_directories_find_base)

	# Check that the candidate temporary directory is on a case-sensitive filesystem
	if ! check_directory_is_case_sensitive "$base_directory"; then
		error_case_insensitive_filesystem_is_not_supported "$base_directory"
		return 1
	fi

	# Check that the candidate temporary directory is on a filesystem with support for UNIX permissions
	if ! check_directory_supports_unix_permissions "$base_directory"; then
		error_unix_permissions_support_is_required "$base_directory"
		return 1
	fi

	# Generate a directory with a unique name for the current instance
	PLAYIT_WORKDIR="$(mktemp --directory --tmpdir="$base_directory" "${GAME_ID}.XXXXX")"
	debug_option_value 'PLAYIT_WORKDIR'
	debug_creating_directory "$PLAYIT_WORKDIR"
	export PLAYIT_WORKDIR

	# Set $postinst and $prerm
	mkdir --parents "$PLAYIT_WORKDIR/scripts"
	postinst="$PLAYIT_WORKDIR/scripts/postinst"
	export postinst
	prerm="$PLAYIT_WORKDIR/scripts/prerm"
	export prerm

	# Export the path to the packages to build as PKG_xxx_PATH
	# Some game scripts are expecting this variable to be set
	# These should be updated to call `package_get_path` instead
	# shellcheck disable=SC2039
	local package
	for package in "$@"; do
		testvar "$package" 'PKG'
		eval "${package}_PATH='$(package_get_path "$package")'"
		export "${package?}_PATH"
	done

	debug_leaving_function 'set_temp_directories'
}

# get available temporary directories list
# For now this list is hardcoded, but might be read in the future from a
# configuration file
# USAGE: temporary_directories_list_candidates
# RETURNS: a list of candidate directories to create ./play.it temporary work directory in,
#          separated by line breaks
temporary_directories_list_candidates() {
	printf '%s\n' \
		"$(realpath "${TMPDIR:-/tmp}")" \
		"${XDG_CACHE_HOME:=$HOME/.cache}" \
		"$PWD"
	return 0
}

# find a temp directory with enough free space to work in
# USAGE: temporary_directories_find_base
# RETURNS: the path to a directory with enough free space to use for ./play.it temporary files
temporary_directories_find_base() {
	debug_entering_function 'temporary_directories_find_base' 2

	# shellcheck disable=SC2039
	local base_directory
	if [ $NO_FREE_SPACE_CHECK -eq 1 ]; then
		# If free space check has been explicitely disabled,
		# use the first directory returned by temporary_directories_list_candidates
		# shellcheck disable=SC2039
		local default_directory
		default_directory="$(temporary_directories_list_candidates | head --lines=1)"
		base_directory=$(temporary_directories_full_path "$default_directory")
	else
		###
		# TODO
		# The required free space should be returned by a dedicated function,
		# instead of relying on a global variable.
		###
		# Fail early if the required free space for the current archive is not computable
		if [ -z "$ARCHIVE_SIZE" ]; then
			error_variable_not_set 'temporary_directories_find_base' 'ARCHIVE_SIZE'
			return 1
		fi

		# Scan candidate directories to find one with enough free space to use for storing temporary files
		# shellcheck disable=SC2039
		local free_space_required free_space_available candidate_directory
		free_space_required=$((ARCHIVE_SIZE * 2))
		while read -r candidate_directory
		do
			if [ ! -d "$candidate_directory" ]; then
				debug_temp_dir_nonexistant "$candidate_directory"
				continue
			fi
			if [ ! -w "$candidate_directory" ]; then
				debug_temp_dir_nonwritable "$candidate_directory"
				continue
			fi
			free_space_available=$(LANG=C df \
				--block-size=1K \
				--output=avail \
				"$candidate_directory" 2>/dev/null | \
				tail --lines=1)
			if [ $free_space_available -ge $free_space_required ]; then
				base_directory=$(temporary_directories_full_path "$candidate_directory")
				break;
			fi
			debug_temp_dir_not_enough_space "$candidate_directory"
		done <<- EOF
		$(temporary_directories_list_candidates)
		EOF

		# Fail with an explicit error if no valid candidate has been found
		if [ -z "$base_directory" ]; then
			# shellcheck disable=SC2046
			error_not_enough_free_space $(temporary_directories_list_candidates)
			return 1
		fi
	fi

	debug_using_directory "$base_directory"
	mkdir --parents "$base_directory"

	printf '%s' "$base_directory"
	debug_leaving_function 'temporary_directories_find_base' 2
	return 0
}

# Get play.it working directory full path
# USAGE: temporary_directories_full_path $parent_dir
# RETURN: the full path to use for ./play.it working directory,
#         it ends with "/play.it" if the parent directory is owned by the current user,
#         or with "/play.it-${USER}" otherwise, where "${USER}" is the current user name
temporary_directories_full_path() {
	# shellcheck disable=SC2039
	local parent_dir
	parent_dir="$1"
	if [ ! -d "$parent_dir" ]; then
		error_not_a_directory "$parent_dir"
	fi

	if [ "$(stat -c '%u' "$parent_dir")" -eq "$(id -u)" ]; then
		printf '%s' "$parent_dir/play.it"
	else
		printf '%s' "$parent_dir/play.it-${USER}"
	fi

	return 0
}
# extract data from given archive
# USAGE: extract_data_from $archive[…]
# NEEDED_VARS: (ARCHIVE) (ARCHIVE_PASSWD) (LANG) (PLAYIT_WORKDIR)
# CALLS: extract_7z
extract_data_from() {
	[ "$PLAYIT_WORKDIR" ] || return 1
	[ "$ARCHIVE" ] || return 1

	debug_entering_function 'extract_data_from'

	local file
	for file in "$@"; do
		information_archive_data_extraction "$(basename "$file")"

		local destination
		destination="$PLAYIT_WORKDIR/gamedata"
		mkdir --parents "$destination"
		if [ "$DRY_RUN" -eq 1 ]; then
			printf '\n'
			return 0
		fi
		local archive_type
		archive_type=$(archive_get_type "$ARCHIVE")
		case "$archive_type" in
			('7z')
				archive_extraction_7z "$file" "$destination"
			;;
			('cabinet')
				debug_external_command "cabextract -L -d \"$destination\" -q \"$file\""
				cabextract -L -d "$destination" -q "$file"
			;;
			('debian')
				archive_extraction_debian "$file" "$destination"
			;;
			('innosetup'*)
				archive_extraction_innosetup "$archive_type" "$file" "$destination"
			;;
			('installshield')
				debug_external_command "unshield -L -d \"$destination\" x \"$file\" >/dev/null"
				unshield -L -d "$destination" x "$file" >/dev/null
			;;
			('lha')
				archive_extraction_lha "$file" "$destination"
			;;
			('msi')
				debug_external_command "msiextract --directory \"$destination\" \"$file\" 1>/dev/null 2>&1"
				msiextract --directory "$destination" "$file" 1>/dev/null 2>&1
				tolower "$destination"
			;;
			('mojosetup'|'iso')
				debug_external_command "bsdtar --directory \"$destination\" --extract --file \"$file\""
				bsdtar --directory "$destination" --extract --file "$file"
				set_standard_permissions "$destination"
			;;
			('nix_stage1')
				local header_length
				local input_blocksize
				header_length="$(grep --text 'offset=.*head.*wc' "$file" | awk '{print $3}' | head --lines=1)"
				input_blocksize=$(head --lines="$header_length" "$file" | wc --bytes | tr --delete ' ')
				debug_external_command "dd if=\"$file\" ibs=\"$input_blocksize\" skip=1 obs=1024 conv=sync 2>/dev/null | gunzip --stdout | tar --extract --file - --directory \"$destination\""
				dd if="$file" ibs=$input_blocksize skip=1 obs=1024 conv=sync 2>/dev/null | gunzip --stdout | tar --extract --file - --directory "$destination"
			;;
			('nix_stage2')
				debug_external_command "tar --extract --xz --file \"$file\" --directory \"$destination\""
				tar --extract --xz --file "$file" --directory "$destination"
			;;
			('rar'|'nullsoft-installer')
				# compute archive password from GOG id
				if [ -z "$ARCHIVE_PASSWD" ] && [ -n "$(get_value "${ARCHIVE}_GOGID")" ]; then
					ARCHIVE_PASSWD="$(printf '%s' "$(get_value "${ARCHIVE}_GOGID")" | md5sum | cut -d' ' -f1)"
				fi
				if [ -n "$ARCHIVE_PASSWD" ]; then
					UNAR_OPTIONS="-password $ARCHIVE_PASSWD"
				fi
				debug_external_command "unar -no-directory -output-directory \"$destination\" $UNAR_OPTIONS \"$file\" 1>/dev/null"
				unar -no-directory -output-directory "$destination" $UNAR_OPTIONS "$file" 1>/dev/null
			;;
			('tar'|'tar.gz')
				debug_external_command "tar --extract --file \"$file\" --directory \"$destination\""
				tar --extract --file "$file" --directory "$destination"
			;;
			('zip')
				debug_external_command "unzip -d \"$destination\" \"$file\" 1>/dev/null"
				unzip -d "$destination" "$file" 1>/dev/null
			;;
			('zip_unclean'|'mojosetup_unzip')
				set +o errexit
				debug_external_command "unzip -d \"$destination\" \"$file\" 1>/dev/null 2>&1"
				unzip -d "$destination" "$file" 1>/dev/null 2>&1
				set -o errexit
				set_standard_permissions "$destination"
			;;
			(*)
				error_invalid_argument "${ARCHIVE}_TYPE" 'extract_data_from'
			;;
		esac

		information_archive_data_extraction_done
	done

	debug_leaving_function 'extract_data_from'
}

# extract data from 7z archive
# USAGE: archive_extraction_7z $archive $destination
# CALLS: error_archive_extraction_7z_no_extractor_found
# CALLED BY: extract_data_from
archive_extraction_7z() {
	local file destination
	file="$1"
	destination="$2"
	if command -v 7zr >/dev/null 2>&1; then
		debug_external_command "7zr x -o\"$destination\" -y \"$file\" 1>/dev/null"
		7zr x -o"$destination" -y "$file" 1>/dev/null
	elif command -v 7za >/dev/null 2>&1; then
		debug_external_command "7za x -o\"$destination\" -y \"$file\" 1>/dev/null"
		7za x -o"$destination" -y "$file" 1>/dev/null
	elif command -v unar >/dev/null 2>&1; then
		debug_external_command "unar -output-directory \"$destination\" -force-overwrite -no-directory \"$file\" 1>/dev/null"
		unar -output-directory "$destination" -force-overwrite -no-directory "$file" 1>/dev/null
	else
		error_archive_no_extractor_found '7z'
	fi
}

# extract data from LHA archive
# USAGE: archive_extraction_lha $archive $destination
# CALLS: error_archive_no_extractor_found
# CALLED BY: extract_data_from
archive_extraction_lha() {
	local file destination
	file="$1"
	destination="$2"
	if command -v lha >/dev/null 2>&1; then
		debug_external_command "lha -ew=\"$destination\" \"$file\" >/dev/null"
		lha -ew="$destination" "$file" >/dev/null
		set_standard_permissions "$destination"
	elif command -v bsdtar >/dev/null 2>&1; then
		debug_external_command "bsdtar --directory \"$destination\" --extract --file \"$file\""
		bsdtar --directory "$destination" --extract --file "$file"
		set_standard_permissions "$destination"
	else
		error_archive_no_extractor_found 'lha'
	fi
}

# extract data from .deb package
# USAGE: archive_extraction_debian $archive $destination
# CALLS: error_archive_extraction_debian_no_extractor_found
# CALLED BY: extract_data_from
archive_extraction_debian() {
	local file destination tmpdir
	file=$(realpath --canonicalize-existing "$1")
	destination="$2"
	tmpdir="$PLAYIT_WORKDIR/extraction"
	if command -v dpkg-deb >/dev/null 2>&1; then
		debug_external_command "dpkg-deb --extract \"$file\" \"$destination\""
		dpkg-deb --extract "$file" "$destination"
	elif command -v bsdtar >/dev/null 2>&1; then
		debug_external_command "bsdtar --extract --to-stdout --file \"$file\" 'data*' | bsdtar --directory \"$destination\" --extract --file /dev/stdin"
		bsdtar --extract --to-stdout --file "$file" 'data*' | \
			bsdtar --directory "$destination" --extract --file /dev/stdin
	elif command -v unar >/dev/null 2>&1; then
		mkdir --parents "$tmpdir"
		debug_external_command "unar -output-directory \"$tmpdir\" -force-overwrite -no-directory \"$file\" 'data*' 1>/dev/null"
		unar -output-directory "$tmpdir" -force-overwrite -no-directory "$file" 'data*' 1>/dev/null
		debug_external_command "unar -output-directory \"$destination\" -force-overwrite -no-directory \"$tmpdir\"/data* 1>/dev/null"
		unar -output-directory "$destination" -force-overwrite -no-directory "$tmpdir"/data* 1>/dev/null
		rm --recursive --force "$tmpdir"
	elif command -v tar >/dev/null 2>&1; then
		if command -v 7z >/dev/null 2>&1; then
			debug_external_command "7z x -i'!data*' -so \"$file\" | tar --directory \"$destination\" --extract"
			7z x -i'!data*' -so "$file" | \
				tar --directory "$destination" --extract
		elif command -v 7zr >/dev/null 2>&1; then
			debug_external_command "7zr x -so \"$file\" | tar --directory \"$destination\" --extract"
			7zr x -so "$file" | \
				tar --directory "$destination" --extract
		elif command -v ar >/dev/null 2>&1; then
			mkdir --parents "$tmpdir"
			archive_path="$(realpath --canonicalize-existing "$file")"
			(
				cd "$tmpdir"
				debug_external_command "ar x \"$archive_path\" \"$(ar t \"$archive_path\" | grep ^data)\""
				ar x "$archive_path" "$(ar t "$archive_path" | grep ^data)"
			)
			debug_external_command "tar --directory \"$destination\" --extract --file \"$tmpdir\"/data*"
			tar --directory "$destination" --extract --file "$tmpdir"/data*
			rm --recursive --force "$tmpdir"
		else
			error_archive_no_extractor_found 'debian'
		fi
	else
		error_archive_no_extractor_found 'debian'
	fi
}

# extract data from InnoSetup archive
# USAGE: archive_extraction_innosetup $archive_type $archive $destination
archive_extraction_innosetup() {
	local archive
	local archive_type
	local destination
	local options
	archive_type="$1"
	archive="$2"
	destination="$3"
	options='--progress=1 --silent'
	if [ -n "${archive_type%%*_nolowercase}" ]; then
		options="$options --lowercase"
	fi
	if ! archive_extraction_innosetup_is_supported "$archive"; then
		error_innoextract_version_too_old "$archive"
	fi
	debug_external_command "innoextract $options --extract --output-dir \"$destination\" \"$file\" 2>/dev/null"
	innoextract $options --extract --output-dir "$destination" "$file" 2>/dev/null
}

# check that the InnoSetup archive can be processed by the available innoextract version
# USAGE: archive_extraction_innosetup_is_supported $archive
# RETURNS: 0 if supported, 1 if unsupported
archive_extraction_innosetup_is_supported() {
	local archive
	archive="$1"

	# Use innoextract internal check
	if innoextract --list --silent "$archive" 2>&1 1>/dev/null | \
		head --lines=1 | \
		grep --ignore-case --quiet 'unexpected setup data version'
	then
		return 1
	fi

	# Check for GOG archives based on Galaxy file fragments, unsupported by innoextract < 1.7
	if innoextract --list "$archive" | \
		grep --quiet ' - "tmp/[0-9a-f]\{2\}/[0-9a-f]\{2\}/[0-9a-f]\{32\}" (.*)'
	then
		return 1
	fi

	return 0
}

# prepare package layout by putting files from archive in the right packages
# directories
# USAGE: prepare_package_layout [$pkg…]
# NEEDED VARS: (LANG) PLAYIT_WORKDIR
prepare_package_layout() {
	if [ -z "$1" ]; then
		# shellcheck disable=SC2046
		prepare_package_layout $(packages_get_list)
		return 0
	fi

	debug_entering_function 'prepare_package_layout'

	local package
	for package in "$@"; do
		PKG="$package"
		export PKG

		organize_data "GAME_${package#PKG_}" "$PATH_GAME"
		organize_data "DOC_${package#PKG_}"  "$PATH_DOC"
		for i in $(seq 0 9); do
			organize_data "GAME${i}_${package#PKG_}" "$PATH_GAME"
			organize_data "DOC${i}_${package#PKG_}"  "$PATH_DOC"
		done
	done

	debug_leaving_function 'prepare_package_layout'
}

# put files from archive in the right package directories
# USAGE: organize_data $id $path
organize_data() {
	local archive_path archive_files source_path destination_path source_files_pattern source_file destination_file

	# get the current package
	local package
	package=$(package_get_current)

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	# Check that the current package is part of the target architectures
	if [ "$OPTION_ARCHITECTURE" != 'all' ] && [ -n "${packages_list##*$package*}" ]; then
		warning_skip_package 'organize_data' "$package"
		return 0
	fi

	use_archive_specific_value "ARCHIVE_${1}_PATH"
	archive_path=$(get_value "ARCHIVE_${1}_PATH")
	use_archive_specific_value "ARCHIVE_${1}_FILES"
	archive_files=$(get_value "ARCHIVE_${1}_FILES")
	destination_path="$(package_get_path "$package")${2}"
	source_path="$PLAYIT_WORKDIR/gamedata/$archive_path"

	# When called in dry-run mode, return early
	if [ $DRY_RUN -eq 1 ]; then
		return 0
	fi

	if \
		[ -n "$archive_path" ] && \
		[ -n "$archive_files" ] && \
		[ -d "$source_path" ]
	then
		mkdir --parents "$destination_path"
		set -o noglob
		for source_files_pattern in $archive_files; do
			set +o noglob
			for source_file in "$source_path"/$source_files_pattern; do
				if [ -e "$source_file" ]; then
					debug_source_file 'Found' "$archive_path${source_file#$source_path}"
					destination_file="${destination_path}/${source_file#$source_path}"
					debug_file_to_package "$package"
					mkdir --parents "$(dirname "$destination_file")"
					cp \
						--recursive \
						--force \
						--link \
						--no-dereference \
						--no-target-directory \
						--preserve=links \
						"$source_file" "$destination_file"
					rm --force --recursive "$source_file"
				else
					debug_source_file 'Missing' "$archive_path${source_file#$source_path}"
				fi
			done
			set -o noglob
		done
		set +o noglob
	fi
}

# update dependencies list with commands needed for icons extraction
# USAGE: icons_list_dependencies
# RETURNS: nothing
# SIDE EFFECT: export $ICONS_DEPS, a variable including a space-separated list of required commands to handle the icons of the current game
icons_list_dependencies() {
	# Do nothing if the calling script explicitely asked for skipping icons extraction
	[ $SKIP_ICONS -eq 1 ] && return 0

	local script
	script="$0"
	if grep \
		--quiet \
		--regexp="^APP_[^_]\\+_ICON='.\\+'" \
		--regexp="^APP_[^_]\\+_ICON_.\\+='.\\+'" \
		"$script"
	then
		ICONS_DEPS="$ICONS_DEPS identify"
		if grep \
			--quiet \
			--regexp="^APP_[^_]\\+_ICON='.\\+\\.bmp'" \
			--regexp="^APP_[^_]\\+_ICON_.\\+='.\\+\\.bmp'" \
			--regexp="^APP_[^_]\\+_ICON='.\\+\\.ico'" \
			--regexp="^APP_[^_]\\+_ICON_.\\+='.\\+\\.ico'" \
			"$script"
		then
			ICONS_DEPS="$ICONS_DEPS convert"
		fi
		if grep \
			--quiet \
			--regexp="^APP_[^_]\\+_ICON='.\\+\\.exe'" \
			--regexp="^APP_[^_]\\+_ICON_.\\+='.\\+\\.exe'" \
			"$script"
		then
			ICONS_DEPS="$ICONS_DEPS convert wrestool"
		fi
	fi
	export ICONS_DEPS
}

# get .png file(s) from various icon sources in current package
# USAGE: icons_get_from_package $app[…]
# RETURNS: nothing
# SIDE EFFECT: get original icon files from the current package,
#              convert them to standard icon formats,
#              and include the standard icons in the current package
icons_get_from_package() {
	# Do nothing if the calling script explicitely asked for skipping icons extraction
	[ $SKIP_ICONS -eq 1 ] && return 0

	# get the current package
	local package
	package=$(package_get_current)

	local path
	path="$(package_get_path "$package")${PATH_GAME}"
	icons_get_from_path "$path" "$@"
}

# get .png file(s) from various icon sources in temporary work directory
# USAGE: icons_get_from_workdir $app[…]
# RETURNS: nothing
# SIDE EFFECT: get original icon files from ./play.it temporary directory,
#              convert them to standard icon formats,
#              and include the standard icons in the current package
icons_get_from_workdir() {
	# Do nothing if the calling script explicitely asked for skipping icons extraction
	[ $SKIP_ICONS -eq 1 ] && return 0

	local path
	path="$PLAYIT_WORKDIR/gamedata"
	icons_get_from_path "$path" "$@"
}

# get .png file(s) from various icon sources
# USAGE: icons_get_from_path $directory $app[…]
# RETURNS: nothing
# SIDE EFFECT: get original icon files from the given directory,
#              convert them to standard icon formats,
#              and include the standard icons in the current package
icons_get_from_path() {
	local app
	local destination
	local directory
	local file
	local icon
	local list
	local wrestool_id

	# get the current package
	local package
	package=$(package_get_current)

	directory="$1"
	shift 1
	destination="$PLAYIT_WORKDIR/icons"
	for app in "$@"; do
		if ! testvar "$app" 'APP'; then
			error_invalid_argument 'app' 'icons_get_from_package'
		fi
		list="$(get_value "${app}_ICONS_LIST")"
		[ -n "$list" ] || list="${app}_ICON"
		for icon in $list; do
			use_archive_specific_value "$icon"
			file="$(get_value "$icon")"
			if [ -z "$file" ]; then
				error_variable_not_set 'icons_get_from_path' '$'"$icon"
			fi

			# Check icon file existence
			file=$(icon_check_file_existence "$directory" "$file")

			wrestool_id="$(get_value "${icon}_ID")"
			icon_extract_png_from_file "$directory/$file" "$destination"
			icons_include_png_from_directory "$app" "$destination"
		done
	done
}

# check icon file existence
# USAGE: icon_check_file_existence $directory $file
# RETURNS: $file or throws an error
icon_check_file_existence() {
	# shellcheck disable=SC2039
	local directory file
	directory="$1"
	file="$2"

	# Return early in dry-run mode
	if [ $DRY_RUN -eq 1 ]; then
		printf '%s' "$file"
		return 0
	fi

	if [ ! -f "$directory/$file" ]; then
		# pre-2.8 scripts could use globbing in file path
		# shellcheck disable=SC2154
		if version_is_at_least '2.8' "$target_version"; then
			error_icon_file_not_found "$directory/$file"
			return 1
		else
			file=$(icon_check_file_existence_pre_2_8 "$directory" "$file")
		fi
	fi

	printf '%s' "$file"
	return 0
}

# extract .png file(s) from target file
# USAGE: icon_extract_png_from_file $file $destination
# RETURNS: nothing
# SIDE EFFECT: convert the given file to .png icons, the .png files are created in the given directory
icon_extract_png_from_file() {
	local destination
	local extension
	local file
	file="$1"
	destination="$2"
	extension="${file##*.}"
	mkdir --parents "$destination"
	case "$extension" in
		('bmp')
			icon_convert_bmp_to_png "$file" "$destination"
		;;
		('exe')
			icon_extract_png_from_exe "$file" "$destination"
		;;
		('ico')
			icon_extract_png_from_ico "$file" "$destination"
		;;
		('png')
			icon_copy_png "$file" "$destination"
		;;
		(*)
			error_invalid_argument 'extension' 'icon_extract_png_from_file'
		;;
	esac
}

# extract .png file(s) for .exe
# USAGE: icon_extract_png_from_exe $file $destination
# RETURNS: nothing
# SIDE EFFECT: extract .png icons from the given .exe file, the .png files are created in the given directory
icon_extract_png_from_exe() {
	[ "$DRY_RUN" -eq 1 ] && return 0
	local destination
	local file
	file="$1"
	destination="$2"
	icon_extract_ico_from_exe "$file" "$destination"
	for file in "$destination"/*.ico; do
		icon_extract_png_from_ico "$file" "$destination"
		rm "$file"
	done
}

# extract .ico file(s) from .exe
# USAGE: icon_extract_ico_from_exe $file $destination
# RETURNS: nothing
# SIDE EFFECT: extract .ico icons from the given .exe file, the .ico files are created in the given directory
icon_extract_ico_from_exe() {
	[ "$DRY_RUN" -eq 1 ] && return 0
	local destination
	local file
	local options
	file="$1"
	destination="$2"
	[ "$wrestool_id" ] && options="--name=$wrestool_id"
	debug_external_command "wrestool --extract --type=14 $options --output=\"$destination\" \"$file\" 2>/dev/null"
	wrestool --extract --type=14 $options --output="$destination" "$file" 2>/dev/null
}

# convert .bmp file to .png
# USAGE: icon_convert_bmp_to_png $file $destination
# RETURNS: nothing
# SIDE EFFECT: convert the given .bmp file to .png, the .png file is created in the given directory
icon_convert_bmp_to_png() { icon_convert_to_png "$@"; }

# extract .png file(s) from .ico
# USAGE: icon_extract_png_from_ico $file $destination
# RETURNS: nothing
# SIDE EFFECT: convert the given .ico file to .png, the .png files are created in the given directory
icon_extract_png_from_ico() { icon_convert_to_png "$@"; }

# convert multiple icon formats to .png
# USAGE: icon_convert_to_png $file $destination
# RETURNS: nothing
# SIDE EFFECT: convert the given image file to .png, the .png files are created in the given directory
icon_convert_to_png() {
	[ "$DRY_RUN" -eq 1 ] && return 0
	local destination
	local file
	local name
	file="$1"
	destination="$2"
	name="$(basename "$file")"
	debug_external_command "convert \"$file\" \"$destination/${name%.*}.png\""
	convert "$file" "$destination/${name%.*}.png"
}

# copy .png file to directory
# USAGE: icon_copy_png $file $destination
# RETURNS: nothing
# SIDE EFFECT: copy the given .png file to the given directory
icon_copy_png() {
	[ "$DRY_RUN" -eq 1 ] && return 0
	local destination
	local file
	file="$1"
	destination="$2"
	cp "$file" "$destination"
}

# get .png file(s) from target directory and put them in current package
# USAGE: icons_include_png_from_directory $app $directory
# RETURNS: nothing
# SIDE EFFECT: take the icons from the given directory, move them to standard paths into the current package
icons_include_png_from_directory() {
	[ "$DRY_RUN" -eq 1 ] && return 0
	local app
	local directory
	local file
	local path
	local path_icon
	local resolution

	# get the current package
	local package
	package=$(package_get_current)

	app="$1"
	directory="$2"
	name="$(get_value "${app}_ID")"
	[ -n "$name" ] || name="$GAME_ID"
	for file in "$directory"/*.png; do
		icon_get_resolution_from_file "$file"
		path_icon="$PATH_ICON_BASE/$resolution/apps"
		path="$(package_get_path "$package")${path_icon}"
		mkdir --parents "$path"
		mv "$file" "$path/$name.png"
	done
}
# compatibility alias
###
# TODO
# This function should be moved to the compatibility aliases source file
###
sort_icons() {
	local app
	local directory
	directory="$PLAYIT_WORKDIR/icons"
	for app in "$@"; do
		icons_include_png_from_directory "$app" "$directory"
	done
}

# get image resolution for target file, exported as $resolution
# USAGE: icon_get_resolution_from_file $file
# RETURNS: nothing
# SIDE EFFECT: export the resolution of the given file as $resolution, using the format "${width}x${height}"
icon_get_resolution_from_file() {
	local file
	file="$1"

	# `identify` should be available when this function is called.
	# Exits with an explicit error if it is missing
	if ! command -v 'identify' >/dev/null 2>&1; then
		error_unavailable_command 'icon_get_resolution_from_file' 'identify'
	fi

	if ! version_is_at_least '2.8' "$target_version" && [ -n "${file##* *}" ]; then
		field=2
		unset resolution
		while
			[ -z "$resolution" ] || \
			[ -n "$(printf '%s' "$resolution" | sed 's/[0-9]*x[0-9]*//')" ]
		do
			resolution="$(identify $file | sed "s;^$file ;;" | cut --delimiter=' ' --fields=$field)"
			resolution="${resolution%+0+0}"
			field=$((field + 1))
		done
	else
		resolution="$(identify "$file" | sed "s;^$file ;;" | cut --delimiter=' ' --fields=2)"
		resolution="${resolution%+0+0}"
	fi
	export resolution
}

# move icons to the target package
# USAGE: icons_move_to $pkg
# RETURNS: nothing
# SIDE EFFECT: move the icons from the current package to the given package
icons_move_to() {
	###
	# TODO
	# Check that $destination_package is set to a valid package
	# Check that $PATH_ICON_BASE is set to an absolute path
	###

	# Do nothing if the calling script explicitely asked for skipping icons extraction
	[ $SKIP_ICONS -eq 1 ] && return 0

	local source_package      source_path      source_directory
	local destination_package destination_path destination_directory
	source_package=$(package_get_current)
	source_directory="$(package_get_path "$source_package")${PATH_ICON_BASE}"
	destination_package="$1"
	destination_directory="$(package_get_path "$destination_package")${PATH_ICON_BASE}"

	# If called in dry-run mode, return early
	if [ $DRY_RUN -eq 1 ]; then
		return 0
	fi

	# a basic `mv` call here would fail if the destination is not empty
	mkdir --parents "$destination_directory"
	cp --link --recursive "$source_directory"/* "$destination_directory"
	rm --recursive "${source_directory:?}"/*
	rmdir --ignore-fail-on-non-empty --parents "$source_directory"
}

# write launcher script
# USAGE: launcher_write_script $app
launcher_write_script() {
	# check that this has been called with exactly one argument
	if [ "$#" -eq 0 ]; then
		error_missing_argument 'launcher_write_script'
	elif [ "$#" -gt 1 ]; then
		error_extra_arguments 'launcher_write_script'
	fi

	# get the current package
	local package
	package=$(package_get_current)

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	# skip any action if called for a package excluded for target architectures
	if [ "$OPTION_ARCHITECTURE" != 'all' ] && [ -n "${packages_list##*$package*}" ]; then
		warning_skip_package 'launcher_write_script' "$package"
		return 0
	fi

	# parse argument
	local application
	application="$1"
	if ! testvar "$application" 'APP'; then
		error_invalid_argument 'application' 'launcher_write_script'
	fi

	# get application type
	local application_type
	application_type="$(get_value "${application}_TYPE")"

	# compute file name and path
	local application_id
	local target_file
	application_id="$(get_value "${application}_ID")"
	if [ -z "$application_id" ]; then
		application_id="$GAME_ID"
	fi
	target_file="$(package_get_path "$package")${PATH_BIN}/$application_id"

	# Check that the launcher target exists
	local binary_file binary_path binary_found tested_package
	case "$application_type" in
		('residualvm'|'scummvm'|'renpy')
			# ResidualVM, ScummVM and Ren'Py games do not rely on a provided binary
		;;
		('mono')
			# Game binary for Mono games may be included in another package than the binaries one
			use_package_specific_value "${application}_EXE"
			binary_file=$(get_value "${application}_EXE")

			# Check that the name of the binary file is not empty
			if [ -z "$binary_file" ]; then
				error_empty_variable "${application}_EXE"
			fi

			binary_found=0
			for tested_package in $packages_list; do
				binary_path="$(package_get_path "$tested_package")${PATH_GAME}/$binary_file"
				if [ -f "$binary_path" ]; then
					binary_found=1
					break;
				fi
			done
			if \
				[ $DRY_RUN -eq 0 ] && \
				[ $binary_found -eq 0 ]
			then
				binary_path="$(package_get_path "$package")${PATH_GAME}/$binary_file"
				error_launcher_missing_binary "$binary_path"
			fi
		;;
		('wine')
			use_package_specific_value "${application}_EXE"
			binary_file=$(get_value "${application}_EXE")

			# Check that the name of the binary file is not empty
			if [ -z "$binary_file" ]; then
				error_empty_variable "${application}_EXE"
			fi

			if [ "$binary_file" != 'winecfg' ]; then
				binary_path="$(package_get_path "$package")${PATH_GAME}/$binary_file"
				if \
					[ $DRY_RUN -eq 0 ] && \
					[ ! -f "$binary_path" ]
				then
					error_launcher_missing_binary "$binary_path"
				fi
			fi
		;;
		(*)
			use_package_specific_value "${application}_EXE"
			binary_file=$(get_value "${application}_EXE")

			# Check that the name of the binary file is not empty
			if [ -z "$binary_file" ]; then
				error_empty_variable "${application}_EXE"
			fi

			binary_path="$(package_get_path "$package")${PATH_GAME}/$binary_file"
			if \
				[ $DRY_RUN -eq 0 ] && \
				[ ! -f "$binary_path" ]
			then
				error_launcher_missing_binary "$binary_path"
			fi
		;;
	esac

	# if called in dry run mode, return before writing anything
	if [ "$DRY_RUN" -eq 1 ]; then
		return 0
	fi

	# write launcher script
	debug_write_launcher "$application_type" "$binary_file"
	mkdir --parents "$(dirname "$target_file")"
	touch "$target_file"
	chmod 755 "$target_file"
	launcher_write_script_headers "$target_file"
	case "$application_type" in
		('dosbox')
			launcher_write_script_dosbox_application_variables "$application" "$target_file"
			launcher_write_script_game_variables "$target_file"
			launcher_write_script_user_files "$target_file"
			launcher_write_script_prefix_variables "$target_file"
			launcher_write_script_prefix_functions "$target_file"
			launcher_write_script_prefix_build "$target_file"
			launcher_write_script_dosbox_run "$application" "$target_file"
		;;
		('java')
			launcher_write_script_java_application_variables "$application" "$target_file"
			launcher_write_script_game_variables "$target_file"
			launcher_write_script_user_files "$target_file"
			launcher_write_script_prefix_variables "$target_file"
			launcher_write_script_prefix_functions "$target_file"
			launcher_write_script_prefix_build "$target_file"
			launcher_write_script_java_run "$application" "$target_file"
		;;
		('native')
			launcher_write_script_native_application_variables "$application" "$target_file"
			launcher_write_script_game_variables "$target_file"
			launcher_write_script_user_files "$target_file"
			launcher_write_script_prefix_variables "$target_file"
			launcher_write_script_prefix_functions "$target_file"
			launcher_write_script_prefix_build "$target_file"
			launcher_write_script_native_run "$application" "$target_file"
		;;
		('native_no-prefix')
			launcher_write_script_native_application_variables "$application" "$target_file"
			launcher_write_script_game_variables "$target_file"
			launcher_write_script_nativenoprefix_run "$application" "$target_file"
		;;
		('scummvm')
			launcher_write_script_scummvm_application_variables "$application" "$target_file"
			launcher_write_script_game_variables "$target_file"
			launcher_write_script_scummvm_run "$application" "$target_file"
		;;
		('renpy')
			launcher_write_script_game_variables "$target_file"
			launcher_write_script_user_files "$target_file"
			launcher_write_script_prefix_variables "$target_file"
			launcher_write_script_prefix_functions "$target_file"
			launcher_write_script_prefix_build "$target_file"
			launcher_write_script_renpy_run "$application" "$target_file"
		;;
		('residualvm')
			launcher_write_script_residualvm_application_variables "$application" "$target_file"
			launcher_write_script_game_variables "$target_file"
			launcher_write_script_residualvm_run "$application" "$target_file"
		;;
		('wine')
			if [ "$application_id" != "${GAME_ID}_winecfg" ]; then
				launcher_write_script_wine_application_variables "$application" "$target_file"
			fi
			launcher_write_script_game_variables "$target_file"
			launcher_write_script_user_files "$target_file"
			launcher_write_script_prefix_variables "$target_file"
			launcher_write_script_prefix_functions "$target_file"
			launcher_write_script_wine_prefix_build "$target_file"
			if [ "$application_id" = "${GAME_ID}_winecfg" ]; then
				launcher_write_script_winecfg_run "$target_file"
			else
				launcher_write_script_wine_run "$application" "$target_file"
			fi
		;;
		('mono')
			launcher_write_script_mono_application_variables "$application" "$target_file"
			launcher_write_script_game_variables "$target_file"
			launcher_write_script_user_files "$target_file"
			launcher_write_script_prefix_variables "$target_file"
			launcher_write_script_prefix_functions "$target_file"
			launcher_write_script_prefix_build "$target_file"
			launcher_write_script_mono_run "$application" "$target_file"
		;;
		(*)
			error_unknown_application_type "$application_type"
		;;
	esac
	cat >> "$target_file" <<- 'EOF'
	exit 0
	EOF

	# for native applications, add execution permissions to the game binary file
	case "$application_type" in
		('native'*)
			local application_exe
			use_package_specific_value "${application}_EXE"
			application_exe="$(get_value "${application}_EXE")"
			chmod +x "$(package_get_path "$package")${PATH_GAME}/$application_exe"
		;;
	esac

	# for WINE applications, write launcher script for winecfg
	case "$application_type" in
		('wine')
			local winecfg_file
			winecfg_file="$(package_get_path "$package")${PATH_BIN}/${GAME_ID}_winecfg"
			if [ ! -e "$winecfg_file" ]; then
				launcher_write_script_wine_winecfg "$application"
			fi
		;;
	esac

	return 0
}

# write launcher script headers
# USAGE: launcher_write_script_headers $file
# NEEDED VARS: LIBRARY_VERSION
# CALLED BY: launcher_write_script
launcher_write_script_headers() {
	local file
	file="$1"
	cat > "$file" <<- EOF
	#!/bin/sh
	# script generated by ./play.it $LIBRARY_VERSION - https://www.dotslashplay.it/
	set -o errexit

	EOF
	return 0
}

# write launcher script game-specific variables
# USAGE: launcher_write_script_game_variables $file
# NEEDED VARS: GAME_ID PATH_GAME
# CALLED BY: launcher_write_script
launcher_write_script_game_variables() {
	local file
	file="$1"
	cat >> "$file" <<- EOF
	# Set game-specific values

	GAME_ID='$GAME_ID'
	PATH_GAME='$PATH_GAME'

	EOF
	return 0
}

# write launcher script list of user-writable files
# USAGE: launcher_write_script_user_files $file
# NEEDED VARS: CONFIG_DIRS CONFIG_FILES DATA_DIRS DATA_FILES
# CALLED BY: launcher_write_script
launcher_write_script_user_files() {
	local file
	file="$1"
	cat >> "$file" <<- EOF
	# Set list of user-writable files

	CONFIG_DIRS='$CONFIG_DIRS'
	CONFIG_FILES='$CONFIG_FILES'
	DATA_DIRS='$DATA_DIRS'
	DATA_FILES='$DATA_FILES'

	EOF
	return 0
}

# write launcher script prefix-related variables
# USAGE: launcher_write_script_prefix_variables $file
# CALLED BY: launcher_write_script
launcher_write_script_prefix_variables() {
	local file
	file="$1"
	cat >> "$file" <<- 'EOF'
	# Set prefix-related values

	: "${PREFIX_ID:="$GAME_ID"}"
	PATH_CONFIG="${XDG_CONFIG_HOME:="$HOME/.config"}/$PREFIX_ID"
	PATH_DATA="${XDG_DATA_HOME:="$HOME/.local/share"}/games/$PREFIX_ID"

	EOF
	return 0
}

# write launcher script prefix functions
# USAGE: launcher_write_script_prefix_functions $file
# CALLED BY: launcher_write_script
launcher_write_script_prefix_functions() {
	local file
	file="$1"
	cat >> "$file" <<- 'EOF'
	# Set prefix-related functions

	init_prefix_dirs() {
	    # shellcheck disable=SC2039
	    local destination directories
	    destination="$1"
	    directories="$2"
	    (
	        cd "$PATH_GAME"
	        for directory in $directories; do
	            mkdir --parents "${destination}/${directory}"
	            mkdir --parents "$(dirname "${PATH_PREFIX}/${directory}")"
	            if \
	                [ -d "${PATH_PREFIX}/${directory}" ] && \
	                [ ! -h "${PATH_PREFIX}/${directory}" ]
	            then
	                # Migrate existing data from the prefix
	                (
	                    cd "$PATH_PREFIX"
	                    find "$directory" -type f | while read -r file; do
	                        cp --parents --remove-destination "$file" "$destination"
	                        rm "$file"
	                    done
	                    find "$directory" -type l | while read -r link; do
	                        cp --parents --dereference --no-clobber "$link" "$destination"
	                        rm "$link"
	                    done
	                )
	                rm --recursive "${PATH_PREFIX:?}/${directory:?}"
	            fi
	            ln --force --symbolic --no-target-directory "${destination}/${directory}" "${PATH_PREFIX}/${directory}"
	        done
	    )
	}

	init_prefix_files() {
	    (
	        local file_prefix
	        local file_real
	        cd "$1"
	        find -L . -type f | while read -r file; do
	            if [ -e "$PATH_PREFIX/$file" ]; then
	                file_prefix="$(readlink -e "$PATH_PREFIX/$file")"
	            else
	                unset file_prefix
	            fi
	            file_real="$(readlink -e "$file")"
	            if [ "$file_real" != "$file_prefix" ]; then
	                if [ "$file_prefix" ]; then
	                    rm --force "$PATH_PREFIX/$file"
	                fi
	                mkdir --parents "$PATH_PREFIX/$(dirname "$file")"
	                ln --symbolic "$file_real" "$PATH_PREFIX/$file"
	            fi
	        done
	    )
	    (
	        cd "$PATH_PREFIX"
	        for file in $2; do
	            if [ -e "$file" ] && [ ! -e "$1/$file" ]; then
	                cp --parents "$file" "$1"
	                rm --force "$file"
	                ln --symbolic "$1/$file" "$file"
	            fi
	        done
	    )
	}

	init_userdir_files() {
	    (
	        cd "$PATH_GAME"
	        for file in $2; do
	            if [ ! -e "$1/$file" ] && [ -e "$file" ]; then
	                cp --parents "$file" "$1"
	            fi
	        done
	    )
	}

	EOF
	sed --in-place 's/    /\t/g' "$file"
	return 0
}

# write launcher script prefix initialization
# USAGE: launcher_write_script_prefix_build $file
# CALLED BY: launcher_write_build
launcher_write_script_prefix_build() {
	local file
	file="$1"
	cat >> "$file" <<- 'EOF'
	# Build user prefix

	PATH_PREFIX="$XDG_DATA_HOME/play.it/prefixes/$PREFIX_ID"
	for dir in "$PATH_PREFIX" "$PATH_CONFIG" "$PATH_DATA"; do
	    if [ ! -e "$dir" ]; then
	        mkdir --parents "$dir"
	    fi
	done
	(
	    cd "$PATH_GAME"
	    find . -type d | while read -r dir; do
	        if [ -h "$PATH_PREFIX/$dir" ]; then
	            rm "$PATH_PREFIX/$dir"
	        fi
	    done
	)
	cp --recursive --remove-destination --symbolic-link "$PATH_GAME"/* "$PATH_PREFIX"
	(
	    cd "$PATH_PREFIX"
	    find . -type l | while read -r link; do
	        if [ ! -e "$link" ]; then
	            rm "$link"
	        fi
	    done
	    find . -depth -type d | while read -r dir; do
	        if [ ! -e "$PATH_GAME/$dir" ]; then
	            rmdir --ignore-fail-on-non-empty "$dir"
	        fi
	    done
	)

	# Use persistent storage for user data
	init_prefix_dirs   "$PATH_CONFIG" "$CONFIG_DIRS"
	init_prefix_dirs   "$PATH_DATA"   "$DATA_DIRS"
	init_userdir_files "$PATH_CONFIG" "$CONFIG_FILES"
	init_userdir_files "$PATH_DATA"   "$DATA_FILES"
	init_prefix_files  "$PATH_CONFIG" "$CONFIG_FILES"
	init_prefix_files  "$PATH_DATA"   "$DATA_FILES"

	EOF
	sed --in-place 's/    /\t/g' "$file"
	return 0
}

# write launcher script pre-run actions
# USAGE: launcher_write_script_prerun $application $file
# CALLED BY: launcher_write_script_dosbox_run launcher_write_script_native_run launcher_write_script_nativenoprefix_run launcher_write_script_scummvm_run launcher_write_script_residualvm_run launcher_write_script_wine_run
launcher_write_script_prerun() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	local application_prerun
	application_prerun="$(get_value "${application}_PRERUN")"
	if [ "$application_prerun" ]; then
		cat >> "$file" <<- EOF
		$application_prerun

		EOF
	fi

	return 0
}

# write launcher script post-run actions
# USAGE: launcher_write_script_postrun $application $file
# CALLED BY: launcher_write_script_dosbox_run launcher_write_script_native_run launcher_write_script_nativenoprefix_run launcher_write_script_scummvm_run launcher_write_script_residualvm_run launcher_write_script_wine_run
launcher_write_script_postrun() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	local application_postrun
	application_postrun="$(get_value "${application}_POSTRUN")"
	if [ "$application_postrun" ]; then
		cat >> "$file" <<- EOF
		$application_postrun

		EOF
	fi

	return 0
}

# write menu entry
# USAGE: launcher_write_desktop $app
# NEEDED VARS: OPTION_ARCHITECTURE GAME_ID GAME_NAME PATH_DESK PATH_BIN
# CALLS: error_missing_argument error_extra_arguments
launcher_write_desktop() {
	# check that this has been called with exactly one argument
	if [ "$#" -eq 0 ]; then
		error_missing_argument 'launcher_write_desktop'
	elif [ "$#" -gt 1 ]; then
		error_extra_arguments 'launcher_write_desktop'
	fi

	# get the current package
	local package
	package=$(package_get_current)

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	# skip any action if called for a package excluded for target architectures
	if [ "$OPTION_ARCHITECTURE" != 'all' ] && [ -n "${packages_list##*$package*}" ]; then
		warning_skip_package 'launcher_write_desktop' "$package"
		return 0
	fi

	# parse argument
	local application
	application="$1"
	if ! testvar "$application" 'APP'; then
		error_invalid_argument 'application' 'launcher_write_desktop'
	fi

	# get application-specific values
	local application_id
	local application_name
	local application_category
	local application_type
	if [ "$application" = 'APP_WINECFG' ]; then
		application_id="${GAME_ID}_winecfg"
		# shellcheck disable=SC2153
		application_name="$GAME_NAME - WINE configuration"
		application_category='Settings'
		application_type='wine'
		application_icon='winecfg'
	else
		application_id="$(get_value "${application}_ID")"
		application_name="$(get_value "${application}_NAME")"
		application_category="$(get_value "${application}_CAT")"
		application_type="$(get_value "${application}_TYPE")"
		: "${application_id:=$GAME_ID}"
		: "${application_name:=$GAME_NAME}"
		: "${application_category:=Game}"
		application_icon="$application_id"
	fi

	# compute file name and path
	local target_file
	target_file="$(package_get_path "$package")${PATH_DESK}/${application_id}.desktop"

	# include full binary path in Exec field if using non-standard installation prefix
	local exec_field
	case "$OPTION_PREFIX" in
		('/usr'|'/usr/local')
			exec_field="$application_id"
		;;
		(*' '*)
			# enclose the path in single quotes if it includes spaces
			exec_field="'$PATH_BIN/$application_id'"
		;;
		(*)
			exec_field="$PATH_BIN/$application_id"
		;;
	esac

	# if called in dry run mode, return before writing anything
	if [ "$DRY_RUN" -eq 1 ]; then
		return 0
	fi

	# write desktop file
	mkdir --parents "$(dirname "$target_file")"
	cat >> "$target_file" <<- EOF
	[Desktop Entry]
	Version=1.0
	Type=Application
	Name=$application_name
	Icon=$application_icon
	Exec=$exec_field
	Categories=$application_category
	EOF

	# for WINE applications, write desktop file for winecfg
	case "$application_type" in
		('wine')
			local winecfg_desktop
			winecfg_desktop="$(package_get_path "$package")${PATH_DESK}/${GAME_ID}_winecfg.desktop"
			if [ ! -e "$winecfg_desktop" ]; then
				launcher_write_desktop 'APP_WINECFG'
			fi
		;;
	esac

	return 0
}

# write both launcher script and menu entry for a single application
# USAGE: launcher_write $application
# NEEDED VARS: OPTION_ARCHITECTURE
# CALLS: launcher_write_script launcher_write_desktop
# CALLED BY: launchers_write
launcher_write() {
	# get the current package
	local package
	package=$(package_get_current)

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	# skip any action if called for a package excluded for target architectures
	if [ "$OPTION_ARCHITECTURE" != 'all' ] && [ -n "${packages_list##*$package*}" ]; then
		warning_skip_package 'launcher_write_script' "$package"
		return 0
	fi

	local application
	application="$1"
	launcher_write_script "$application"
	launcher_write_desktop "$application"
	return 0
}

# write both launcher script and menu entry for a list of applications
# USAGE: launchers_write $application[…]
# NEEDED VARS: OPTION_ARCHITECTURE
# CALLS: launcher_write
launchers_write() {

	debug_entering_function 'launchers_write' 2

	# get the current package
	local package
	package=$(package_get_current)

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	# skip any action if called for a package excluded for target architectures
	if [ "$OPTION_ARCHITECTURE" != 'all' ] && [ -n "${packages_list##*$package*}" ]; then
		warning_skip_package 'launcher_write_script' "$package"
		return 0
	fi

	local application
	for application in "$@"; do
		launcher_write "$application"
	done

	debug_leaving_function 'launchers_write' 2

	return 0
}

# DOSBox - write application-specific variables
# USAGE: launcher_write_script_dosbox_application_variables $application $file
# CALLED BY: launcher_write_script
launcher_write_script_dosbox_application_variables() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	# compute application-specific variables values
	local application_exe
	local application_options
	use_package_specific_value "${application}_EXE"
	use_package_specific_value "${application}_OPTIONS"
	application_exe="$(get_value "${application}_EXE")"
	application_options="$(get_value "${application}_OPTIONS")"

	cat >> "$file" <<- EOF
	# Set application-specific values

	APP_EXE='$application_exe'
	APP_OPTIONS="$application_options"

	EOF
	return 0
}

# DOSBox - run the game
# USAGE: launcher_write_script_dosbox_run $application $file
# NEEDED_VARS: GAME_IMAGE GAME_IMAGE_TYPE PATH_GAME
# CALLS: launcher_write_script_prerun launcher_write_script_postrun
# CALLED BY: launcher_write_script
launcher_write_script_dosbox_run() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	cat >> "$file" <<- 'EOF'
	# Run the game

	cd "$PATH_PREFIX"
	"${PLAYIT_DOSBOX_BINARY:-dosbox}" -c "mount c .
	c:
	EOF

	# mount CD image file
	if [ "$GAME_IMAGE" ]; then
		case "$GAME_IMAGE_TYPE" in
			('cdrom')
				local image
				local package

				# Get packages list for the current game
				local packages_list
				packages_list=$(packages_get_list)

				for package in $packages_list; do
					if [ -e "$(package_get_path "$package")$PATH_GAME/$GAME_IMAGE" ]; then
						image="$(package_get_path "$package")$PATH_GAME/$GAME_IMAGE"
						break;
					fi
				done
				if [ -d "$image" ]; then
					cat >> "$file" <<- EOF
					mount d $GAME_IMAGE -t cdrom
					EOF
				else
					cat >> "$file" <<- EOF
					imgmount d $GAME_IMAGE -t cdrom
					EOF
				fi
			;;
			('iso'|*)
				cat >> "$file" <<- EOF
				imgmount d $GAME_IMAGE -t iso -fs iso
				EOF
			;;
		esac
	fi

	launcher_write_script_prerun "$application" "$file"

	cat >> "$file" <<- 'EOF'
	$APP_EXE $APP_OPTIONS $@
	EOF

	launcher_write_script_postrun "$application" "$file"

	cat >> "$file" <<- 'EOF'
	exit"

	EOF
	return 0
}

# Java - write application-specific variables
# USAGE: launcher_write_script_java_application_variables $application $file
# CALLED BY: launcher_write_script
launcher_write_script_java_application_variables() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	# compute application-specific variables values
	local application_exe
	local application_java_options
	local application_libs
	local application_options
	use_package_specific_value "${application}_EXE"
	use_package_specific_value "${application}_JAVA_OPTIONS"
	use_package_specific_value "${application}_LIBS"
	use_package_specific_value "${application}_OPTIONS"
	application_exe="$(get_value "${application}_EXE")"
	application_java_options="$(get_value "${application}_JAVA_OPTIONS")"
	application_libs="$(get_value "${application}_LIBS")"
	application_options="$(get_value "${application}_OPTIONS")"

	cat >> "$file" <<- EOF
	# Set application-specific values

	APP_EXE='$application_exe'
	APP_LIBS='$application_libs'
	APP_OPTIONS="$application_options"
	JAVA_OPTIONS='$application_java_options'

	EOF
	return 0
}

# Java - run the game
# USAGE: launcher_write_script_java_run $application $file
# CALLED BY: launcher_write_script
launcher_write_script_java_run() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	cat >> "$file" <<- 'EOF'
	# Run the game

	cd "$PATH_PREFIX"

	EOF

	launcher_write_script_prerun "$application" "$file"

	cat >> "$file" <<- 'EOF'
	library_path=
	if [ -n "$APP_LIBS" ]; then
	    library_path="$APP_LIBS:"
	fi
	EOF
	local extra_library_path
	extra_library_path="$(launcher_native_get_extra_library_path)"
	if [ -n "$extra_library_path" ]; then
		cat >> "$file" <<- EOF
		library_path="\${library_path}$extra_library_path"
		EOF
	fi
	cat >> "$file" <<- 'EOF'
	if [ -n "$library_path" ]; then
	    LD_LIBRARY_PATH="${library_path}$LD_LIBRARY_PATH"
	    export LD_LIBRARY_PATH
	fi
	JAVA_OPTIONS="$(eval printf -- '%b' \"$JAVA_OPTIONS\")"
	java $JAVA_OPTIONS -jar "$APP_EXE" $APP_OPTIONS "$@"

	EOF

	launcher_write_script_postrun "$application" "$file"

	sed --in-place 's/    /\t/g' "$file"
	return 0
}

# Mono - write application-specific variables
# USAGE: launcher_write_script_mono_application_variables $application $file
# CALLED BY: launcher_write_script
launcher_write_script_mono_application_variables() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	# compute application-specific variables values
	local application_exe
	local application_mono_options
	local application_libs
	local application_options
	use_package_specific_value "${application}_EXE"
	use_package_specific_value "${application}_MONO_OPTIONS"
	use_package_specific_value "${application}_LIBS"
	use_package_specific_value "${application}_OPTIONS"
	application_exe="$(get_value "${application}_EXE")"
	application_mono_options="$(get_value "${application}_MONO_OPTIONS")"
	application_libs="$(get_value "${application}_LIBS")"
	application_options="$(get_value "${application}_OPTIONS")"

	cat >> "$file" <<- EOF
	# Set application-specific values

	APP_EXE='$application_exe'
	APP_LIBS='$application_libs'
	APP_OPTIONS="$application_options"
	MONO_OPTIONS='$application_mono_options'

	EOF
	return 0
}

# Mono - run the game
# USAGE: launcher_write_script_mono_run $application $file
# CALLED BY: launcher_write_script
launcher_write_script_mono_run() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	cat >> "$file" <<- 'EOF'
	# Run the game

	cd "$PATH_PREFIX"

	EOF

	launcher_write_script_prerun "$application" "$file"

	cat >> "$file" <<- 'EOF'
	library_path=
	if [ -n "$APP_LIBS" ]; then
	    library_path="$APP_LIBS:"
	fi
	EOF
	local extra_library_path
	extra_library_path="$(launcher_native_get_extra_library_path)"
	if [ -n "$extra_library_path" ]; then
		cat >> "$file" <<- EOF
		library_path="\${library_path}$extra_library_path"
		EOF
	fi
	cat >> "$file" <<- 'EOF'
	# Work around terminfo Mono bug, cf. https://github.com/mono/mono/issues/6752
	export TERM="${TERM%-256color}"

	# Work around Mono unpredictable behaviour with non-US locales
	export LANG=C

	if [ -n "$library_path" ]; then
	    LD_LIBRARY_PATH="${library_path}$LD_LIBRARY_PATH"
	    export LD_LIBRARY_PATH
	fi
	MONO_OPTIONS="$(eval printf -- '%b' \"$MONO_OPTIONS\")"
	mono $MONO_OPTIONS "$APP_EXE" $APP_OPTIONS "$@"

	EOF

	launcher_write_script_postrun "$application" "$file"

	sed --in-place 's/    /\t/g' "$file"
	return 0
}

# native - write application-specific variables
# USAGE: launcher_write_script_native_application_variables $application $file
# CALLED BY: launcher_write_script
launcher_write_script_native_application_variables() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	# compute application-specific variables values
	local application_exe
	local application_libs
	local application_options
	use_package_specific_value "${application}_EXE"
	use_package_specific_value "${application}_LIBS"
	use_package_specific_value "${application}_OPTIONS"
	application_exe="$(get_value "${application}_EXE")"
	application_libs="$(get_value "${application}_LIBS")"
	application_options="$(get_value "${application}_OPTIONS")"

	cat >> "$file" <<- EOF
	# Set application-specific values

	APP_EXE='$application_exe'
	APP_LIBS='$application_libs'
	APP_OPTIONS="$application_options"

	EOF
	return 0
}

# native - run the game (with prefix)
# USAGE: launcher_write_script_native_run $application $file
# CALLED BY: launcher_write_script
launcher_write_script_native_run() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	cat >> "$file" <<- 'EOF'
	# Copy the game binary into the user prefix

	if [ -e "$PATH_DATA/$APP_EXE" ]; then
	    source_dir="$PATH_DATA"
	else
	    source_dir="$PATH_GAME"
	fi

	(
	    cd "$source_dir"
	    cp --parents --dereference --remove-destination "$APP_EXE" "$PATH_PREFIX"
	)

	# Run the game

	cd "$PATH_PREFIX"

	EOF
	sed --in-place 's/    /\t/g' "$file"

	launcher_write_script_native_run_common "$application" "$file"

	return 0
}

# native - run the game (without prefix)
# USAGE: launcher_write_script_nativenoprefix_run $application $file
# CALLED BY: launcher_write_script
launcher_write_script_nativenoprefix_run() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	cat >> "$file" <<- 'EOF'
	# Run the game

	cd "$PATH_GAME"

	EOF

	launcher_write_script_native_run_common "$application" "$file"

	return 0
}

# native - get extra LD_LIBRARY_PATH entries (with a trailing :)
# USAGE: launcher_native_get_extra_library_path
# NEEDED VARS: OPTION_PACKAGE
# CALLED BY: launcher_write_script_native_run_common
launcher_native_get_extra_library_path() {
	# get the current package
	local package
	package=$(package_get_current)

	if [ "$OPTION_PACKAGE" = 'gentoo' ] && get_value "${package}_DEPS" | sed --regexp-extended 's/\s+/\n/g' | grep --fixed-strings --line-regexp --quiet 'libcurl-gnutls'; then
		# Get package architecture
		# shellcheck disable=SC2039
		local pkg_architecture
		pkg_architecture=$(package_get_architecture_string "$PKG")

		printf '%s' "/usr/\$(portageq envvar 'LIBDIR_$pkg_architecture')/debiancompat:"
	fi
}

# native - run the game (common part)
# USAGE: launcher_write_script_native_run_common $application $file
# CALLS: launcher_write_script_prerun launcher_write_script_postrun launcher_native_get_extra_library_path
# CALLED BY: launcher_write_script_native_run launcher_write_script_nativenoprefix_run
launcher_write_script_native_run_common() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	launcher_write_script_prerun "$application" "$file"

	cat >> "$file" <<- 'EOF'
	library_path=
	if [ -n "$APP_LIBS" ]; then
	    library_path="$APP_LIBS:"
	fi
	EOF
	local extra_library_path
	extra_library_path="$(launcher_native_get_extra_library_path)"
	if [ -n "$extra_library_path" ]; then
		cat >> "$file" <<- EOF
		library_path="\${library_path}$extra_library_path"
		EOF
	fi
	cat >> "$file" <<- 'EOF'
	if [ -n "$library_path" ]; then
	    LD_LIBRARY_PATH="${library_path}$LD_LIBRARY_PATH"
	    export LD_LIBRARY_PATH
	fi
	"./$APP_EXE" $APP_OPTIONS $@

	EOF

	launcher_write_script_postrun "$application" "$file"

	sed --in-place 's/    /\t/g' "$file"
	return 0
}
# Ren'Py - run the game
# USAGE: launcher_write_script_renpy_run $application $file
# CALLED BY: launcher_write_script
launcher_write_script_renpy_run() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	cat >> "$file" <<- 'EOF'
	# Run the game

	cd "$PATH_PREFIX"

	EOF

	launcher_write_script_prerun "$application" "$file"

	cat >> "$file" <<- 'EOF'
	renpy . "$@"

	EOF

	launcher_write_script_postrun "$application" "$file"

	sed --in-place 's/    /\t/g' "$file"
	return 0
}

# ResidualVM - write application-specific variables
# USAGE: launcher_write_script_residualvm_application_variables $application $file
# CALLED BY: launcher_write_script
launcher_write_script_residualvm_application_variables() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	# compute application-specific variables values
	local application_residualid
	use_package_specific_value "${application}_RESIDUALID"
	application_residualid="$(get_value "${application}_RESIDUALID")"

	cat >> "$file" <<- EOF
	# Set application-specific values

	RESIDUALVM_ID='$application_residualid'

	EOF
	return 0
}

# ResidualVM - run the game
# USAGE: launcher_write_script_residualvm_run $application $file
# CALLS: launcher_write_script_prerun launcher_write_script_postrun
# CALLED BY: launcher_write_script
launcher_write_script_residualvm_run() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	cat >> "$file" <<- 'EOF'
	# Run the game

	EOF

	launcher_write_script_prerun "$application" "$file"

	cat >> "$file" <<- 'EOF'
	residualvm -p "$PATH_GAME" $APP_OPTIONS $@ $RESIDUALVM_ID

	EOF

	launcher_write_script_postrun "$application" "$file"

	return 0
}

# ScummVM - write application-specific variables
# USAGE: launcher_write_script_scummvm_application_variables $application $file
# CALLED BY: launcher_write_script
launcher_write_script_scummvm_application_variables() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	# compute application-specific variables values
	local application_scummid
	use_package_specific_value "${application}_SCUMMID"
	application_scummid="$(get_value "${application}_SCUMMID")"

	cat >> "$file" <<- EOF
	# Set application-specific values

	SCUMMVM_ID='$application_scummid'

	EOF
	return 0
}

# ScummVM - run the game
# USAGE: launcher_write_script_scummvm_run $application $file
# CALLS: launcher_write_script_prerun launcher_write_script_postrun
# CALLED BY: launcher_write_script
launcher_write_script_scummvm_run() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	cat >> "$file" <<- 'EOF'
	# Run the game

	EOF

	launcher_write_script_prerun "$application" "$file"

	cat >> "$file" <<- 'EOF'
	scummvm -p "$PATH_GAME" $APP_OPTIONS $@ $SCUMMVM_ID

	EOF

	launcher_write_script_postrun "$application" "$file"

	return 0
}

# WINE - write application-specific variables
# USAGE: launcher_write_script_wine_application_variables $application $file
# CALLED BY: launcher_write_script
launcher_write_script_wine_application_variables() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	# compute application-specific variables values
	local application_exe
	local application_options
	use_package_specific_value "${application}_EXE"
	use_package_specific_value "${application}_OPTIONS"
	application_exe="$(get_value "${application}_EXE")"
	application_options="$(get_value "${application}_OPTIONS")"

	cat >> "$file" <<- EOF
	# Set application-specific values

	APP_EXE='$application_exe'
	APP_OPTIONS="$application_options"
	APP_WINE_LINK_DIRS="$APP_WINE_LINK_DIRS"

	EOF
	return 0
}

# WINE - write launcher script prefix initialization
# USAGE: launcher_write_script_wine_prefix_build $file
# NEEDED VARS: APP_WINETRICKS APP_REGEDIT
# CALLED BY: launcher_write_build
launcher_write_script_wine_prefix_build() {
	local file
	file="$1"

	# get the current package
	local package
	package=$(package_get_current)

	# compute WINE prefix architecture
	local architecture
	local winearch
	use_archive_specific_value "${package}_ARCH"
	architecture="$(get_value "${package}_ARCH")"
	case "$architecture" in
		('32') winearch='win32' ;;
		('64') winearch='win64' ;;
	esac

	cat >> "$file" <<- EOF
	# Build user prefix

	WINEARCH='$winearch'
	EOF

	cat >> "$file" <<- 'EOF'
	: "${WINEDEBUG:=-all}"
	WINEDLLOVERRIDES='winemenubuilder.exe,mscoree,mshtml=d'
	WINEPREFIX="$XDG_DATA_HOME/play.it/prefixes/$PREFIX_ID"
	# Work around WINE bug 41639
	FREETYPE_PROPERTIES="truetype:interpreter-version=35"

	PATH_PREFIX="$WINEPREFIX/drive_c/$GAME_ID"

	export WINEARCH WINEDEBUG WINEDLLOVERRIDES WINEPREFIX FREETYPE_PROPERTIES

	if ! [ -e "$WINEPREFIX" ]; then
	    mkdir --parents "$(dirname "$WINEPREFIX")"
	    # Use LANG=C to avoid localized directory names
	    LANG=C wineboot --init 2>/dev/null
	EOF

	if version_is_at_least '2.8' "$target_version"; then
		cat >> "$file" <<- 'EOF'
		    # Remove most links pointing outside of the WINE prefix
		    rm "$WINEPREFIX/dosdevices/z:"
		    find "$WINEPREFIX/drive_c/users/$(whoami)" -type l | while read -r directory; do
		        rm "$directory"
		        mkdir "$directory"
		    done
		EOF
	fi

	if [ "$APP_WINETRICKS" ]; then
		cat >> "$file" <<- EOF
		    if [ -t 0 ] || command -v zenity kdialog >/dev/null; then
		        winetricks $APP_WINETRICKS
		    elif command -v xterm >/dev/null; then
		        xterm -e winetricks $APP_WINETRICKS
		    else
		        winetricks $APP_WINETRICKS
		    fi
		    sleep 1s
		EOF
	fi

	if [ "$APP_REGEDIT" ]; then
		cat >> "$file" <<- EOF
		    for reg_file in $APP_REGEDIT; do
		EOF
		cat >> "$file" <<- 'EOF'
		    (
		        cd "$WINEPREFIX/drive_c/"
		        cp "$PATH_GAME/$reg_file" .
		        reg_file_basename="$(basename "$reg_file")"
		        wine regedit "$reg_file_basename"
		        rm "$reg_file_basename"
		    )
		    done
		EOF
	fi

	cat >> "$file" <<- 'EOF'
	fi
	EOF

	cat >> "$file" <<- 'EOF'
	for dir in "$PATH_PREFIX" "$PATH_CONFIG" "$PATH_DATA"; do
	    if [ ! -e "$dir" ]; then
	        mkdir --parents "$dir"
	    fi
	done
	(
	    cd "$PATH_GAME"
	    find . -type d | while read -r dir; do
	        if [ -h "$PATH_PREFIX/$dir" ]; then
	            rm "$PATH_PREFIX/$dir"
	        fi
	    done
	)
	cp --recursive --remove-destination --symbolic-link "$PATH_GAME"/* "$PATH_PREFIX"
	(
	    cd "$PATH_PREFIX"
	    find . -type l | while read -r link; do
	        if [ ! -e "$link" ]; then
	            rm "$link"
	        fi
	    done
	    find . -depth -type d | while read -r dir; do
	        if [ ! -e "$PATH_GAME/$dir" ]; then
	            rmdir --ignore-fail-on-non-empty "$dir"
	        fi
	    done
	)

	# Move files that should be diverted to persistent paths to the game directory
	printf '%s' "$APP_WINE_LINK_DIRS" | grep ':' | while read -r line; do
	    prefix_dir="$PATH_PREFIX/${line%%:*}"
	    wine_dir="$WINEPREFIX/drive_c/${line#*:}"
	    if [ ! -h "$wine_dir" ]; then
	        mkdir --parents "$prefix_dir"
	        if [ -d "$wine_dir" ]; then
	            # Migrate existing user data to the persistent path
	            find "$prefix_dir" -type l -delete
	            cp --no-target-directory --recursive --remove-destination "$wine_dir" "$prefix_dir"
	            rm --recursive "$wine_dir"
	        fi
	        mkdir --parents "$(dirname "$wine_dir")"
	        ln --symbolic "$prefix_dir" "$wine_dir"
	    fi
	done

	# Use persistent storage for user data
	init_prefix_dirs   "$PATH_CONFIG" "$CONFIG_DIRS"
	init_prefix_dirs   "$PATH_DATA"   "$DATA_DIRS"
	init_userdir_files "$PATH_CONFIG" "$CONFIG_FILES"
	init_userdir_files "$PATH_DATA"   "$DATA_FILES"
	init_prefix_files  "$PATH_CONFIG" "$CONFIG_FILES"
	init_prefix_files  "$PATH_DATA"   "$DATA_FILES"

	EOF
	sed --in-place 's/    /\t/g' "$file"
	return 0
}

# WINE - write launcher script for winecfg
# USAGE: launcher_write_script_wine_winecfg $application
# NEEDED VARS: GAME_ID
# CALLED BY: launcher_write_script_wine_winecfg
launcher_write_script_wine_winecfg() {
	local application
	application="$1"
	# shellcheck disable=SC2034
	APP_WINECFG_ID="${GAME_ID}_winecfg"
	# shellcheck disable=SC2034
	APP_WINECFG_TYPE='wine'
	# shellcheck disable=SC2034
	APP_WINECFG_EXE='winecfg'
	launcher_write_script 'APP_WINECFG'
	return 0
}

# WINE - run the game
# USAGE: launcher_write_script_wine_run $application $file
# CALLS: launcher_write_script_prerun launcher_write_script_postrun
# CALLED BY: launcher_write_script
launcher_write_script_wine_run() {
	# parse arguments
	local application
	local file
	application="$1"
	file="$2"

	cat >> "$file" <<- 'EOF'
	# Run the game

	cd "$PATH_PREFIX"

	EOF

	launcher_write_script_prerun "$application" "$file"

	cat >> "$file" <<- 'EOF'
	wine "$APP_EXE" $APP_OPTIONS $@

	EOF

	launcher_write_script_postrun "$application" "$file"

	return 0
}

# WINE - run winecfg
# USAGE: launcher_write_script_winecfg_run $file
# CALLED BY: launcher_write_script
launcher_write_script_winecfg_run() {
	# parse arguments
	local file
	file="$1"

	cat >> "$file" <<- 'EOF'
	# Run WINE configuration

	winecfg

	EOF

	return 0
}

# write package meta-data
# USAGE: write_metadata [$pkg…]
# NEEDED VARS: (ARCHIVE) GAME_NAME (OPTION_PACKAGE) (PKG_ARCH) PKG_DEPS_ARCH PKG_DEPS_DEB PKG_DESCRIPTION PKG_ID PKG_PROVIDE
# CALLS: pkg_write_arch pkg_write_deb pkg_write_gentoo testvar
write_metadata() {
	if [ $# -eq 0 ]; then
		# shellcheck disable=SC2046
		write_metadata $(packages_get_list)
		return 0
	fi

	debug_entering_function 'write_metadata'

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	for pkg in "$@"; do
		if ! testvar "$pkg" 'PKG'; then
			error_invalid_argument 'pkg' 'write_metadata'
		fi
		if [ "$OPTION_ARCHITECTURE" != all ] && [ -n "${packages_list##*$pkg*}" ]; then
			warning_skip_package 'write_metadata' "$pkg"
			continue
		fi

		[ "$DRY_RUN" -eq 1 ] && continue

		case $OPTION_PACKAGE in
			('arch')
				pkg_write_arch
			;;
			('deb')
				pkg_write_deb
			;;
			('gentoo')
				pkg_write_gentoo
			;;
			('egentoo')
				pkg_write_egentoo $pkg
			;;
			(*)
				error_invalid_argument 'OPTION_PACKAGE' 'write_metadata'
			;;
		esac
	done
	rm  --force "$postinst" "$prerm"

	debug_leaving_function 'write_metadata'
}

# build .pkg.tar or .deb package
# USAGE: build_pkg [$pkg…]
# NEEDED VARS: (OPTION_COMPRESSION) (LANG) (OPTION_PACKAGE) (PKG_PATH) PLAYIT_WORKDIR
# CALLS: pkg_build_arch pkg_build_deb pkg_build_gentoo testvar
build_pkg() {
	if [ $# -eq 0 ]; then
		# shellcheck disable=SC2046
		build_pkg $(packages_get_list)
		return 0
	fi

	debug_entering_function 'build_pkg'

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	for pkg in "$@"; do
		if ! testvar "$pkg" 'PKG'; then
			error_invalid_argument 'pkg' 'build_pkg'
		fi
		if [ "$OPTION_ARCHITECTURE" != all ] && [ -n "${packages_list##*$pkg*}" ]; then
			warning_skip_package 'build_pkg' "$pkg"
			return 0
		fi
		case $OPTION_PACKAGE in
			('arch')
				pkg_build_arch "$(package_get_path "$pkg")"
			;;
			('deb')
				pkg_build_deb "$(package_get_path "$pkg")"
			;;
			('gentoo')
				pkg_build_gentoo "$(package_get_path "$pkg")"
			;;
			('egentoo')
				pkg_build_egentoo "$pkg"
			;;
			(*)
				error_invalid_argument 'OPTION_PACKAGE' 'build_pkg'
			;;
		esac
	done

	debug_leaving_function 'build_pkg'
}

# guess package format to build from host OS
# USAGE: packages_guess_format $variable_name
# NEEDED VARS: (LANG) DEFAULT_OPTION_PACKAGE
packages_guess_format() {
	local guessed_host_os
	local variable_name
	eval variable_name=\"$1\"
	if [ -e '/etc/os-release' ]; then
		guessed_host_os="$(grep '^ID=' '/etc/os-release' | cut --delimiter='=' --fields=2)"
	elif command -v lsb_release >/dev/null 2>&1; then
		guessed_host_os="$(lsb_release --id --short | tr '[:upper:]' '[:lower:]')"
	fi
	case "$guessed_host_os" in
		('debian'|\
		 'ubuntu'|\
		 'linuxmint'|\
		 'handylinux')
			eval $variable_name=\'deb\'
		;;
		('arch'|\
		 'manjaro'|'manjarolinux'|\
		 'endeavouros')
			eval $variable_name=\'arch\'
		;;
		('gentoo')
			eval $variable_name=\'gentoo\'
		;;
		(*)
			warning_package_format_guessing_failed "$DEFAULT_OPTION_PACKAGE"
			eval $variable_name=\'$DEFAULT_OPTION_PACKAGE\'
		;;
	esac
	export ${variable_name?}
}

# get the current package
# USAGE: package_get_current
package_get_current() {
	local package
	package="$PKG"

	# Fall back on a default value if $PKG is not set
	if [ -z "$package" ]; then
		package='PKG_MAIN'
	fi

	printf '%s' "$package"

	return 0
}

# get the full list of packages to generate
# USAGE: packages_get_list
packages_get_list() {
	local packages_list
	packages_list="$PACKAGES_LIST"

	# Fall back on a default list if $PACKAGES_LIST is not set
	if [ -z "$packages_list" ]; then
		packages_list='PKG_MAIN'
	fi

	printf '%s' "$packages_list"

	return 0
}

# get ID of given package
# USAGE: package_get_id $package
# RETURNS: package ID, as a non-empty string
package_get_id() {
	# single argument should be the package name
	# shellcheck disable=SC2039
	local package
	package="$1"
	if [ -z "$package" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_id' '$package'
		return 1
	fi

	# get package ID from its name
	# shellcheck disable=SC2039
	local package_id
	use_archive_specific_value "${package}_ID"
	package_id=$(get_value "${package}_ID")

	# if no package-specific ID is set, fall back to game ID
	if [ -z "$package_id" ]; then
		package_id="$GAME_ID"
	fi

	# on Arch Linux, prepend "lib32-" to the ID of 32-bit packages
	case "$OPTION_PACKAGE" in
		('arch')
			case "$(package_get_architecture "$package")" in
				('32')
					package_id="lib32-${package_id}"
				;;
			esac
		;;
	esac

	# on Gentoo, avoid mixups between numbers in package ID and version number
	case "$OPTION_PACKAGE" in
		('gentoo'|'egentoo')
			package_id=$(printf '%s' "$package_id" | sed 's/-/_/g')
		;;
	esac

	printf '%s' "$package_id"
	return 0
}

# get architecture of given package
# USAGE: package_get_architecture $package
# RETURNS: package architecture, as a non-empty string
package_get_architecture() {
	# single argument should be the package name
	# shellcheck disable=SC2039
	local package
	package="$1"
	if [ -z "$package" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_architecture' '$package'
		return 1
	fi

	# get package architecture from its name
	# shellcheck disable=SC2039
	local package_architecture
	use_archive_specific_value "${package}_ARCH"
	package_architecture=$(get_value "${package}_ARCH")

	# if no package-specific architecture is set, fall back to "all"
	if [ -z "$package_architecture" ]; then
		package_architecture='all'
	fi

	printf '%s' "$package_architecture"
	return 0
}

# get architecture string for given package
# USAGE: package_get_architecture_string $package
# RETURNS: package architecture, as a non-empty string, ready to include in package meta-data
package_get_architecture_string() {
	# single argument should be the package name
	# shellcheck disable=SC2039
	local package
	package="$1"
	if [ -z "$package" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_architecture_string' '$package'
		return 1
	fi

	# get package architecture
	# shellcheck disable=SC2039
	local package_architecture
	package_architecture=$(package_get_architecture "$package")

	# set package architecture string, based on its architecture and target package format
	# shellcheck disable=SC2039
	local package_architecture_string
	case "$OPTION_PACKAGE" in
		('arch')
			case "$package_architecture" in
				('32'|'64')
					package_architecture_string='x86_64'
				;;
				(*)
					package_architecture_string='any'
				;;
			esac
		;;
		('deb')
			case "$package_architecture" in
				('32')
					package_architecture_string='i386'
				;;
				('64')
					package_architecture_string='amd64'
				;;
				(*)
					package_architecture_string='all'
				;;
			esac
		;;
		('gentoo'|'egentoo')
			case "$package_architecture" in
				('32')
					package_architecture_string='x86'
				;;
				('64')
					package_architecture_string='amd64'
				;;
				(*)
					package_architecture_string='data' # We could put anything here, it shouldn't be used for package metadata
				;;
			esac
		;;
		(*)
			error_invalid_argument 'OPTION_PACKAGE' 'package_get_architecture_string'
		;;
	esac

	printf '%s' "$package_architecture_string"
	return 0
}

# get description for given package
# USAGE: package_get_description $package
# RETURNS: package description, as a non-empty string
package_get_description() {
	# single argument should be the package name
	# shellcheck disable=SC2039
	local package
	package="$1"
	if [ -z "$package" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_description' '$package'
		return 1
	fi

	# get package description from its name
	# shellcheck disable=SC2039
	local package_description
	use_archive_specific_value "${package}_DESCRIPTION"
	package_description=$(get_value "${package}_DESCRIPTION")

	###
	# TODO
	# Check that $GAME_NAME and $script_version are non-empty strings
	# Display an explicit error message if one is unset or empty
	###

	# generate a multi-lines or single-line description based on the target package format
	# shellcheck disable=SC2039
	local package_description_full
	case "$OPTION_PACKAGE" in
		('deb')
			if [ -n "$package_description" ]; then
				package_description_full='Description: %s - %s'
				package_description_full="${package_description_full}"'\n ./play.it script version %s'
				# shellcheck disable=SC2059,SC2154
				package_description_full=$(printf "$package_description_full" "$GAME_NAME" "$package_description" "$script_version")
			else
				package_description_full='Description: %s'
				package_description_full="${package_description_full}"'\n ./play.it script version %s'
				# shellcheck disable=SC2059,SC2154
				package_description_full=$(printf "$package_description_full" "$GAME_NAME" "$script_version")
			fi
		;;
		('arch'|'gentoo')
			if [ -n "$package_description" ]; then
				# shellcheck disable=SC2154
				package_description_full="$GAME_NAME - $package_description - ./play.it script version $script_version"
			else
				# shellcheck disable=SC2154
				package_description_full="$GAME_NAME - ./play.it script version $script_version"
			fi
		;;
	esac

	printf '%s' "$package_description_full"
	return 0
}

# get "provide" field of given package
# USAGE: package_get_provide $package
# RETURNS: provided package ID as a non-empty string, or an empty string is none is provided
package_get_provide() {
	# single argument should be the package name
	# shellcheck disable=SC2039
	local package
	package="$1"
	if [ -z "$package" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_provide' '$package'
		return 1
	fi

	# get provided package ID from its name
	# shellcheck disable=SC2039
	local package_provide
	use_archive_specific_value "${package}_PROVIDE"
	package_provide=$(get_value "${package}_PROVIDE")

	# if no package is provided, return early
	if [ -z "$package_provide" ]; then
		return 0
	fi

	# on Gentoo, avoid mixups between numbers in package ID and version number
	# and add the required "!!games-playit/" prefix to the package ID
	case "$OPTION_PACKAGE" in
		('gentoo')
			package_provide=$(printf '%s' "$package_provide" | sed 's/-/_/g')
			package_provide="!!games-playit/${package_provide}"
		;;
	esac

	printf '%s' "$package_provide"
	return 0
}

# get path to the directory where the given package is prepared
# USAGE: package_get_path $package
# RETURNS: path to a directory, it is not checked that it exists or is writable
package_get_path() {
	# single argument should be the package name
	# shellcheck disable=SC2039
	local package
	package="$1"
	if [ -z "$package" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_path' '$package'
		return 1
	fi


	# check that an archive is set by the global context
	# shellcheck disable=SC2153
	if [ -z "$ARCHIVE" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_name' '$ARCHIVE'
	fi

	# check that PLAYIT_WORKDIR is set by the global context
	if [ -z "$PLAYIT_WORKDIR" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_name' '$PLAYIT_WORKDIR'
	fi

	# compute the package path from its identifier
	# shellcheck disable=SC2039
	local package_path
	package_path="${PLAYIT_WORKDIR}/$(package_get_id "$package")_$(packages_get_version "$ARCHIVE")_$(package_get_architecture_string "$package")"

	printf '%s' "$package_path"
	return 0
}

# get name of the given package’s archive without suffix
# USAGE: package_get_name $package
# RETURNS: filename, without any suffix
package_get_name() {
	# single argument should be the package name
	# shellcheck disable=SC2039
	local package
	package="$1"
	if [ -z "$package" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_name' '$package'
		return 1
	fi

	# check that an archive is set by the global context
	# shellcheck disable=SC2153
	if [ -z "$ARCHIVE" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'package_get_name' '$ARCHIVE'
	fi

	# compute the package path from its identifier
	# shellcheck disable=SC2039
	local package_name
	case "$OPTION_PACKAGE" in
		('arch'|'deb')
			package_name=$(basename "$(package_get_path "$package")")
		;;
		('gentoo'|'egentoo')
			package_name="$(package_get_id "$package")-$(packages_get_version "$ARCHIVE")"
		;;
		(*)
			error_invalid_argument 'OPTION_PACKAGE' 'package_get_name'
		;;
	esac

	printf '%s' "$package_name"
	return 0
}

# get the maintainer string
# USAGE: packages_get_maintainer
# RETURNS: packages maintainer, as a non-empty string
packages_get_maintainer() {
	# get maintainer string from /etc/makepkg.conf
	# shellcheck disable=SC2039
	local maintainer
	if \
		[ -r '/etc/makepkg.conf' ] && \
		grep --quiet '^PACKAGER=' '/etc/makepkg.conf'
	then
		if grep --quiet '^PACKAGER=".*"' '/etc/makepkg.conf'; then
			maintainer=$(sed 's/^PACKAGER="\(.*\)"/\1/' '/etc/makepkg.conf')
		elif grep --quiet "^PACKAGER='.*'" '/etc/makepkg.conf'; then
			maintainer=$(sed "s/^PACKAGER='\\(.*\\)'/\\1/" '/etc/makepkg.conf')
		else
			maintainer=$(sed 's/^PACKAGER=\(.*\)/\1/' '/etc/makepkg.conf')
		fi
	fi

	# return early if we already have a maintainer string
	if [ -n "$maintainer" ]; then
		printf '%s' "$maintainer"
		return 0
	fi

	# get current machine hostname
	# falls back on using "localhost"
	# shellcheck disable=SC2039
	local hostname
	if command -v 'hostname' >/dev/null 2>&1; then
		hostname=$(hostname)
	elif [ -r '/etc/hostname' ]; then
		hostname=$(cat '/etc/hostname')
	else
		hostname='localhost'
	fi

	# get current user name
	# falls back on "user"
	# shellcheck disable=SC2039
	local username
	if [ -n "$USER" ]; then
		username="$USER"
	elif command -v 'whoami' >/dev/null 2>&1; then
		username=$(whoami)
	elif command -v 'id' >/dev/null 2>&1; then
		username=$(id --name --user)
	else
		username='user'
	fi

	printf '%s@%s' "$username" "$hostname"
	return 0
}

# get the packages version string for a given archive
# USAGE: packages_get_version $archive
# RETURNS: packages version, as a non-empty string
packages_get_version() {
	# single argument should be the archive name
	# shellcheck disable=SC2039
	local archive
	archive="$1"
	if [ -z "$archive" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'packages_get_version' '$archive'
		return 1
	fi

	###
	# TODO
	# Check that $script_version is a non-empty string
	# Display an explicit error message if it is unset or empty
	###

	# get the version string for the current archive
	# falls back on "1.0-1"
	# shellcheck disable=SC2039
	local packages_version
	packages_version=$(get_value "${archive}_VERSION")
	if [ -z "$packages_version" ]; then
		packages_version='1.0-1'
	fi
	packages_version="${packages_version}+${script_version}"

	# Portage doesn't like some of our version names (See https://devmanual.gentoo.org/ebuild-writing/file-format/index.html)
	case "$OPTION_PACKAGE" in
		('gentoo'|'egentoo')
			set +o errexit
			packages_version=$(printf '%s' "$packages_version" | grep --extended-regexp --only-matching '^([0-9]{1,18})(\.[0-9]{1,18})*[a-z]?')
			set -o errexit
			if [ -z "$packages_version" ]; then
				packages_version='1.0'
			fi
		;;
	esac

	printf '%s' "$packages_version"
	return 0
}

# write .pkg.tar package meta-data
# USAGE: pkg_write_arch
# NEEDED VARS: GAME_NAME PKG_DEPS_ARCH
# CALLED BY: write_metadata
pkg_write_arch() {
	local pkg_deps
	use_archive_specific_value "${pkg}_DEPS"
	if [ "$(get_value "${pkg}_DEPS")" ]; then
		# shellcheck disable=SC2046
		pkg_set_deps_arch $(get_value "${pkg}_DEPS")
	fi
	use_archive_specific_value "${pkg}_DEPS_ARCH"
	if [ "$(get_value "${pkg}_DEPS_ARCH")" ]; then
		pkg_deps="$pkg_deps $(get_value "${pkg}_DEPS_ARCH")"
	fi
	local pkg_size
	pkg_size=$(du --total --block-size=1 --summarize "$(package_get_path "$pkg")" | tail --lines=1 | cut --fields=1)
	local target
	target="$(package_get_path "$pkg")/.PKGINFO"

	mkdir --parents "$(dirname "$target")"

	cat > "$target" <<- EOF
	# Generated by ./play.it $LIBRARY_VERSION
	pkgname = $(package_get_id "$pkg")
	pkgver = $(packages_get_version "$ARCHIVE")
	packager = $(packages_get_maintainer)
	builddate = $(date +%s)
	size = $pkg_size
	arch = $(package_get_architecture_string "$pkg")
	pkgdesc = $(package_get_description "$pkg")
	EOF

	for dep in $pkg_deps; do
		cat >> "$target" <<- EOF
		depend = $dep
		EOF
	done

	if [ -n "$(package_get_provide "$pkg")" ]; then
		cat >> "$target" <<- EOF
		conflict = $(package_get_provide "$pkg")
		provides = $(package_get_provide "$pkg")
		EOF
	fi

	if [ "$(package_get_architecture "$pkg")" = '32' ]; then
		cat >> "$target" <<- EOF
		conflict = $(package_get_id "$pkg" |sed 's/^lib32-//')
		provides = $(package_get_id "$pkg" |sed 's/^lib32-//')
		EOF
	fi

	target="$(package_get_path "$pkg")/.INSTALL"

	if [ -n "$(get_value "${pkg}_POSTINST_RUN")" ]; then
		cat >> "$target" <<- EOF
		post_install() {
		$(get_value "${pkg}_POSTINST_RUN")
		}

		post_upgrade() {
		post_install
		}
		EOF
	# For compatibility with pre-2.12 scripts, ignored if a package-specific value is already set
	elif [ -e "$postinst" ]; then
		compat_pkg_write_arch_postinst "$target"
	fi

	if [ -n "$(get_value "${pkg}_PRERM_RUN")" ]; then
		cat >> "$target" <<- EOF
		pre_remove() {
		$(get_value "${pkg}_PRERM_RUN")
		}

		pre_upgrade() {
		pre_remove
		}
		EOF
	# For compatibility with pre-2.12 scripts, ignored if a package-specific value is already set
	elif [ -e "$postinst" ]; then
		compat_pkg_write_arch_prerm "$target"
	fi
}

# set list or Arch Linux dependencies from generic names
# USAGE: pkg_set_deps_arch $dep[…]
# CALLS: pkg_set_deps_arch32 pkg_set_deps_arch64
# CALLED BY: pkg_write_arch
pkg_set_deps_arch() {
	case "$(package_get_architecture "$pkg")" in
		('32')
			pkg_set_deps_arch32 "$@"
		;;
		(*)
			pkg_set_deps_arch64 "$@"
		;;
	esac
}

# set list or Arch Linux 32-bit dependencies from generic names
# USAGE: pkg_set_deps_arch32 $dep[…]
# CALLS: warning_missing_library
# CALLED BY: pkg_set_deps_arch
pkg_set_deps_arch32() {
	for dep in "$@"; do
		case $dep in
			('alsa')
				pkg_dep='lib32-alsa-lib lib32-alsa-plugins'
			;;
			('bzip2')
				pkg_dep='lib32-bzip2'
			;;
			('dosbox')
				pkg_dep='dosbox'
			;;
			('freetype')
				pkg_dep='lib32-freetype2'
			;;
			('gcc32')
				pkg_dep='gcc-multilib lib32-gcc-libs'
			;;
			('gconf')
				pkg_dep='lib32-gconf'
			;;
			('libgdk_pixbuf-2.0.so.0')
				pkg_dep='lib32-gdk-pixbuf2'
			;;
			('glibc')
				pkg_dep='lib32-glibc'
			;;
			('libglib-2.0.so.0'|'libgobject-2.0.so.0')
				pkg_dep='lib32-glib2'
			;;
			('glu'|'libGLU.so.1')
				pkg_dep='lib32-glu'
			;;
			('glx')
				pkg_dep='lib32-libgl'
			;;
			('libgdk-x11-2.0.so.0'|'gtk2')
				pkg_dep='lib32-gtk2'
			;;
			('java')
				pkg_dep='jre8-openjdk'
			;;
			('json')
				pkg_dep='lib32-json-c'
			;;
			('libasound.so.2')
				pkg_dep='lib32-alsa-lib'
			;;
			('libasound_module_'*'.so')
				pkg_dep='lib32-alsa-plugins'
			;;
			('libcurl')
				pkg_dep='lib32-curl'
			;;
			('libcurl-gnutls')
				pkg_dep='lib32-libcurl-gnutls'
			;;
			('libmbedtls.so.12')
				warning_missing_library 'libmbedtls.so.12' 'Arch Linux' '32bits'
			;;
			('libpng16.so.16')
				pkg_dep='lib32-libpng'
			;;
			('libpulse.so.0'|'libpulse-simple.so.0')
				pkg_dep='lib32-libpulse'
			;;
			('libstdc++')
				pkg_dep='lib32-gcc-libs'
			;;
			('libudev1')
				pkg_dep='lib32-systemd'
			;;
			('libX11.so.6')
				pkg_dep='lib32-libx11'
			;;
			('libxrandr')
				pkg_dep='lib32-libxrandr'
			;;
			('nss')
				pkg_dep='lib32-nss'
			;;
			('openal'|'libopenal.so.1')
				pkg_dep='lib32-openal'
			;;
			('pulseaudio')
				pkg_dep='pulseaudio'
			;;
			('renpy')
				pkg_dep='renpy'
			;;
			('sdl1.2'|'libSDL-1.2.so.0')
				pkg_dep='lib32-sdl'
			;;
			('sdl2'|'libSDL2-2.0.so.0')
				pkg_dep='lib32-sdl2'
			;;
			('sdl2_image')
				pkg_dep='lib32-sdl2_image'
			;;
			('sdl2_mixer')
				pkg_dep='lib32-sdl2_mixer'
			;;
			('theora')
				pkg_dep='lib32-libtheora'
			;;
			('libturbojpeg.so.0')
				pkg_dep='lib32-libjpeg-turbo'
			;;
			('libuv.so.1')
				warning_missing_library 'libuv.so.1' 'Arch Linux' '32bits'
			;;
			('vorbis'|'libvorbisfile.so.3')
				pkg_dep='lib32-libvorbis'
			;;
			('wine'|'wine32'|'wine64')
				pkg_dep='wine'
			;;
			('wine-staging'|'wine32-staging'|'wine64-staging')
				pkg_dep='wine-staging'
			;;
			('winetricks')
				pkg_dep='winetricks xterm'
			;;
			('xcursor')
				pkg_dep='lib32-libxcursor'
			;;
			('xft')
				pkg_dep='lib32-libxft'
			;;
			('xgamma')
				pkg_dep='xorg-xgamma'
			;;
			('xrandr')
				pkg_dep='xorg-xrandr'
			;;
			('libz.so.1')
				pkg_dep='lib32-zlib'
			;;
			(*)
				pkg_dep="$dep"
			;;
		esac
		pkg_deps="$pkg_deps $pkg_dep"
	done
}

# set list or Arch Linux 64-bit dependencies from generic names
# USAGE: pkg_set_deps_arch64 $dep[…]
# CALLED BY: pkg_set_deps_arch
pkg_set_deps_arch64() {
	for dep in "$@"; do
		case $dep in
			('alsa')
				pkg_dep='alsa-lib alsa-plugins'
			;;
			('bzip2')
				pkg_dep='bzip2'
			;;
			('dosbox')
				pkg_dep='dosbox'
			;;
			('freetype')
				pkg_dep='freetype2'
			;;
			('gcc32')
				pkg_dep='gcc-multilib lib32-gcc-libs'
			;;
			('gconf')
				pkg_dep='gconf'
			;;
			('libgdk_pixbuf-2.0.so.0')
				pkg_dep='gdk-pixbuf2'
			;;
			('glibc')
				pkg_dep='glibc'
			;;
			('libgobject-2.0.so.0'|'libglib-2.0.so.0')
				pkg_dep='glib2'
			;;
			('glu'|'libGLU.so.1')
				pkg_dep='glu'
			;;
			('glx')
				pkg_dep='libgl'
			;;
			('libgdk-x11-2.0.so.0'|'gtk2')
				pkg_dep='gtk2'
			;;
			('java')
				pkg_dep='jre8-openjdk'
			;;
			('json')
				pkg_dep='json-c'
			;;
			('libasound.so.2')
				pkg_dep='alsa-lib'
			;;
			('libasound_module_'*'.so')
				pkg_dep='alsa-plugins'
			;;
			('libcurl')
				pkg_dep='curl'
			;;
			('libcurl-gnutls')
				pkg_dep='libcurl-gnutls'
			;;
			('libmbedtls.so.12')
				pkg_dep='mbedtls'
			;;
			('libpng16.so.16')
				pkg_dep='libpng'
			;;
			('libpulse.so.0'|'libpulse-simple.so.0')
				pkg_dep='libpulse'
			;;
			('libstdc++')
				pkg_dep='gcc-libs'
			;;
			('libudev1')
				pkg_dep='libsystemd'
			;;
			('libX11.so.6')
				pkg_dep='libx11'
			;;
			('libxrandr')
				pkg_dep='libxrandr'
			;;
			('nss')
				pkg_dep='nss'
			;;
			('mono')
				pkg_dep='mono'
			;;
			('openal'|'libopenal.so.1')
				pkg_dep='openal'
			;;
			('pulseaudio')
				pkg_dep='pulseaudio'
			;;
			('renpy')
				pkg_dep='renpy'
			;;
			('sdl1.2'|'libSDL-1.2.so.0')
				pkg_dep='sdl'
			;;
			('sdl2'|'libSDL2-2.0.so.0')
				pkg_dep='sdl2'
			;;
			('sdl2_image')
				pkg_dep='sdl2_image'
			;;
			('sdl2_mixer')
				pkg_dep='sdl2_mixer'
			;;
			('theora')
				pkg_dep='libtheora'
			;;
			('libturbojpeg.so.0')
				pkg_dep='libjpeg-turbo'
			;;
			('libuv.so.1')
				pkg_dep='libuv'
			;;
			('vorbis'|'libvorbisfile.so.3')
				pkg_dep='libvorbis'
			;;
			('wine'|'wine32'|'wine64')
				pkg_dep='wine'
			;;
			('winetricks')
				pkg_dep='winetricks'
			;;
			('xcursor')
				pkg_dep='libxcursor'
			;;
			('xft')
				pkg_dep='libxft'
			;;
			('xgamma')
				pkg_dep='xorg-xgamma'
			;;
			('xrandr')
				pkg_dep='xorg-xrandr'
			;;
			('libz.so.1')
				pkg_dep='zlib'
			;;
			(*)
				pkg_dep="$dep"
			;;
		esac
		pkg_deps="$pkg_deps $pkg_dep"
	done
}

# build .pkg.tar package
# USAGE: pkg_build_arch $pkg_path
# NEEDED VARS: (OPTION_COMPRESSION) (LANG) PLAYIT_WORKDIR
# CALLED BY: build_pkg
pkg_build_arch() {
	local pkg_filename
	pkg_filename=$(realpath "$OPTION_OUTPUT_DIR/$(basename "$1").pkg.tar")

	if [ -e "$pkg_filename" ] && [ $OVERWRITE_PACKAGES -ne 1 ]; then
		information_package_already_exists "$(basename "$pkg_filename")"
		eval ${pkg}_PKG=\"$pkg_filename\"
		export ${pkg?}_PKG
		return 0
	fi

	local tar_options
	tar_options='--create'
	if [ -z "$PLAYIT_TAR_IMPLEMENTATION" ]; then
		guess_tar_implementation
	fi
	case "$PLAYIT_TAR_IMPLEMENTATION" in
		('gnutar')
			tar_options="$tar_options --group=root --owner=root"
		;;
		('bsdtar')
			tar_options="$tar_options --gname=root --uname=root"
		;;
		(*)
			error_unknown_tar_implementation
		;;
	esac

	case $OPTION_COMPRESSION in
		('gzip')
			tar_options="$tar_options --gzip"
			pkg_filename="${pkg_filename}.gz"
		;;
		('xz')
			export XZ_DEFAULTS="${XZ_DEFAULTS:=--threads=0}"
			tar_options="$tar_options --xz"
			pkg_filename="${pkg_filename}.xz"
		;;
		('bzip2')
			tar_options="$tar_options --bzip2"
			pkg_filename="${pkg_filename}.bz2"
		;;
		('zstd')
			tar_options="$tar_options --zstd"
			pkg_filename="${pkg_filename}.zst"
		;;
		('none') ;;
		(*)
			error_invalid_argument 'OPTION_COMPRESSION' 'pkg_build_arch'
		;;
	esac

	information_package_building "$(basename "$pkg_filename")"
	if [ "$DRY_RUN" -eq 1 ]; then
		printf '\n'
		eval ${pkg}_PKG=\"$pkg_filename\"
		export ${pkg?}_PKG
		return 0
	fi

	(
		cd "$1"
		local files
		files='.PKGINFO *'
		if [ -e '.INSTALL' ]; then
			files=".INSTALL $files"
		fi
		debug_external_command "tar $tar_options --file \"$pkg_filename\" $files"
		tar $tar_options --file "$pkg_filename" $files
	)

	eval ${pkg}_PKG=\"$pkg_filename\"
	export ${pkg?}_PKG

	information_package_building_done
}

# write .deb package meta-data
# USAGE: pkg_write_deb
# NEEDED VARS: GAME_NAME PKG_DEPS_DEB
# CALLED BY: write_metadata
pkg_write_deb() {
	###
	# TODO
	# $pkg should be passed as a function argument, not inherited from the calling function
	###

	local pkg_deps pkg_size control_directory control_file postinst_script prerm_script

	control_directory="$(package_get_path "$pkg")/DEBIAN"
	control_file="$control_directory/control"
	postinst_script="$control_directory/postinst"
	prerm_script="$control_directory/prerm"

	# Get package dependencies list
	use_archive_specific_value "${pkg}_DEPS"
	if [ "$(get_value "${pkg}_DEPS")" ]; then
		# shellcheck disable=SC2046
		pkg_set_deps_deb $(get_value "${pkg}_DEPS")
	fi
	use_archive_specific_value "${pkg}_DEPS_DEB"
	if [ "$(get_value "${pkg}_DEPS_DEB")" ]; then
		if [ -n "$pkg_deps" ]; then
			pkg_deps="$pkg_deps, $(get_value "${pkg}_DEPS_DEB")"
		else
			pkg_deps="$(get_value "${pkg}_DEPS_DEB")"
		fi
	fi

	# Get package size
	pkg_size=$(du --total --block-size=1K --summarize "$(package_get_path "$pkg")" | tail --lines=1 | cut --fields=1)

	# Create metadata directory, enforce correct permissions
	mkdir --parents "$control_directory"
	chmod 755 "$control_directory"

	# Write main metadata file, enforce correct permissions
	cat > "$control_file" <<- EOF
	Package: $(package_get_id "$pkg")
	Version: $(packages_get_version "$ARCHIVE")
	Architecture: $(package_get_architecture_string "$pkg")
	Multi-Arch: foreign
	Maintainer: $(packages_get_maintainer)
	Installed-Size: $pkg_size
	Section: non-free/games
	EOF
	if [ -n "$(package_get_provide "$pkg")" ]; then
		cat >> "$control_file" <<- EOF
		Conflicts: $(package_get_provide "$pkg")
		Provides: $(package_get_provide "$pkg")
		Replaces: $(package_get_provide "$pkg")
		EOF
	fi
	if [ -n "$pkg_deps" ]; then
		cat >> "$control_file" <<- EOF
		Depends: $pkg_deps
		EOF
	fi
	cat >> "$control_file" <<- EOF
	$(package_get_description "$pkg")
	EOF
	chmod 644 "$control_file"

	# Write postinst/prerm scripts, enforce correct permissions
	if [ -n "$(get_value "${pkg}_POSTINST_RUN")" ]; then
		cat > "$postinst_script" <<- EOF
		#!/bin/sh -e

		$(get_value "${pkg}_POSTINST_RUN")

		exit 0
		EOF
		chmod 755 "$postinst_script"
	# For compatibility with pre-2.12 scripts, ignored if a package-specific value is already set
	elif [ -e "$postinst" ]; then
		compat_pkg_write_deb_postinst "$postinst_script"
	fi

	if [ -n "$(get_value "${pkg}_PRERM_RUN")" ]; then
		cat > "$prerm_script" <<- EOF
		#!/bin/sh -e

		$(get_value "${pkg}_PRERM_RUN")

		exit 0
		EOF
		chmod 755 "$prerm_script"
	# For compatibility with pre-2.12 scripts, ignored if a package-specific value is already set
	elif [ -e "$prerm" ]; then
		compat_pkg_write_deb_prerm "$prerm_script"
	fi
}

# set list of Debian dependencies from generic names
# USAGE: pkg_set_deps_deb $dep[…]
# CALLED BY: pkg_write_deb
pkg_set_deps_deb() {
	for dep in "$@"; do
		case $dep in
			('alsa')
				pkg_dep='libasound2-plugins'
			;;
			('bzip2')
				pkg_dep='libbz2-1.0'
			;;
			('dosbox')
				pkg_dep='dosbox'
			;;
			('freetype')
				pkg_dep='libfreetype6'
			;;
			('gcc32')
				pkg_dep='gcc-multilib:amd64 | gcc'
			;;
			('gconf')
				pkg_dep='libgconf-2-4'
			;;
			('libgdk_pixbuf-2.0.so.0')
				pkg_dep='libgdk-pixbuf-2.0-0 | libgdk-pixbuf2.0-0'
			;;
			('glibc')
				pkg_dep='libc6'
			;;
			('libgobject-2.0.so.0'|'libglib-2.0.so.0')
				pkg_dep='libglib2.0-0'
			;;
			('glu'|'libGLU.so.1')
				pkg_dep='libglu1-mesa | libglu1'
			;;
			('glx')
				pkg_dep='libgl1 | libgl1-mesa-glx, libglx-mesa0 | libgl1-mesa-glx'
			;;
			('libgdk-x11-2.0.so.0'|'gtk2')
				pkg_dep='libgtk2.0-0'
			;;
			('java')
				pkg_dep='default-jre:amd64 | java-runtime:amd64 | default-jre | java-runtime'
			;;
			('json')
				pkg_dep='libjson-c3 | libjson-c2 | libjson0'
			;;
			('libasound.so.2')
				pkg_dep='libasound2'
			;;
			('libasound_module_'*'.so')
				pkg_dep='libasound2-plugins'
			;;
			('libcurl')
				pkg_dep='libcurl4 | libcurl3'
			;;
			('libcurl-gnutls')
				pkg_dep='libcurl3-gnutls'
			;;
			('libmbedtls.so.12')
				pkg_dep='libmbedtls12'
			;;
			('libpng16.so.16')
				pkg_dep='libpng16-16'
			;;
			('libpulse.so.0'|'libpulse-simple.so.0')
				pkg_dep='libpulse0'
			;;
			('libstdc++')
				pkg_dep='libstdc++6'
			;;
			('libudev1')
				pkg_dep='libudev1'
			;;
			('libX11.so.6')
				pkg_dep='libx11-6'
			;;
			('libxrandr')
				pkg_dep='libxrandr2'
			;;
			('mono')
				pkg_dep='mono-runtime'
			;;
			('nss')
				pkg_dep='libnss3'
			;;
			('openal'|'libopenal.so.1')
				pkg_dep='libopenal1'
			;;
			('pulseaudio')
				pkg_dep='pulseaudio:amd64 | pulseaudio'
			;;
			('renpy')
				pkg_dep='renpy'
			;;
			('sdl1.2'|'libSDL-1.2.so.0')
				pkg_dep='libsdl1.2debian'
			;;
			('sdl2'|'libSDL2-2.0.so.0')
				pkg_dep='libsdl2-2.0-0'
			;;
			('sdl2_image')
				pkg_dep='libsdl2-image-2.0-0'
			;;
			('sdl2_mixer')
				pkg_dep='libsdl2-mixer-2.0-0'
			;;
			('theora')
				pkg_dep='libtheora0'
			;;
			('libturbojpeg.so.0')
				pkg_dep='libturbojpeg0'
			;;
			('libuv.so.1')
				pkg_dep='libuv1'
			;;
			('vorbis'|'libvorbisfile.so.3')
				pkg_dep='libvorbisfile3'
			;;
			('wine')
				case "$(package_get_architecture "$pkg")" in
					('32') pkg_set_deps_deb 'wine32' ;;
					('64') pkg_set_deps_deb 'wine64' ;;
				esac
			;;
			('wine32')
				pkg_dep='wine32 | wine32-development | wine-bin | wine-i386 | wine-staging-i386, wine'
			;;
			('wine64')
				pkg_dep='wine64 | wine64-development | wine64-bin | wine-amd64 | wine-staging-amd64, wine'
			;;
			('wine-staging')
				case "$(package_get_architecture "$pkg")" in
					('32') pkg_set_deps_deb 'wine32-staging' ;;
					('64') pkg_set_deps_deb 'wine64-staging' ;;
				esac
			;;
			('wine32-staging')
				pkg_dep='wine-staging-i386, winehq-staging:amd64 | winehq-staging'
			;;
			('wine64-staging')
				pkg_dep='wine-staging-amd64, winehq-staging'
			;;
			('winetricks')
				pkg_dep='winetricks, xterm:amd64 | xterm | zenity:amd64 | zenity | kdialog:amd64 | kdialog'
			;;
			('xcursor')
				pkg_dep='libxcursor1'
			;;
			('xft')
				pkg_dep='libxft2'
			;;
			('xgamma'|'xrandr')
				pkg_dep='x11-xserver-utils:amd64 | x11-xserver-utils'
			;;
			('libz.so.1')
				pkg_dep='zlib1g'
			;;
			(*)
				pkg_dep="$dep"
			;;
		esac
		if [ -n "$pkg_deps" ]; then
			pkg_deps="$pkg_deps, $pkg_dep"
		else
			pkg_deps="$pkg_dep"
		fi
	done
}

# build .deb package
# USAGE: pkg_build_deb $pkg_path
# NEEDED VARS: (OPTION_COMPRESSION) (LANG) PLAYIT_WORKDIR
# CALLED BY: build_pkg
pkg_build_deb() {
	local pkg_filename
	pkg_filename="$OPTION_OUTPUT_DIR/$(basename "$1").deb"
	if [ -e "$pkg_filename" ] && [ $OVERWRITE_PACKAGES -ne 1 ]; then
		information_package_already_exists "$(basename "$pkg_filename")"
		eval ${pkg}_PKG=\"$pkg_filename\"
		export ${pkg?}_PKG
		return 0
	fi

	local dpkg_options
	case $OPTION_COMPRESSION in
		('gzip'|'none'|'xz')
			dpkg_options="-Z$OPTION_COMPRESSION"
		;;
		(*)
			error_invalid_argument 'OPTION_COMPRESSION' 'pkg_build_deb'
		;;
	esac


	information_package_building "$(basename "$pkg_filename")"
	if [ "$DRY_RUN" -eq 1 ]; then
		printf '\n'
		eval ${pkg}_PKG=\"$pkg_filename\"
		export ${pkg?}_PKG
		return 0
	fi
	debug_external_command "TMPDIR=\"$PLAYIT_WORKDIR\" fakeroot -- dpkg-deb $dpkg_options --build \"$1\" \"$pkg_filename\" 1>/dev/null"
	TMPDIR="$PLAYIT_WORKDIR" fakeroot -- dpkg-deb $dpkg_options --build "$1" "$pkg_filename" 1>/dev/null
	eval ${pkg}_PKG=\"$pkg_filename\"
	export ${pkg?}_PKG

	information_package_building_done
}

# write .ebuild package meta-data
# USAGE: pkg_write_egentoo PKG_xxx
# NEEDED VARS: PKG_DEPS_GENTOO OPTION_COMPRESSION OPTION_OUTPUT_DIR
# CALLED BY: write_metadata
pkg_write_egentoo() {
	local package
	local pkg_deps
	local build_deps
	local unpack_f
	local package_filename
	local package_architectures
	local ebuild_path

	package="$1"
	if [ -z "$package" ];then
		# shellcheck disable=SC2016
		error_empty_string 'pkg_write_egentoo' '$package'
		return 1
	fi

	use_archive_specific_value "${package}_DEPS"
	if [ "$(get_value "${package}_DEPS")" ]; then
		#shellcheck disable=SC2046
		pkg_set_deps_gentoo $(get_value "${package}_DEPS")
	fi
	use_archive_specific_value "${package}_DEPS_GENTOO"
	if [ "$(get_value "${package}_DEPS_GENTOO")" ]; then
		pkg_deps="$pkg_deps $(get_value "${package}_DEPS_GENTOO")"
	fi
	if [ -n "$(package_get_provide "$package")" ]; then
		pkg_deps="${pkg_deps} $(package_get_provide "$package")"
	fi

	package_filename="$(package_get_name "$package").tar"
	unpack_f="default"
	case $OPTION_COMPRESSION in
		('gzip')
			package_filename="${package_filename}.gz"
		;;
		('xz')
			package_filename="${package_filename}.xz"
		;;
		('bzip2')
			package_filename="${package_filename}.bz2"
		;;
		('zstd')
			package_filename="${package_filename}.zst"
			build_deps="$build_deps app-arch/zstd"
			unpack_f="unzstd --force --quiet \$DISTDIR/$package_filename -o \$T/${package_filename%.zst} || die
	unpack \$T/${package_filename%.zst}"
		;;
		('lzip')
			package_filename="${package_filename}.lz"
			build_deps="$build_deps app-arch/lzip"
			unpack_f="lzip --decompress --keep \$DISTDIR/$package_filename -o \$T/${package_filename%.lz} || die
	unpack \$T/${package_filename%.lz}"
		;;
		('lzop')
			package_filename="${package_filename}.lzo"
			build_deps="$build_deps app-arch/lzop"
			unpack_f="lzop --decompress --path=\$T \$DISTDIR/$package_filename || die
	unpack \$T/${package_filename%.lzo}"
		;;
		('none') ;;
		(*)
			error_invalid_argument 'OPTION_COMPRESSION' 'pkg_write_egentoo'
		;;
	esac

	case "$(package_get_architecture "$package")" in
		('32')
			package_architectures='-* x86 amd64'
		;;
		('64')
			package_architectures='-* amd64'
		;;
		(*)
			package_architectures='x86 amd64' # data packages
		;;
	esac

	mkdir --parents "$OPTION_OUTPUT_DIR/$(package_get_architecture_string "$package")"
	ebuild_path=$(realpath "$OPTION_OUTPUT_DIR/$(package_get_architecture_string "$package")/$(package_get_name "$package").ebuild")

	cat > "$ebuild_path" << EOF
# Copyright 1999-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

EAPI=7
RESTRICT="fetch strip binchecks"

KEYWORDS="$package_architectures"
DESCRIPTION="$(package_get_description "$package")"
SRC_URI="$package_filename"
SLOT="0"

RDEPEND="$pkg_deps"
BDEPEND="$build_deps"

S=\${WORKDIR}

pkg_nofetch() {
	elog "Please move \$SRC_URI"
	elog "to your distfiles folder."
}

src_unpack() {
	$unpack_f
}

src_install() {
	cp --recursive --link --verbose \$S/* \$D || die
}
EOF

	if [ -n "$(get_value "${package}_POSTINST_RUN")" ]; then
		cat >> "$ebuild_path" << EOF

pkg_postinst() {
	$(get_value "${package}_POSTINST_RUN")
}
EOF

	# For compatibility with pre-2.12 scripts,
	# ignored if a package-specific value is already set
	elif [ -e "$postinst" ]; then
		compat_pkg_write_gentoo_postinst "$ebuild_path"
	fi

	if [ -n "$(get_value "${package}_PRERM_RUN")" ]; then
		cat >> "$ebuild_path" << EOF

pkg_prerm() {
	$(get_value "${package}_PRERM_RUN")
}
EOF

	# For compatibility with pre-2.12 scripts,
	# ignored if a package-specific value is already set
	elif [ -e "$prerm" ]; then
		compat_pkg_write_gentoo_prerm "$ebuild_path"
	fi
}

# builds dummy egentoo package
# USAGE: pkg_build_egentoo PKG_xxx
# NEEDED VARS: (LANG) PLAYIT_WORKDIR
# CALLED BY: build_pkg
pkg_build_egentoo() {
	local package
	local package_path
	local package_filename
	local tar_options

	package="$1"
	if [ -z "$package" ]; then
		# shellcheck disable=SC2016
		error_empty_string 'pkg_build_egentoo' '$package'
		return 1
	fi
	package_path=$(package_get_path "$package")

	# We don’t want both binary packages to overwrite each other
	mkdir --parents "$OPTION_OUTPUT_DIR/$(package_get_architecture_string "$package")"
	package_filename=$(realpath "$OPTION_OUTPUT_DIR/$(package_get_architecture_string "$package")/$(package_get_name "$package").tar")

	tar_options='--create'
	if [ -z "$PLAYIT_TAR_IMPLEMENTATION" ]; then
		guess_tar_implementation
	fi
	case "$PLAYIT_TAR_IMPLEMENTATION" in
		('gnutar')
			tar_options="$tar_options --group=root --owner=root"
		;;
		('bsdtar')
			tar_options="$tar_options --gname=root --uname=root"
		;;
		(*)
			error_unknown_tar_implementation
		;;
	esac

	case $OPTION_COMPRESSION in
		('gzip')
			tar_options="$tar_options --gzip"
			package_filename="${package_filename}.gz"
		;;
		('xz')
			export XZ_DEFAULTS="${XZ_DEFAULTS:=--threads=0}"
			tar_options="$tar_options --xz"
			package_filename="${package_filename}.xz"
		;;
		('bzip2')
			tar_options="$tar_options --bzip2"
			package_filename="${package_filename}.bz2"
		;;
		('zstd')
			tar_options="$tar_options --zstd"
			package_filename="${package_filename}.zst"
		;;
		('lzip')
			tar_options="$tar_options --lzip"
			package_filename="${package_filename}.lz"
		;;
		('lzop')
			tar_options="$tar_options --lzop"
			package_filename="${package_filename}.lzo"
		;;
		('none') ;;
		(*)
			error_invalid_argument 'OPTION_COMPRESSION' 'pkg_build_egentoo'
		;;
	esac

	if [ -e "$package_filename" ] && [ "$OVERWRITE_PACKAGES" -ne 1 ]; then
		information_package_already_exists "$(basename "$package_filename")"
		eval "${package}"_PKG=\""$package_filename"\"
		export "${package}"_PKG
		return 0
	fi

	information_package_building "$(basename "$package_filename")"
	if [ "$DRY_RUN" -eq 1 ]; then
		printf '\n'
		eval "${package}"_PKG=\""$package_filename"\"
		export "${package}"_PKG
		return 0
	fi

	debug_external_command "tar --directory \"$package_path\" $tar_options --file \"$package_filename\" ."
	# shellcheck disable=SC2046
	tar --directory "$package_path" $tar_options --file "$package_filename" .

	eval "${package}"_PKG=\""$package_filename"\"
	export "${package}"_PKG

	information_package_building_done
}
# write .ebuild package meta-data
# USAGE: pkg_write_gentoo
# NEEDED VARS: GAME_NAME PKG_DEPS_GENTOO
# CALLED BY: write_metadata
pkg_write_gentoo() {
	local pkg_deps
	use_archive_specific_value "${pkg}_DEPS"
	if [ "$(get_value "${pkg}_DEPS")" ]; then
		# shellcheck disable=SC2046
		pkg_set_deps_gentoo $(get_value "${pkg}_DEPS")
		export GENTOO_OVERLAYS
	fi
	use_archive_specific_value "${pkg}_DEPS_GENTOO"
	if [ "$(get_value "${pkg}_DEPS_GENTOO")" ]; then
		pkg_deps="$pkg_deps $(get_value "${pkg}_DEPS_GENTOO")"
	fi

	if [ -n "$(package_get_provide "$pkg")" ]; then
		pkg_deps="${pkg_deps} $(package_get_provide "$pkg")"
	fi

	mkdir --parents \
		"$PLAYIT_WORKDIR/$pkg/gentoo-overlay/metadata" \
		"$PLAYIT_WORKDIR/$pkg/gentoo-overlay/profiles" \
		"$PLAYIT_WORKDIR/$pkg/gentoo-overlay/games-playit/$(package_get_id "$pkg")/files"
	printf '%s\n' "masters = gentoo" > "$PLAYIT_WORKDIR/$pkg/gentoo-overlay/metadata/layout.conf"
	printf '%s\n' 'games-playit' > "$PLAYIT_WORKDIR/$pkg/gentoo-overlay/profiles/categories"
	ln --symbolic --force --no-target-directory "$(package_get_path "$pkg")" "$PLAYIT_WORKDIR/$pkg/gentoo-overlay/games-playit/$(package_get_id "$pkg")/files/install"
	local target
	target="$PLAYIT_WORKDIR/$pkg/gentoo-overlay/games-playit/$(package_get_id "$pkg")/$(package_get_id "$pkg")-$(packages_get_version "$ARCHIVE").ebuild"

	cat > "$target" <<- EOF
	EAPI=7
	RESTRICT="fetch strip binchecks"
	EOF
	local pkg_architectures
	case "$(package_get_architecture "$pkg")" in
		('32')
			pkg_architectures='-* x86 amd64'
		;;
		('64')
			pkg_architectures='-* amd64'
		;;
		(*)
			pkg_architectures='x86 amd64' #data packages
		;;
	esac
	cat >> "$target" <<- EOF
	KEYWORDS="$pkg_architectures"
	DESCRIPTION="$(package_get_description "$pkg")"
	SLOT="0"
	EOF

	# fakeroot >=1.25.1 considers all files belong to root by default
	cat >> "$target" <<- EOF
	RDEPEND="$pkg_deps"

	src_unpack() {
		mkdir --parents "\$S"
	}
	src_install() {
		cp --recursive --link \$FILESDIR/install/* \$ED/
	}
	EOF

	if [ -n "$(get_value "${pkg}_POSTINST_RUN")" ]; then
		cat >> "$target" <<- EOF
		pkg_postinst() {
		$(get_value "${pkg}_POSTINST_RUN")
		}
		EOF
	# For compatibility with pre-2.12 scripts, ignored if a package-specific value is already set
	elif [ -e "$postinst" ]; then
		compat_pkg_write_gentoo_postinst "$target"
	fi

	if [ -n "$(get_value "${pkg}_PRERM_RUN")" ]; then
		cat >> "$target" <<- EOF
		pkg_prerm() {
		$(get_value "${pkg}_PRERM_RUN")
		}
		EOF
	# For compatibility with pre-2.12 scripts, ignored if a package-specific value is already set
	elif [ -e "$prerm" ]; then
		compat_pkg_write_gentoo_prerm "$target"
	fi
}

# set list or Gentoo Linux dependencies from generic names
# USAGE: pkg_set_deps_gentoo $dep[…]
# CALLED BY: pkg_write_gentoo
pkg_set_deps_gentoo() {
	local architecture_suffix
	local architecture_suffix_use
	case "$(package_get_architecture "$pkg")" in
		('32')
			architecture_suffix='[abi_x86_32]'
			architecture_suffix_use=',abi_x86_32'
		;;
		('64')
			architecture_suffix=''
			architecture_suffix_use=''
		;;
	esac
	for dep in "$@"; do
		case $dep in
			('alsa')
				pkg_dep="media-libs/alsa-lib$architecture_suffix media-plugins/alsa-plugins$architecture_suffix"
			;;
			('bzip2')
				pkg_dep="app-arch/bzip2$architecture_suffix"
			;;
			('dosbox')
				pkg_dep="games-emulation/dosbox"
			;;
			('freetype')
				pkg_dep="media-libs/freetype$architecture_suffix"
			;;
			('gcc32')
				pkg_dep='' #gcc (in @system) should be multilib unless it is a no-multilib profile, in which case the 32 bits libraries wouldn't work
			;;
			('gconf')
				pkg_dep="gnome-base/gconf$architecture_suffix"
			;;
			('libgdk_pixbuf-2.0.so.0')
				pkg_dep="x11-libs/gdk-pixbuf:2$architecture_suffix"
			;;
			('glibc')
				pkg_dep="sys-libs/glibc"
				if [ "$(package_get_architecture "$pkg")" = '32' ]; then
					pkg_dep="$pkg_dep amd64? ( sys-libs/glibc[multilib] )"
				fi
			;;
			('libglib-2.0.so.0'|'libgobject-2.0.so.0')
				pkg_dep="dev-libs/glib:2$architecture_suffix"
			;;
			('glu'|'libGLU.so.1')
				pkg_dep="virtual/glu$architecture_suffix"
			;;
			('glx')
				pkg_dep="virtual/opengl$architecture_suffix"
			;;
			('libgdk-x11-2.0.so.0'|'gtk2')
				pkg_dep="x11-libs/gtk+:2$architecture_suffix"
			;;
			('java')
				pkg_dep='virtual/jre'
			;;
			('json')
				pkg_dep="dev-libs/json-c$architecture_suffix"
			;;
			('libasound.so.2')
				pkg_dep="media-libs/alsa-lib${architecture_suffix}"
			;;
			('libasound_module_'*'.so')
				pkg_dep="media-plugins/alsa-plugins${architecture_suffix}"
			;;
			('libcurl')
				pkg_dep="net-misc/curl$architecture_suffix"
			;;
			('libcurl-gnutls')
				pkg_dep="net-libs/libcurl-debian$architecture_suffix"
				pkg_overlay='steam-overlay'
			;;
			('libmbedtls.so.12')
				pkg_dep="net-libs/mbedtls:0/12$architecture_suffix"
			;;
			('libpng16.so.16')
				pkg_dep="media-libs/libpng:0/16$architecture_suffix"
			;;
			('libpulse.so.0'|'libpulse-simple.so.0')
				pkg_dep="media-sound/pulseaudio${architecture_suffix}"
			;;
			('libstdc++')
				pkg_dep='' #maybe this should be virtual/libstdc++, otherwise, it is included in gcc, which should be in @system
			;;
			('libudev1')
				pkg_dep="virtual/libudev$architecture_suffix"
			;;
			('libX11.so.6')
				pkg_dep="x11-libs/libX11${architecture_suffix}"
			;;
			('libxrandr')
				pkg_dep="x11-libs/libXrandr$architecture_suffix"
			;;
			('mono')
				pkg_dep="dev-lang/mono$architecture_suffix"
			;;
			('nss')
				pkg_dep="dev-libs/nss$architecture_suffix"
			;;
			('openal'|'libopenal.so.1')
				pkg_dep="media-libs/openal$architecture_suffix"
			;;
			('pulseaudio')
				pkg_dep='media-sound/pulseaudio'
			;;
			('renpy')
				pkg_dep='games-engines/renpy'
			;;
			('sdl1.2'|'libSDL-1.2.so.0')
				pkg_dep="media-libs/libsdl$architecture_suffix"
			;;
			('sdl2'|'libSDL2-2.0.so.0')
				pkg_dep="media-libs/libsdl2$architecture_suffix"
			;;
			('sdl2_image')
				# Most games will require at least jpeg and png
				# Maybe we should add gif and tiff to that list?
				pkg_dep="media-libs/sdl2-image[jpeg,png$architecture_suffix_use]"
			;;
			('sdl2_mixer')
				#Most games will require at least one of flac, mp3, vorbis or wav USE flags, it should better to require them all instead of not requiring any and having non-fonctionnal sound in some games.
				pkg_dep="media-libs/sdl2-mixer[flac,mp3,vorbis,wav$architecture_suffix_use]"
			;;
			('theora')
				pkg_dep="media-libs/libtheora$architecture_suffix"
			;;
			('libturbojpeg.so.0')
				pkg_dep="media-libs/libjpeg-turbo$architecture_suffix"
			;;
			('libuv.so.1')
				pkg_dep="dev-libs/libuv:0/1$architecture_suffix"
			;;
			('vorbis'|'libvorbisfile.so.3')
				pkg_dep="media-libs/libvorbis$architecture_suffix"
			;;
			('wine')
				case "$(package_get_architecture "$pkg")" in
					('32') pkg_set_deps_gentoo 'wine32' ;;
					('64') pkg_set_deps_gentoo 'wine64' ;;
				esac
			;;
			('wine32')
				pkg_dep='virtual/wine[abi_x86_32]'
			;;
			('wine64')
				pkg_dep='virtual/wine[abi_x86_64]'
			;;
			('wine-staging')
				case "$(package_get_architecture "$pkg")" in
					('32') pkg_set_deps_gentoo 'wine32-staging' ;;
					('64') pkg_set_deps_gentoo 'wine64-staging' ;;
				esac
			;;
			('wine32-staging')
				pkg_dep='virtual/wine[staging,abi_x86_32]'
			;;
			('wine64-staging')
				pkg_dep='virtual/wine[staging,abi_x86_64]'
			;;
			('winetricks')
				pkg_dep='app-emulation/winetricks
				|| (
					x11-terms/xterm
					gnome-extra/zenity
					kde-apps/kdialog
				)'
			;;
			('xcursor')
				pkg_dep="x11-libs/libXcursor$architecture_suffix"
			;;
			('xft')
				pkg_dep="x11-libs/libXft$architecture_suffix"
			;;
			('xgamma')
				pkg_dep='x11-apps/xgamma'
			;;
			('xrandr')
				pkg_dep='x11-apps/xrandr'
			;;
			('libz.so.1')
				pkg_dep="sys-libs/zlib:0/1$architecture_suffix"
			;;
			(*)
				pkg_dep="games-playit/$(printf '%s' "$dep" | sed 's/-/_/g')"
				# shellcheck disable=SC2039
				local package
				for package in $PACKAGES_LIST; do
					if [ "$package" != "$pkg" ]; then
						if [ "$(package_get_provide "$package")" = "$(printf '%s' "!!games-playit/${dep}" | sed 's/-/_/g')" ]; then
							pkg_dep="|| ( ${pkg_dep} )"
						fi
					fi
				done
			;;
		esac
		pkg_deps="$pkg_deps $pkg_dep"
		if [ -n "$pkg_overlay" ]; then
			if ! printf '%s' "$GENTOO_OVERLAYS" | sed --regexp-extended 's/\s+/\n/g' | grep --fixed-strings --line-regexp --quiet "$pkg_overlay"; then
				GENTOO_OVERLAYS="$GENTOO_OVERLAYS $pkg_overlay"
			fi
			pkg_overlay=''
		fi
	done
}

# build .tbz2 gentoo package
# USAGE: pkg_build_gentoo $pkg_path
# NEEDED VARS: (LANG) PLAYIT_WORKDIR
# CALLED BY: build_pkg
pkg_build_gentoo() {
	local pkg_filename_base
	pkg_filename_base="$(package_get_id "$pkg")-$(packages_get_version "$ARCHIVE").tbz2"

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	for package in $packages_list; do
		if [ "$package" != "$pkg" ] && [ "$(package_get_id "$package")" = "$(package_get_id "$pkg")" ]; then
			pkg_filename_base="$(package_get_architecture_string "$pkg")/$pkg_filename_base"
			mkdir --parents "$(dirname "$pkg_filename_base")"
		fi
	done
	local pkg_filename
	pkg_filename="$OPTION_OUTPUT_DIR/$pkg_filename_base"

	if [ -e "$pkg_filename" ] && [ $OVERWRITE_PACKAGES -ne 1 ]; then
		information_package_already_exists "$pkg_filename_base"
		eval ${pkg}_PKG=\"$pkg_filename\"
		export ${pkg}_PKG
		return 0
	fi

	information_package_building "$pkg_filename_base"
	if [ "$DRY_RUN" -eq 1 ]; then
		printf '\n'
		eval ${pkg}_PKG=\"$pkg_filename\"
		export ${pkg}_PKG
		return 0
	fi

	mkdir --parents "$PLAYIT_WORKDIR/portage-tmpdir"
	local ebuild_path
	ebuild_path="$PLAYIT_WORKDIR/$pkg/gentoo-overlay/games-playit/$(package_get_id "$pkg")/$(package_get_id "$pkg")-$(packages_get_version "$ARCHIVE").ebuild"
	ebuild "$ebuild_path" manifest 1>/dev/null
	debug_external_command "PORTAGE_TMPDIR=\"$PLAYIT_WORKDIR/portage-tmpdir\" PKGDIR=\"$PLAYIT_WORKDIR/gentoo-pkgdir\" BINPKG_COMPRESS=\"$OPTION_COMPRESSION\" fakeroot -- ebuild \"$ebuild_path\" package 1>/dev/null"
	PORTAGE_TMPDIR="$PLAYIT_WORKDIR/portage-tmpdir" PKGDIR="$PLAYIT_WORKDIR/gentoo-pkgdir" BINPKG_COMPRESS="$OPTION_COMPRESSION" fakeroot -- ebuild "$ebuild_path" package 1>/dev/null
	mv "$PLAYIT_WORKDIR/gentoo-pkgdir/games-playit/$(package_get_id "$pkg")-$(packages_get_version "$ARCHIVE").tbz2" "$pkg_filename"
	rm --recursive "$PLAYIT_WORKDIR/portage-tmpdir"

	eval ${pkg}_PKG=\"$pkg_filename\"
	export ${pkg}_PKG

	information_package_building_done
}

# print installation instructions
# USAGE: print_instructions $pkg[…]
# NEEDED VARS: (GAME_NAME) (OPTION_PACKAGE)
print_instructions() {
	[ "$GAME_NAME" ] || return 1
	if [ $# -eq 0 ]; then
		# shellcheck disable=SC2046
		print_instructions $(packages_get_list)
		return 0
	fi
	local package_arch
	local packages_list_32
	local packages_list_64
	local packages_list_all
	local string
	for package in "$@"; do
		package_arch="$(get_value "${package}_ARCH")"
		case "$package_arch" in
			('32')
				packages_list_32="$packages_list_32 $package"
			;;
			('64')
				packages_list_64="$packages_list_64 $package"
			;;
			(*)
				packages_list_all="$packages_list_all $package"
			;;
		esac

	done
	if [ "$OPTION_PACKAGE" = 'gentoo' ] && [ -n "$GENTOO_OVERLAYS" ]; then
		information_required_gentoo_overlays "$GENTOO_OVERLAYS"
	fi
	if [ "$OPTION_PACKAGE" = 'egentoo' ]; then
		info_local_overlay_gentoo
	fi
	information_installation_instructions_common "$GAME_NAME"
	if [ -n "$packages_list_32" ] && [ -n "$packages_list_64" ]; then
		print_instructions_architecture_specific '32' $packages_list_all $packages_list_32
		print_instructions_architecture_specific '64' $packages_list_all $packages_list_64
	else
		case $OPTION_PACKAGE in
			('arch')
				print_instructions_arch "$@"
			;;
			('deb')
				print_instructions_deb "$@"
			;;
			('gentoo')
				print_instructions_gentoo "$@"
			;;
			('egentoo')
				print_instructions_egentoo "$@"
			;;
			(*)
				error_invalid_argument 'OPTION_PACKAGE' 'print_instructions'
			;;
		esac
	fi
	printf '\n'
}

# print installation instructions, for a given architecture
# USAGE: print_instructions_architecture_specific $pkg[…]
# CALLS: print_instructions_arch print_instructions_deb print_instructions_gentoo
print_instructions_architecture_specific() {
	local architecture_variant
	architecture_variant="${1}-bit"
	information_installation_instructions_variant "$architecture_variant"
	shift 1
	case $OPTION_PACKAGE in
		('arch')
			print_instructions_arch "$@"
		;;
		('deb')
			print_instructions_deb "$@"
		;;
		('gentoo')
			print_instructions_gentoo "$@"
		;;
		('egentoo')
			print_instructions_egentoo "$@"
		;;
		(*)
			error_invalid_argument 'OPTION_PACKAGE' 'print_instructions'
		;;
	esac
}

# print installation instructions for Arch Linux
# USAGE: print_instructions_arch $pkg[…]
print_instructions_arch() {
	local pkg_path
	local str_format
	printf 'pacman -U'

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	for pkg in "$@"; do
		if [ "$OPTION_ARCHITECTURE" != all ] && [ -n "${packages_list##*$pkg*}" ]; then
			warning_skip_package 'print_instructions_arch' "$pkg"
			return 0
		fi
		pkg_path=$(realpath "$(get_value "${pkg}_PKG")")
		if [ -z "${pkg_path##* *}" ]; then
			str_format=' "%s"'
		else
			str_format=' %s'
		fi
		printf "$str_format" "$pkg_path"
	done
	printf '\n'
}

# print installation instructions for Debian
# USAGE: print_instructions_deb $pkg[…]
# CALLS: print_instructions_deb_apt print_instructions_deb_dpkg
print_instructions_deb() {
	if command -v apt >/dev/null 2>&1; then
		# shellcheck disable=SC2039
		local apt_version
		apt_version=$(LANG=C apt --version 2>/dev/null | \
			grep --extended-regexp --only-matching '[0-9]+(\.[0-9]+)+')
		if version_is_at_least '1.1' "$apt_version"; then
			print_instructions_deb_apt "$@"
		else
			print_instructions_deb_dpkg "$@"
		fi
	else
		print_instructions_deb_dpkg "$@"
	fi
}

# print installation instructions for Debian with apt
# USAGE: print_instructions_deb_apt $pkg[…]
# CALLS: print_instructions_deb_common
# CALLED BY: print_instructions_deb
print_instructions_deb_apt() {
	printf 'apt install'
	print_instructions_deb_common "$@"
}

# print installation instructions for Debian with dpkg + apt-get
# USAGE: print_instructions_deb_dpkg $pkg[…]
# CALLS: print_instructions_deb_common
# CALLED BY: print_instructions_deb
print_instructions_deb_dpkg() {
	printf 'dpkg -i'
	print_instructions_deb_common "$@"
	printf 'apt-get install -f\n'
}

# print installation instructions for Debian (common part)
# USAGE: print_instructions_deb_common $pkg[…]
# CALLED BY: print_instructions_deb_apt print_instructions_deb_dpkg
print_instructions_deb_common() {
	local pkg_path
	local str_format

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	for pkg in "$@"; do
		if [ "$OPTION_ARCHITECTURE" != all ] && [ -n "${packages_list##*$pkg*}" ]; then
			warning_skip_package 'print_instructions_deb_common' "$pkg"
			return 0
		fi
		pkg_path=$(realpath "$(get_value "${pkg}_PKG")")
		if [ -z "${pkg_path##* *}" ]; then
			str_format=' "%s"'
		else
			str_format=' %s'
		fi
		printf "$str_format" "$pkg_path"
	done
	printf '\n'
}

# print installation instructions for Gentoo Linux with ebuilds
# USAGE: print_instructions_egentoo $pkg[…]
print_instructions_egentoo() {
	local pkg_path
	local ebuild_path
	local pkg_id

	info_package_to_distfiles

	for pkg in "$@"; do
		if [ "$OPTION_ARCHITECTURE" != all ] && [ -n "${PACKAGES_LIST##*$pkg*}" ]; then
			warning_skip_package 'print_instructions_egentoo' "$pkg"
			return 0
		fi
		pkg_path=$(realpath "$(get_value "${pkg}_PKG")")
		ebuild_path="$(basename "${pkg_path%%.*}").ebuild"
		pkg_id="$(package_get_id "$pkg")"

		printf 'mkdir -p ${OVERLAY_PATH}/games-playit/%s\n' "${pkg_id}"
		printf 'mv %s ${OVERLAY_PATH}/games-playit/%s/\n' \
			"${ebuild_path}" "${pkg_id}"
		printf 'ebuild ${OVERLAY_PATH}/games-playit/%s/%s manifest\n' \
			"${pkg_id}" "${ebuild_path}"
		printf 'emerge games-playit/%s\n' "${pkg_id}"
	done
}

# print installation instructions for Gentoo Linux
# USAGE: print_instructions_gentoo $pkg[…]
print_instructions_gentoo() {
	local pkg_path
	local str_format
	printf 'quickunpkg --'

	# Get packages list for the current game
	local packages_list
	packages_list=$(packages_get_list)

	for pkg in "$@"; do
		if [ "$OPTION_ARCHITECTURE" != all ] && [ -n "${packages_list##*$pkg*}" ]; then
			warning_skip_package 'print_instructions_gentoo' "$pkg"
			return 0
		fi
		pkg_path=$(realpath "$(get_value "${pkg}_PKG")")
		if [ -z "${pkg_path##* *}" ]; then
			str_format=' "%s"'
		else
			str_format=' %s'
		fi
		printf "$str_format" "$pkg_path"
	done
	printf ' # https://downloads.dotslashplay.it/resources/gentoo/ '
	information_installation_instructions_gentoo_comment
	printf '\n'
}

# check that the target directory is on a case-sensitive filesystem
# USAGE: check_directory_is_case_sensitive $tested_directory
# RETURNS: 0 if case-sensitive, 1 if case-insensitive
check_directory_is_case_sensitive() {
	# the first argument should be a writable directory
	# shellcheck disable=SC2039
	local tested_directory
	tested_directory="$1"
	if [ ! -d "$tested_directory" ]; then
		error_not_a_directory "$tested_directory"
		return 1
	fi
	if [ ! -w "$tested_directory" ]; then
		error_not_writable "$tested_directory"
		return 1
	fi

	# check if "a" and "A" are created as distinct files, or as a single one
	# tests are done in an inner temporary directory to avoid messing up with existing files
	# shellcheck disable=SC2039
	local inner_temp_directory
	inner_temp_directory=$(mktemp --directory --tmpdir="$tested_directory")
	touch "${inner_temp_directory}/a"
	touch "${inner_temp_directory}/A"
	# shellcheck disable=SC2039
	local files_count
	files_count=$(find "$inner_temp_directory" -mindepth 1 -maxdepth 1 -iname a | wc --lines)
	rm --recursive "$inner_temp_directory"

	# if "a" and "A" were created as distinct files, the file system is case-sensitive
	# if they were created as a single file, it is case-insensitive
	case "$files_count" in
		(1)
			return 1
		;;
		(2)
			return 0
		;;
		(*)
			# we did not get an expected numeric value (1 or 2), let us assume we do not want to work with this directory
			# it might be better to actually throw some kind of explicit error with a message here
			return 1
		;;
	esac
}

# check that the target directory is on a filesystem supporting UNIX permissions
# USAGE: check_directory_supports_unix_permissions $tested_directory
# RETURNS: 0 if has support for UNIX permissions, 1 if has no support for UNIX permissions
check_directory_supports_unix_permissions() {
	# the first argument should be a writable directory
	# shellcheck disable=SC2039
	local tested_directory
	tested_directory="$1"
	if [ ! -d "$tested_directory" ]; then
		error_not_a_directory "$tested_directory"
		return 1
	fi
	if [ ! -w "$tested_directory" ]; then
		error_not_writable "$tested_directory"
		return 1
	fi

	# change permissions on a file, and check it has an actual effect
	# tests are done in an inner temporary directory to avoid messing up with existing files
	# shellcheck disable=SC2039
	local inner_temp_directory tested_temp_file
	inner_temp_directory=$(mktemp --directory --tmpdir="$tested_directory")
	tested_temp_file="${inner_temp_directory}/a"
	touch "$tested_temp_file"
	# shellcheck disable=SC2039
	local file_permissions
	for file_permissions in '600' '700'; do
		set +o errexit
		chmod "$file_permissions" "$tested_temp_file" 2>/dev/null
		set -o errexit
		if [ "$(stat --printf='%a' "$tested_temp_file")" != "$file_permissions" ]; then
			return 1
		fi
	done
	rm --recursive "$inner_temp_directory"
	return 0
}

# Compare two version strings using the x.y.z format
# Arbitrary number of numeric fields should be supported
# USAGE: version_is_at_least $version_reference $version_tested
# RETURNS: 0 if bigger or equal, 1 otherwise
version_is_at_least() {
	# shellcheck disable=SC2039
	local version_reference version_tested
	version_reference="$1"
	version_tested="$2"

	# If both version numbers are the same, return early
	if [ "$version_tested" = "$version_reference" ]; then
			return 0
	fi

	# Compare version fields one at a time
	# shellcheck disable=SC2039
	local field field_reference field_tested
	field=1
	field_reference=$(printf '%s' "$version_reference" | cut --delimiter='.' --fields=$field)
	field_tested=$(printf '%s' "$version_tested" | cut --delimiter='.' --fields=$field)
	while [ -n "$field_reference" ] && [ -n "$field_tested" ]; do
		if [ "$field_tested" -lt "$field_reference" ]; then
			return 1
		elif [ "$field_tested" -gt "$field_reference" ]; then
			return 0
		else
			field=$((field + 1))
			field_reference=$(printf '%s' "$version_reference" | cut --delimiter='.' --fields=$field)
			field_tested=$(printf '%s' "$version_tested" | cut --delimiter='.' --fields=$field)
		fi
	done

	# Both version number have a different number of fields, and all compared fields are the same
	# If the reference is longer, the tested version number is lower
	if [ -n "$field_reference" ]; then
		return 1
	# If the tested version is longer, it is higher than the reference one
	elif [ -n "$field_tested" ]; then
		return 0
	# We should never enter this last case
	else
		###
		# TODO
		# Fail with some explicit error
		###
		return 1
	fi
}

# print OK
# USAGE: print_ok
print_ok() {
	printf '\t\033[1;32mOK\033[0m\n'
}

# print a localized error message
# USAGE: print_error
print_error() {
	local string
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			string='Erreur :'
		;;
		('en'|*)
			string='Error:'
		;;
	esac
	exec 1>&2
	printf '\n\033[1;31m%s\033[0m\n' "$string"
}

# print a localized warning message
# USAGE: print_warning
print_warning() {
	local string
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			string='Avertissement :'
		;;
		('en'|*)
			string='Warning:'
		;;
	esac
	printf '\n\033[1;33m%s\033[0m\n' "$string"
}

# print DEBUG
# USAGE: print_debug
print_debug() {
	printf '\033[1;32mDEBUG:\033[0m ' >> /dev/stderr
	return 0
}

# print an option’s value
# USAGE: debug_option_value $option_name
debug_option_value() {
	local option_name
	local option_value

	if [ "$DEBUG" -eq 0 ]; then
		return 0
	else
		option_name="$1"

		if [ -z "$option_name" ]; then
			# shellcheck disable=SC2016
			error_empty_string 'debug_option_value' '$option_name'
			return 1

		else
			option_value=$(get_value "$option_name")

			print_debug
			printf '%s: %s\n' "$option_name" "$option_value" >> /dev/stderr
			return 0
		fi
	fi
}

# print the name of the entered function
# USAGE: debug_entering_function $function_name [$debug_level]
debug_entering_function() {
	local debug_level
	local function_name

	if [ "$#" -eq 2 ]; then
		debug_level="$2"

		case "$debug_level" in
			([0-9]) ;;
			(*) error_invalid_argument 'debug_level' 'debug_entering_function' ;;
		esac

	else
		debug_level=1
	fi

	if [ "$DEBUG" -lt "$debug_level" ]; then
		return 0
	else
		function_name="$1"

		if [ -z "$function_name" ]; then
			# shellcheck disable=SC2016
			error_empty_string 'debug_entering_function' '$function_name'
			return 1

		else
			print_debug
			printf 'Entering %s\n' "$function_name" >> /dev/stderr
			return 0
		fi
	fi
}

# print the name of the leaved function
# USAGE: debug_leaving_function $function_name [$debug_level]
debug_leaving_function() {
	local debug_level
	local function_name

	if [ "$#" -eq 2 ]; then
		debug_level="$2"

		case "$debug_level" in
			([0-9]) ;;
			(*) error_invalid_argument 'debug_level' 'debug_entering_function' ;;
		esac

	else
		debug_level=1
	fi

	if [ "$DEBUG" -lt "$debug_level" ]; then
		return 0
	else
		function_name="$1"

		if [ -z "$function_name" ]; then
			# shellcheck disable=SC2016
			error_empty_string 'debug_leaving_function' '$function_name'
			return 1

		else
			print_debug
			printf 'Leaving %s\n' "$function_name" >> /dev/stderr
			return 0
		fi
	fi
}

# print the name of the created directory
# USAGE: debug_creating_directory $dir_path
debug_creating_directory() {
	local dir_path

	if [ "$DEBUG" -eq 0 ]; then
		return 0
	else
		dir_path="$1"

		if [ -z "$dir_path" ]; then
			# shellcheck disable=SC2016
			error_empty_string 'debug_creating_directory' '$dir_path'
			return 1

		else
			print_debug
			printf 'Creating directory %s\n' "$dir_path" >> /dev/stderr
			return 0
		fi
	fi
}

# print the name of an used directory
# USAGE: debug_using_directory $dir_path
debug_using_directory() {
	if [ "$DEBUG" -eq 0 ]; then
		return 0
	fi

	# shecllcheck disable=SC2039
	local dir_path
	dir_path="$1"

	if [ -z "$dir_path" ]; then
		error_empty_string 'debug_using_directory' 'dir_path'
		return 1
	fi

	print_debug
	printf 'Using existing directory %s\n' "$dir_path" >> /dev/stderr
	return 0
}

# print the name of a source file found
# USAGE: debug_source_file $filename $source_path $inner_path
debug_source_file() {
	local found_status
	local filename

	###
	# TODO
	# check number of arguments
	###

	if [ "$DEBUG" -lt 2 ]; then
		return 0
	else
		found_status="$1"
		filename="$2"

		if [ -z "$filename" ]; then
			# shellcheck disable=SC2016
			error_empty_string 'debug_source_file' '$filename'
			return 1
		elif [ -z "$found_status" ]; then
			# shellcheck disable=SC2016
			error_empty_string 'debug_source_file' '$found_status'
			return 1

		else
			print_debug
			printf '%s file: %s\n' "$found_status" "${filename}" >> /dev/stderr
			return 0
		fi
	fi
}

# print the id of the package a source file is moved to
# intended to be used after debug_source_file
# USAGE: debug_file_to_package $package_id
debug_file_to_package() {
	local package_id

	if [ "$DEBUG" -lt 3 ]; then
		return 0
	else
		package_id="$1"

		if [ -z "$package_id" ]; then
			# shellcheck disable=SC2016
			error_empty_string 'debug_file_to_package' '$package_id'
			return 1

		else
			print_debug
			printf 'Moving file to: %s\n' "$package_id" >> /dev/stderr
			return 0
		fi
	fi
}

# print the type of the launcher being created
# USAGE: debug_write_launcher $launcher_type
debug_write_launcher() {
	local launcher_type
	local binary_file

	if [ "$DEBUG" -eq 0 ]; then
		return 0
	else
		launcher_type="$1"
		binary_file="$2"

		if [ -z "$launcher_type" ]; then
			# shellcheck disable=SC2016
			error_empty_string 'debug_write_launcher' '$launcher_type'
			return 1
		else
			print_debug
			if [ -z "$binary_file" ]; then
				printf 'Writing launcher: %s\n' "$launcher_type" >> /dev/stderr
			else
				printf 'Writing %s launcher: %s\n' "$launcher_type" "$binary_file" >> /dev/stderr
			fi
			return 0
		fi
	fi
}

# print an external command
# USAGE: debug_external_command $command
debug_external_command() {
	local external_command

	if [ "$DEBUG" -eq 0 ]; then
		return 0
	else
		external_command="$1"

		if [ -z "$external_command" ]; then
			# shellcheck disable=SC2016
			error_empty_string 'debug_external_command' '$external_command'
			return 1
		else
			print_debug
			printf 'Running: %s\n' "$external_command" >> /dev/stderr
			return 0
		fi
	fi
}

# print the path of a non-existant temporary directory
# USAGE: debug_temp_dir_nonexistant $directory
debug_temp_dir_nonexistant() {
	if [ "$DEBUG" -le 1 ]; then
		return 0
	fi

	# shellcheck disable=SC2039
	local directory
	directory="$1"

	if [ -z "$directory" ]; then
		error_empty_string 'debug_temp_dir_nonexistant' 'directory'
		return 1
	fi

	print_debug
	printf 'Skipping non-existant directory: %s\n' "$directory" >> /dev/stderr
	return 0
}

# print the path of a non-writable temporary directory
# USAGE: debug_temp_dir_nonwritable $directory
debug_temp_dir_nonwritable() {
	if [ "$DEBUG" -le 1 ]; then
		return 0
	fi

	# shellcheck disable=SC2039
	local directory
	directory="$1"

	if [ -z "$directory" ]; then
		error_empty_string 'debug_temp_dir_nonwritable' 'directory'
		return 1
	fi

	print_debug
	printf 'Skipping non-writable directory: %s\n' "$directory" >> /dev/stderr
	return 0
}

# print the path of a too small temporary directory
# USAGE: debug_temp_dir_not_enough_space $directory
debug_temp_dir_not_enough_space() {
	if [ "$DEBUG" -le 1 ]; then
		return 0
	fi

	# shellcheck disable=SC2039
	local directory
	directory="$1"

	if [ -z "$directory" ]; then
		error_empty_string 'debug_temp_dir_not_enough_space' 'directory'
		return 1
	fi

	print_debug
	printf 'Skipping too small directory: %s\n' "$directory" >> /dev/stderr
	return 0
}
# display an error when a function has been called with an invalid argument
# USAGE: error_invalid_argument $var_name $calling_function
error_invalid_argument() {
	local var value func
	var="$1"
	value="$(get_value "$var")"
	func="$2"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Valeur incorrecte pour %s appelée par %s : %s\n'
		;;
		('en'|*)
			message='Invalid value for %s called by %s: %s\n'
		;;
	esac
	print_error
	printf "$message" "$var" "$func" "$value"
	return 1
}

# display an error when a function is called without an argument, but is expecting some
# USAGE: error_missing_argument $function
error_missing_argument() {
	local message function
	function="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La fonction "%s" ne peut pas être appelée sans argument.\n'
		;;
		('en'|*)
			message='"%s" function can not be called without an argument.\n'
		;;
	esac
	print_error
	printf "$message" "$function"
	return 1
}

# display an error when a function is called with multiple arguments, but is expecting no more than one
# USAGE: error_extra_arguments $function
error_extra_arguments() {
	local message function
	function="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La fonction "%s" ne peut pas être appelée avec plus dʼun argument.\n'
		;;
		('en'|*)
			message='"%s" function can not be called with mor than one single argument.\n'
		;;
	esac
	print_error
	printf "$message" "$function"
	return 1
}

# display an error when a file is expected and something else has been given
# USAGE: error_not_a_file $param
error_not_a_file() {
	local message param
	param="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='"%s" nʼest pas un fichier valide.\n'
		;;
		('en'|*)
			message='"%s" is not a valid file.\n'
		;;
	esac
	print_error
	printf "$message" "$param"
	return 1
}

# display an error when an unknown application type is used
# USAGE: error_unknown_application_type $app_type
error_unknown_application_type() {
	local message application_type
	application_type="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Le type dʼapplication "%s" est inconnu.\n'
			message="$message"'Merci de signaler cette erreur sur notre outil de gestion de bugs : %s\n'
		;;
		('en'|*)
			message='"%s" application type is unknown.\n'
			message="$message"'Please report this issue in our bug tracker: %s\n'
		;;
	esac
	print_error
	printf "$message" "$application_type" "$PLAYIT_GAMES_BUG_TRACKER_URL"
	return 1
}

# display an error when the available tar implementation is not supported
# USAGE: error_unknown_tar_implementation
error_unknown_tar_implementation() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La version de tar présente sur ce système nʼest pas reconnue.\n'
			message="$message"'./play.it ne peut utiliser que GNU tar ou bsdtar.\n'
			message="$message"'Merci de signaler cette erreur sur notre outil de gestion de bugs : %s\n'
		;;
		('en'|*)
			message='The tar implementation on this system wasnʼt recognized.\n'
			message="$message"'./play.it can only use GNU tar or bsdtar.\n'
			message="$message"'Please report this issue in our bug tracker: %s\n'
		;;
	esac
	print_error
	printf "$message" "$PLAYIT_BUG_TRACKER_URL"
	return 1
}

# display a message when a required dependency is missing
# USAGE: error_dependency_not_found $command_name
error_dependency_not_found() {
	local message command_name provider
	command_name="$1"
	provider="$(dependency_provided_by "$command_name")"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='%s est introuvable. Installez %s avant de lancer ce script.\n'
		;;
		('en'|*)
			message='%s not found. Install %s before running this script.\n'
		;;
	esac
	print_error
	printf "$message" "$command_name" "$provider"
	return 1
}

# display an error message if an icon dependency is missing
# USAGE: error_icon_dependency_not_found $command_name
# NEEDED VARS: (LANG)
# CALLED BY: check_deps
error_icon_dependency_not_found() {
	local message command_name
	command_name="$1"
	set +o errexit
	error_dependency_not_found "$command_name"
	set -o errexit
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Vous pouvez aussi utiliser --icons=no ou --icons=auto\n'
		;;
		('en'|*)
			message='You can also use --icons=no or --icons=auto\n'
		;;
	esac
	printf "$message"
	return 1
}

# display an error when trying to extract an archive but no extractor is present
# USAGE: error_archive_no_extractor_found $archive_type
error_archive_no_extractor_found() {
	local message archive_type
	archive_type="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Ce script a essayé dʼextraire le contenu dʼune archive de type "%s", mais aucun outil approprié nʼa été trouvé.\n'
			message="$message"'Merci de signaler cette erreur sur notre outil de gestion de bugs : %s\n'
		;;
		('en'|*)
			message='This script tried to extract the contents of a "%s" archive, but not appropriate tool could be found.\n'
			message="$message"'Please report this issue in our bug tracker: %s\n'
		;;
	esac
	print_error
	printf "$message" "$archive_type" "$PLAYIT_GAMES_BUG_TRACKER_URL"
	return 1
}

# Display an error when trying to write a launcher for a missing binary
# USAGE: error_launcher_missing_binary $binary
# CALLS: print_error
error_launcher_missing_binary() {
	local binary message
	binary="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Le fichier suivant est introuvable, mais la création dʼun lanceur pour celui-ci a été demandée : %s\n'
			message="$message"'Merci de signaler cette erreur sur notre outil de gestion de bugs : %s\n'
		;;
		('en'|*)
			message='The following file can not be found, but a launcher targeting it should have been created: %s\n'
			message="$message"'Please report this issue on our bug tracker: %s\n'
		;;
	esac
	print_error
	printf "$message" "$binary" "$PLAYIT_GAMES_BUG_TRACKER_URL"
	return 1
}

# display an error when the value assigned to a given option is not valid
# USAGE: error_option_invalid $option_name $option_value
error_option_invalid() {
	local message option_name option_value
	option_name="$1"
	option_value="$2"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='%s nʼest pas une valeur valide pour --%s.\n'
			message="$message"'Lancez le script avec lʼoption --%s=help pour une liste des valeurs acceptés.\n'
		;;
		('en'|*)
			message='%s is not a valid value for --%s.\n'
			message="$message"'Run the script with the option --%s=help to get a list of supported values.\n'
		;;
	esac
	print_error
	printf "$message" "$option_value" "$option_name" "$option_name"
	return 1
}

# display an error when the compression method is not compatible with the
# package format
# USAGE: error_compression_invalid
error_compression_invalid() {
	# shellcheck disable=SC2039
	local compression_method allowed_values package_format message

	compression_method="$OPTION_COMPRESSION"
	allowed_values="$ALLOWED_VALUES_COMPRESSION"
	package_format="$OPTION_PACKAGE"

	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La méthode de compression "%s" nʼest pas compatible avec le format de paquets "%s".\n'
			message="$message"'Seules les méthodes suivantes sont acceptées :\n'
			message="$message"'\t%s\n'
			;;
		('en'|*)
			message='"%s" compression method is not compatible with "%s" package format.\n'
			message="$message"'Only the following options are accepted:\n'
			message="$message"'\t%s\n'
			;;
	esac
	print_error
	# shellcheck disable=SC2059
	printf "$message" "$compression_method" "$package_format" "$allowed_values"
	return 1
}

# display an error message when a required archive is not found
# list all the archives that could fulfill the requirement, with their download URL if one is provided
# USAGE: error_archive_not_found $archive[…]
error_archive_not_found() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			if [ $# -eq 1 ]; then
				message='Le fichier suivant est introuvable :\n'
			else
				message='Aucun des fichiers suivants nʼest présent :\n'
			fi
		;;
		('en'|*)
			if [ $# -eq 1 ]; then
				message='The following file could not be found:\n'
			else
				message='None of the following files could be found:\n'
			fi
		;;
	esac
	print_error
	printf "$message"
	information_archives_list "$@"
	return 1
}

# display an error message when we failed to guess the type of an archive
# USAGE: error_archive_type_not_set $archive
error_archive_type_not_set() {
	local message archive
	archive="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='ARCHIVE_TYPE nʼest pas défini pour %s et nʼa pas pu être détecté automatiquement.\n'
		;;
		('en'|*)
			message='ARCHIVE_TYPE is not set for %s and could not be guessed.\n'
		;;
	esac
	print_error
	printf "$message" "$archive"
	return 1
}

# display an error message when an integrity check fails
# USAGE: error_hashsum_mismatch $file
error_hashsum_mismatch() {
	local message file
	file=$(basename "$1")
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Somme de contrôle incohérente. %s nʼest pas le fichier attendu.\n'
			message="$message"'Utilisez --checksum=none pour forcer son utilisation.\n'
		;;
		('en'|*)
			message='Hashsum mismatch. %s is not the expected file.\n'
			message="$message"'Use --checksum=none to force its use.\n'
		;;
	esac
	print_error
	printf "$message" "$file"
	return 1
}

# display an error when the selected architecture is not supported
# USAGE: error_architecture_not_supported $architecture
error_architecture_not_supported() {
	local message architecture
	architecture="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Lʼarchitecture demandée nʼest pas gérée : %s\n'
		;;
		('en'|*)
			message='Selected architecture is not supported: %s\n'
		;;
	esac
	print_error
	printf "$message" "$architecture"
	return 1
}

# display an error when a variable required by the calling function is not set
# USAGE: error_variable_not_set $function $variable
error_variable_not_set() {
	local message function variable
	function="$1"
	variable="$2"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La fonction "%s" ne peut pas être appelée lorsque "%s" nʼa pas de valeur définie.\n'
		;;
		('en'|*)
			message='"%s" function can not be called when "%s" is not set.\n'
		;;
	esac
	print_error
	printf "$message" "$function" "$variable"
	return 1
}

# display an error if there is not enough free storage space
# print a list of directories that have been scanned for available space
# USAGE: error_not_enough_free_space $directory[…]
error_not_enough_free_space() {
	local message directory
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Il nʼy a pas assez dʼespace libre dans les différents répertoires testés :\n'
		;;
		('en'|*)
			message='There is not enough free space in the tested directories:\n'
		;;
	esac
	print_error
	printf "$message"
	for directory in "$@"; do
		printf '%s\n' "$directory"
	done
	return 1
}

# print error when available version of innoextract is too old
# USAGE: error_innoextract_version_too_old $archive
error_innoextract_version_too_old() {
	local message archive
	archive="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La version de innoextract disponible sur ce système est trop ancienne pour extraire les données de lʼarchive suivante : %s\n'
			message="$message"'Des instructions de mise-à-jour sont proposées :\n'
			message="$message"'- pour Debian : %s\n'
			message="$message"'- pour Ubuntu : %s\n'
		;;
		('en'|*)
			message='Available innoextract version is too old to extract data from the following archive: %s\n'
			message="$message"'Update instructions are proposed:\n'
			message="$message"'- for Debian: %s\n'
			message="$message"'- for Ubuntu: %s\n'
		;;
	esac
	print_error
	printf "$message" "$archive" \
		'https://forge.dotslashplay.it/play.it/doc/-/wikis/distributions/debian#available-innoextract-version-is-too-old' \
		'https://forge.dotslashplay.it/play.it/doc/-/wikis/distributions/ubuntu#innoextract-version-is-too-old'
	return 1
}

# diplay an error message if an icon file can not be found
# USAGE: error_icon_file_not_found $file
error_icon_file_not_found() {
	local message file
	file="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Le fichier dʼicône suivant est introuvable : %s\n'
			message="$message"'Merci de signaler cette erreur sur notre outil de gestion de bugs : %s\n'
		;;
		('en'|*)
			message='The following icon file could not be found: %s\n'
			message="$message"'Please report this issue in our bug tracker: %s\n'
		;;
	esac
	print_error
	printf "$message" "$file" "$PLAYIT_GAMES_BUG_TRACKER_URL"
	return 1
}

# display an error when called with an unknown option
# USAGE: error_option_unknown $option_name
error_option_unknown() {
	local message option_name
	option_name="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Option inconnue : %s\n'
		;;
		('en'|*)
			message='Unkown option: %s\n'
		;;
	esac
	print_error
	printf "$message" "$option_name"
	return 1
}

# display an error when a given path is not a directory
# USAGE: error_not_a_directory $path
error_not_a_directory() {
	local message path
	path="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='"%s" nʼest pas un répertoire.\n'
		;;
		('en'|*)
			message='"%s" is not a directory.\n'
		;;
	esac
	print_error
	printf "$message" "$path"
	return 1
}

# display an error when a given path is not writable
# USAGE: error_not_writable $path
error_not_writable() {
	local message path
	path="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='"%s" nʼest pas accessible en écriture.\n'
		;;
		('en'|*)
			message='"%s" is not writable.\n'
		;;
	esac
	print_error
	printf "$message" "$path"
	return 1
}

# display an error when a function has been given an unexpected empty string
# USAGE: error_empty_string $function $string
error_empty_string() {
	local message function string
	function="$1"
	string="$2"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Lʼargument "%s" fourni à la fonction "%s" ne doit pas être vide.\n'
			message="$message"'Merci de signaler cette erreur sur notre outil de gestion de bugs : %s\n'
		;;
		('en'|*)
			message='Argument "%s" as provided to function "%s" can not be empty.\n'
			message="$message"'Please report this issue in our bug tracker: %s\n'
		;;
	esac
	print_error
	printf "$message" "$string" "$function" "$PLAYIT_BUG_TRACKER_URL"
	return 1
}

# display an error when a required command is missing, but a function was expecting it to be available
# USAGE: error_unavailable_command $function $command
error_unavailable_command() {
	local message function command
	function="$1"
	command="$2"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La commande "%s" nʼest pas disponible, mais elle est requise par la fonction "%s".\n'
			message="$message"'Merci de signaler cette erreur sur notre outil de gestion de bugs : %s\n'
		;;
		('en'|*)
			message='"%s" command is not available, but it is required by function "%s".\n'
			message="$message"'Please report this issue in our bug tracker: %s\n'
		;;
	esac
	print_error
	printf "$message" "$command" "$function" "$PLAYIT_BUG_TRACKER_URL"
	return 1
}

# display an error when the calling script does not set its target library version
# USAGE: error_missing_target_version
# CALLS: print_error
error_missing_target_version() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Ce script ne donne aucune indication sur les versions de la bibliothèque ./play.it avec lesquelles il est compatible.\n'
			message="$message"'Merci de signaler cette erreur sur notre outil de gestion de bugs : %s\n'
		;;
		('en'|*)
			message='This script gives no indication about its compatibility level with the ./play.it library.\n'
			message="$message"'Please report this issue in our bug tracker: %s\n'
		;;
	esac
	print_error
	printf "$message" "$PLAYIT_GAMES_BUG_TRACKER_URL"
	return 1
}

# display an error when the calling script is not compatible with the provided library version
# USAGE: error_incompatible_versions
# CALLS: print_error
error_incompatible_versions() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Ce script nʼest pas compatible avec la version fournie de la bibliothèque ./play.it fournie.\n'
			message="$message"'La bibliothèque utilisée fournit actuellement la version %s, mais le script attend une version ≥ %s et < %s.\n'
		;;
		('en'|*)
			message='This script is not compatible with the ./play.it library version provided.\n'
			message="$message"'The library in use currently provides version %s, but the script expects a version ≥ %s and < %s.\n'
		;;
	esac
	print_error
	printf "$message" \
		"$VERSION_MAJOR_PROVIDED.$VERSION_MINOR_PROVIDED" \
		"$VERSION_MAJOR_TARGET.$VERSION_MINOR_TARGET" \
		"$((VERSION_MAJOR_TARGET + 1)).0"
	return 1
}

# display an error when no game script has been found for a given archive
# USAGE: error_no_script_found_for_archive $archive
error_no_script_found_for_archive() {
	local message archive
	archive="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Impossible de trouver un script pour le fichier %s\n'
		;;
		('en'|*)
			message='Could not find script for file %s\n'
		;;
	esac
	print_error
	printf "$message" "$archive"
	return 1
}

# display an error when a variable is empty
# USAGE: error_empty_variable $variable
error_empty_variable() {
	local message variable
	variable="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La variable "%s" nʼest pas définie, mais elle requise.\n'
			message="$message"'Merci de signaler cette erreur sur notre outil de gestion de bugs : %s\n'
		;;
		('en'|*)
			message='Variable "%s" is not set, but it is required.\n'
			message="$message"'Please report this issue in our bug tracker: %s\n'
		;;
	esac
	print_error
	printf "$message" "$variable" "$PLAYIT_GAMES_BUG_TRACKER_URL"
	return 1
}

# display an error when trying to use a case-insensitive filesystem
# USAGE: error_case_insensitive_filesystem_is_not_supported $directory
error_case_insensitive_filesystem_is_not_supported() {
	# shellcheck disable=SC2039
	local directory
	directory="$1"

	# shellcheck disable=SC2039
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Ce répertoire se trouve sur un système de fichiers insensible à la casse : %s\n'
			message="$message"'Ce type de système de fichiers nʼest pas géré pour lʼopération demandée.\n'
		;;
		('en'|*)
			message='The following directory is on a case-insensitive filesystem: %s\n'
			message="$message"'Such filesystems are not supported for the current operation.\n'
		;;
	esac

	print_error
	# shellcheck disable=SC2059
	printf "$message" "$directory"

	return 1
}


# display an error when trying to use a filesystem without support for UNIX permissions
# USAGE: error_unix_permissions_support_is_required $directory
error_unix_permissions_support_is_required() {
	# shellcheck disable=SC2039
	local directory
	directory="$1"

	# shellcheck disable=SC2039
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Ce répertoire se trouve sur un système de fichiers ne gérant pas les permissions UNIX : %s\n'
			message="$message"'Ce type de système de fichiers nʼest pas géré pour lʼopération demandée.\n'
		;;
		('en'|*)
			message='The following directory is on filesystem with no support for UNIX permissions: %s\n'
			message="$message"'Such filesystems are not supported for the current operation.\n'
		;;
	esac

	print_error
	# shellcheck disable=SC2059
	printf "$message" "$directory"

	return 1
}

# display full usage instructions
# USAGE: help
help() {
	local message script_name
	script_name=$(basename "$0")

	# print general usage instructions
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			if [ "$script_name" = 'play.it' ]; then
				message='\nUtilisation : %s ARCHIVE [OPTION]…\n\n'
			else
				message='\nUtilisation : %s [OPTION]… [ARCHIVE]\n\n'
			fi
		;;
		('en'|*)
			if [ "$script_name" = 'play.it' ]; then
				message='\nUsage: %s ARCHIVE [OPTION]…\n\n'
			else
				message='\nUsage: %s [OPTION]… [ARCHIVE]\n\n'
			fi
		;;
	esac
	printf "$message" "$script_name"

	# print details about options usage
	printf 'OPTIONS\n\n'
	help_architecture
	help_checksum
	help_compression
	help_prefix
	help_package
	help_dryrun
	help_skipfreespacecheck
	help_icons
	help_overwrite
	help_output_dir
	help_debug

	# do not print a list of supported archives if called throught the "play.it" wrapper script
	if [ "$script_name" = 'play.it' ]; then
		return 0
	fi

	# print list of supported archives
	printf 'ARCHIVE\n\n'
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Ce script reconnaît les archives suivantes :'
		;;
		('en'|*)
			message='This script can work on the following archives:'
		;;
	esac
	printf '%s\n' "$message"
	# shellcheck disable=SC2046
	information_archives_list $(archives_return_list)

	return 0
}

# display --architecture option usage
# USAGE: help_architecture
help_architecture() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tChoix de lʼarchitecture à construire\n\n'
			message="$message"'\t%s\ttoutes les architectures disponibles\n'                        # all
			message="$message"'\t%s\tpaquets 32-bit seulement\n'                                    # 32
			message="$message"'\t%s\tpaquets 64-bit seulement\n'                                    # 64
			message="$message"'\t%s\tpaquets pour lʼarchitecture du système courant uniquement\n\n' # auto
		;;
		('en'|*)
			message='\tTarget architecture selection\n\n'
			message="$message"'\t%s\tall available architectures\n'                     # all
			message="$message"'\t%s\t32-bit packages only\n'                            # 32
			message="$message"'\t%s\t64-bit packages only\n'                            # 64
			message="$message"'\t%s\tpackages for current system architecture only\n\n' # auto
		;;
	esac
	printf -- '--architecture=all|32|64|auto\n'
	printf -- '--architecture all|32|64|auto\n\n'
	printf "$message" 'all' '32' '64' 'auto'
	return 0
}

# display --checksum option usage
# USAGE: help_checksum
help_checksum() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tChoix de la méthode de vérification dʼintégrité de lʼarchive\n\n'
			message="$message"'\t%s\tvérification via md5sum\n' # md5
			message="$message"'\t%s\tpas de vérification\n\n'   # none
		;;
		('en'|*)
			message='\tArchive integrity verification method selection\n\n'
			message="$message"'\t%s\tmd5sum verification\n' # md5
			message="$message"'\t%s\tno verification\n\n'   # none
		;;
	esac
	printf -- '--checksum=md5|none\n'
	printf -- '--checksum md5|none\n\n'
	printf "$message" 'md5' 'none'
	return 0
}

# display --compression option usage
# CALLED BY: help
help_compression() {
	# shellcheck disable=SC2039
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tChoix de la méthode de compression des paquets générés\n'
			message="$message"'(Certaines options peuvent ne pas être disponible suivant le format de paquet choisi.)\n\n'
			message="$message"'\t%s\tpas de compression\n'
			message="$message"'\t%s\tcompression gzip (rapide)\n'
			message="$message"'\t%s\tcompression xz (plus lent mais plus efficace que gzip)\n'
			message="$message"'\t%s\tcompression bzip2\n'
			message="$message"'\t%s\tcompression zstd\n'
			message="$message"'\t%s\tcompression lz4 (le plus rapide, mais le plus lourd)\n'
			message="$message"'\t%s\tcompression lzip (similaire à xz)\n'
			message="$message"'\t%s\tcompression lzop (plus lent que lz4 à décompresser mais plus efficace)\n\n'
		;;
		('en'|*)
			message='\tGenerated packages compression method selection\n'
			message="$message"'(Some options may not be available depending on the chosen package format.)\n\n'
			message="$message"'\t%s\tno compression\n'
			message="$message"'\t%s\tgzip compression (fast)\n'
			message="$message"'\t%s\txz compression (slower but more efficient than gzip)\n'
			message="$message"'\t%s\tbzip2 compression\n'
			message="$message"'\t%s\tzstd compression\n'
			message="$message"'\t%s\tlz4 compression (fastest but biggest files)\n'
			message="$message"'\t%s\tlzip compression (similar to xz)\n'
			message="$message"'\t%s\tlzop compression (slower than lz4 at inflating but more efficient)\n\n'
		;;
	esac
	printf -- '--compression=none|gzip|xz|bzip2|zstd|lz4|lzip|lzop\n'
	printf -- '--compression none|gzip|xz|bzip2|zstd|lz4|lzip|lzop\n\n'
	# shellcheck disable=SC2059
	printf "$message" 'none' 'gzip' 'xz' 'bzip2' 'zstd' 'lz4' 'lzip' 'lzop'
	return 0
}

# display --prefix option usage
# USAGE: help_prefix
help_prefix() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tChoix du chemin dʼinstallation du jeu\n\n'
			message="$message"'\tCette option accepte uniquement un chemin absolu.\n\n'
		;;
		('en'|*)
			message='\tGame installation path setting\n\n'
			message="$message"'\tThis option accepts an absolute path only.\n\n'
		;;
	esac
	printf -- '--prefix=$path\n'
	printf -- '--prefix $path\n\n'
	printf "$message"
	return 0
}

# display --package option usage
# USAGE: help_package
help_package() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tChoix du type de paquet à construire\n\n'
			message="$message"'\t%s\tpaquet .pkg.tar (Arch Linux)\n' # arch
			message="$message"'\t%s\tpaquet .deb (Debian, Ubuntu)\n' # deb
			message="$message"'\t%s\tpaquet .tbz2 (Gentoo)\n\n'      # gentoo
		;;
		('en'|*)
			message='\tGenerated package type selection\n\n'
			message="$message"'\t%s\t.pkg.tar package (Arch Linux)\n' # arch
			message="$message"'\t%s\t.deb package (Debian, Ubuntu)\n' # deb
			message="$message"'\t%s\t.tbz2 package (Gentoo)\n\n'      #gentoo
		;;
	esac
	printf -- '--package=arch|deb|gentoo\n'
	printf -- '--package arch|deb|gentoo\n\n'
	printf "$message" 'arch' 'deb' 'gentoo'
	return 0
}

# display --dry-run option usage
# USAGE: help_dryrun
help_dryrun() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tEffectue des tests de syntaxe mais nʼextrait pas de données et ne construit pas de paquets.\n\n'
		;;
		('en'|*)
			message='\tRun syntax checks but do not extract data nor build packages.\n\n'
		;;
	esac
	printf -- '--dry-run\n\n'
	printf "$message"
	return 0
}

# display --skip-free-space-check option usage
# USAGE: help_skipfreespacecheck
help_skipfreespacecheck() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tNe teste pas lʼespace libre disponible. Les répertoires temporaires seront créés sous $TMPDIR, ou /tmp par défaut.\n\n'
		;;
		('en'|*)
			message='\tDo not check for free space. Temporary directories are created under $TMPDIR, or /tmp by default.\n\n'
		;;
	esac
	printf -- '--skip-free-space-check\n\n'
	printf "$message"
	return 0
}

# display --icons option usage
# USAGE: help_icons
help_icons() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tInclure ou non les icônes dans les paquets\n\n'
			message="$message"'\t%s\tInclure les icônes et sʼarrêter si une dépendance nʼa pas pu être trouvée\n' # yes
			message="$message"'\t%s\tNe pas inclure les icônes même si toutes les dépendances sont présentes\n'   # no
			message="$message"'\t%s\tInclure les icônes seulement si toutes les dépendances sont présentes\n\n'   # auto
		;;
		('en'|*)
			message='\tInclude icons in packages\n\n'
			message="$message"'\t%s\tInclude icons and stop if a dependency wasnʼt found\n'      # yes
			message="$message"'\t%s\tDonʼt include icons even if all dependencies are present\n' # no
			message="$message"'\t%s\tOnly include icons if all dependencies are present\n\n'     # auto
		;;
	esac
	printf -- '--icons=yes|no|auto\n'
	printf -- '--icons yes|no|auto\n\n'
	printf "$message" 'yes' 'no' 'auto'
	return 0
}

# display --overwrite option usage
# USAGE: help_overwrite
help_overwrite() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tRemplace les paquets si ils existent déjà.\n\n'
		;;
		('en'|*)
			message='\tReplace packages if they already exist.\n\n'
		;;
	esac
	printf -- '--overwrite\n\n'
	printf "$message"
	return 0
}

# display --output-dir option usage
# USAGE: help_output_dir
help_output_dir() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tDéfinit le répertoire de destination des paquets générés.\n\n'
		;;
		('en'|*)
			message='\tSet the output directory for generated packages.\n\n'
		;;
	esac
	printf -- '--output-dir\n\n'
	printf "$message"
	return 0
}

# display --debug option usage
# USAGE: help_debug
help_debug() {
	local message

	#shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\tDéfinit le niveau de debug. Il vaut 1 par défaut.\n\n'
			;;
		('en'|*)
			message='\tSet the debug level. Default is 1.\n\n'
			;;
	esac

	printf -- '--debug\n'
	printf -- '--debug=N\n'
	printf -- '--debug N\n\n'
	printf "$message"

	return 0
}
# display a list of archives, one per line, with their download URL if one is provided
# USAGE: information_archives_list $archive[…]
information_archives_list() {
	local archive archive_name archive_url
	for archive in "$@"; do
		archive_name="$(get_value "$archive")"
		archive_url="$(get_value "${archive}_URL")"
		if [ -n "$archive_url" ]; then
			printf '%s — %s\n' "$archive_name" "$archive_url"
		else
			printf '%s\n' "$archive_name"
		fi
	done
	return 0
}

# display the name of a file currently processed
# USAGE: information_file_in_use $file
information_file_in_use() {
	local message file
	file="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Utilisation de %s\n'
		;;
		('en'|*)
			message='Using %s\n'
		;;
	esac
	printf "$message" "$file"
	return 0
}

# display a notification when trying to build a package that already exists
# USAGE: information_package_already_exists $file
information_package_already_exists() {
	local message file
	file="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='%s existe déjà.\n'
		;;
		('en'|*)
			message='%s already exists.\n'
		;;
	esac
	printf "$message" "$file"
	return 0
}

# print integrity check message
# USAGE: information_file_integrity_check $file
information_file_integrity_check() {
	local message file
	file=$(basename "$1")
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Contrôle de lʼintégrité de %s'
		;;
		('en'|*)
			message='Checking integrity of %s'
		;;
	esac
	printf "$message" "$file"
	return 0
}

# print data extraction message
# USAGE: information_archive_data_extraction $file
information_archive_data_extraction() {
	# shellcheck disable=SC2039
	local message file
	file="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Extraction des données de %s…\n'
		;;
		('en'|*)
			message='Extracting data from %s…\n'
		;;
	esac
	# shellcheck disable=SC2059
	printf "$message" "$file"
	return 0
}

# print data extraction success message
# USAGE: information_archive_data_extraction_done
information_archive_data_extraction_done() {
	# shellcheck disable=SC2039
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Extraction réussie !'
		;;
		('en'|*)
			message='Extraction done!'
		;;
	esac
	printf '%s\n' "$message"
	return 0
}

# print package building message
# USAGE: information_package_building $file
information_package_building() {
	# shellcheck disable=SC2039
	local message file
	file="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Construction de %s…\n'
		;;
		('en'|*)
			message='Building %s…\n'
		;;
	esac
	# shellcheck disable=SC2059
	printf "$message" "$file"
	return 0
}

# print package building message
# USAGE: information_package_building $file
information_package_building_done() {
	# shellcheck disable=SC2039
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Contruction terminée !'
		;;
		('en'|*)
			message='Building done!'
		;;
	esac
	printf '%s\n' "$message"
	return 0
}

# print notification about required overlays when building Gentoo packages
# USAGE: information_required_gentoo_overlays $overlays
information_required_gentoo_overlays() {
	local message overlays
	overlays="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\nVous pouvez avoir besoin des overlays suivants pour installer ces paquets : %s\n'
		;;
		('en'|*)
			message='\nYou may need the following overlays to install these packages: %s\n'
		;;
	esac
	printf "$message" "$overlays"
	return 0
}

# print common part of packages installation instructions
# USAGE: information_installation_instructions_common $game_name
information_installation_instructions_common() {
	local message game_name
	game_name="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\nInstallez "%s" en lançant la série de commandes suivantes en root :\n'
		;;
		('en'|*)
			message='\nInstall "%s" by running the following commands as root:\n'
		;;
	esac
	printf "$message" "$game_name"
}

# print variant precision for packages installation instructions
# USAGE: information_installation_instructions_variant $variant
information_installation_instructions_variant() {
	local message variant
	variant="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\nversion %s :\n'
		;;
		('en'|*)
			message='\n%s version:\n'
		;;
	esac
	printf "$message" "$variant"
	return 0
}

# add comment to packages installation instructions on Gentoo
# USAGE: information_installation_instructions_gentoo_comment
information_installation_instructions_gentoo_comment() {
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='ou mettez les paquets dans un PKGDIR (dans un dossier nommé games-playit) et emergez-les'
		;;
		('en'|*)
			message='or put the packages in a PKGDIR (in a folder named games-playit) and emerge them'
		;;
	esac
	printf "$message"
	return 0
}

# print integrity check message
# USAGE: info_archive_integrity_check $file
info_archive_integrity_check() {
	local file message
	file=$(basename "$1")
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Contrôle de lʼintégrité de %s'
		;;
		('en'|*)
			message='Checking integrity of %s'
		;;
	esac
	printf "$message" "$file"
	return 0
}

# print hash computation message
# USAGE: info_archive_hash_computation $file
info_archive_hash_computation() {
	# shellcheck disable=SC2039
	local file message
	file=$(basename "$1")
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Calcul de la somme de contrôle de %s…\n'
		;;
		('en'|*)
			message='Computing hashsum for %s…\n'
		;;
	esac
	# shellcheck disable=SC2059
	printf "$message" "$file"
	return 0
}

# print hash computation success message
# USAGE: info_archive_hash_computation_done
info_archive_hash_computation_done() {
	# shellcheck disable=SC2039
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Somme de contrôle calculée !'
		;;
		('en'|*)
			message='Hashsum computed!'
		;;
	esac
	printf '%s\n' "$message"
	return 0
}

# inform the need of a local overlay on gentoo for ebuilds
# USAGE: info_local_overlay_gentoo
info_local_overlay_gentoo() {
	# shellcheck disable=SC2039
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='\nUn overlay local est nécessaire pour utiliser les ebuilds générés par ./play.it\n'
			message="$message"'Dans la suite OVERLAY_PATH correspond au chemin de votre overlay local\n'
		;;
		('en'|*)
			message='\nA local overlay is needed to use the ebuilds generated by ./play.it\n'
			message="$message"'In what comes next, OVERLAY_PATH is the path to your local overlay\n'
		;;
	esac
	# shellcheck disable=SC2059
	printf "$message"
	return 0
}

# inform the need to move the packages to a distfile on egentoo
# USAGE: info_package_to_distfiles
info_package_to_distfiles() {
	# shellcheck disable=SC2039
	local message
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Déplacez les paquets créés dans votre disfile\n'
			message="$message"'puis exécutez les instructions suivantes :\n'
			;;
		('en'|*)
			message='Move the generated packages into your distfile\n'
			message="$message"'then run the following commands:\n'
			;;
	esac
	# shellcheck disable=SC2059
	printf "$message"
	return 0
}

# display a warning when PKG value is not included in PACKAGES_LIST
# USAGE: warning_skip_package $function_name $package
warning_skip_package() {
	local message function package
	function="$1"
	package="$2"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La valeur de PKG fournie à %s ne fait pas partie de la liste de paquets à construire : %s\n'
		;;
		('en'|*)
			message='The PKG value used by %s is not part of the list of packages to build: %s\n'
		;;
	esac
	print_warning
	printf "$message" "$function" "$package"
	return 0
}

# display a warning when the selected architecture is not available
# USAGE: warning_architecture_not_available $architecture
warning_architecture_not_available() {
	local message architecture
	architecture="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Lʼarchitecture demandée nʼest pas disponible : %s\n'
		;;
		('en'|*)
			message='Selected architecture is not available: %s\n'
		;;
	esac
	print_warning
	printf "$message" "$architecture"
	return 0
}

# display a warning when using an option not supported by the current script
# USAGE: warning_option_not_supported $option
warning_option_not_supported() {
	local message option
	option="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Lʼoption %s nʼest pas gérée par ce script.\n'
		;;
		('en'|*)
			message='%s option is not supported by this script.\n'
		;;
	esac
	print_warning
	printf "$message" "$option"
	return 0
}

# display a warning when output package format guessing failed
# USAGE: warning_package_format_guessing_failed $fallback_value
warning_package_format_guessing_failed() {
	local message fallback_value
	fallback_value="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='Lʼauto-détection du format de paquet le plus adapté a échoué.\n'
			message="$message"'Le format de paquet %s sera utilisé par défaut.\n'
		;;
		('en'|*)
			message='Most pertinent package format auto-detection failed.\n'
			message="$message"'%s package format will be used by default.\n'
		;;
	esac
	print_warning
	printf "$message" "$fallback_value"
	return 0
}

# display a warning message if an icon dependency is missing
# USAGE: warning_icon_dependency_not_found $dependency
# NEEDED VARS: (LANG)
# CALLED BY: check_deps
warning_icon_dependency_not_found() {
	local message dependency
	dependency="$1"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='%s est introuvable. Installez-le pour inclure les icônes.\n'
			message="$message"'Vous pouvez aussi utiliser --icons=no ou --icons=yes\n\n'
		;;
		('en'|*)
			message='%s not found. Install it to include icons.\n'
			message="$message"'You can also use --icons=no or --icons=yes\n\n'
		;;
	esac
	print_warning
	printf "$message" "$dependency"
	return 0
}

# display a warning when using a library not available on target system in a
# given architecture
# USAGE: warning_missing_library $lib $target_system $architecture
warning_missing_library() {
	local lib
	local target_system
	local architecture
	lib="$1"
	target_system="$2"
	architecture="$3"
	# shellcheck disable=SC2031
	case "${LANG%_*}" in
		('fr')
			message='La bibliothèque %s nʼest pas disponible pour %s (architecture %s).\n'
		;;
		('en'|*)
			message='The library %s is not available on %s (%s architecture).\n'
		;;
	esac
	print_warning
	printf "$message" "$lib" "$target_system" "$architecture"
	return 0
}

# Keep compatibility with 2.12 and older

archives_get_list() {
	ARCHIVES_LIST=$(archives_return_list)
	export ARCHIVES_LIST
}

get_package_version() {
	PKG_VERSION=$(packages_get_version "$ARCHIVE")
	export PKG_VERSION
}

set_architecture() {
	pkg_architecture=$(package_get_architecture_string "$1")
	export pkg_architecture
}

icons_linking_postinst() {
	if \
		! version_is_at_least '2.8' "$target_version" && \
		[ -z "${PACKAGES_LIST##*PKG_DATA*}" ]
	then
		(
			PKG='PKG_DATA'
			icons_get_from_package "$@"
		)
	else
		icons_get_from_package "$@"
	fi
}

archive_set() {
	archive_initialize_optional "$@"
	# shellcheck disable=SC2039
	local archive
	archive=$(archive_find_from_candidates "$@")
	if [ -n "$archive" ]; then
		ARCHIVE="$archive"
		export ARCHIVE
	fi
}

version_target_is_older_than() {
	if [ "$1" = "${VERSION_MAJOR_TARGET}.${VERSION_MINOR_TARGET}" ]; then
		return 1
	fi
	version_is_at_least "${VERSION_MAJOR_TARGET}.${VERSION_MINOR_TARGET}" "$1"
}

# Keep compatibility with 2.11 and older

compat_pkg_write_arch_postinst() {
	local target
	target="$1"
	cat >> "$target" <<- EOF
	post_install() {
	$(cat "$postinst")
	}

	post_upgrade() {
	post_install
	}
	EOF
}

compat_pkg_write_arch_prerm() {
	local target
	target="$1"
	cat >> "$target" <<- EOF
	pre_remove() {
	$(cat "$prerm")
	}

	pre_upgrade() {
	pre_remove
	}
	EOF
}

compat_pkg_write_deb_postinst() {
	local target
	target="$1"
	cat > "$target" <<- EOF
	#!/bin/sh -e

	$(cat "$postinst")

	exit 0
	EOF
	chmod 755 "$target"
}

compat_pkg_write_deb_prerm() {
	local target
	target="$1"
	cat > "$target" <<- EOF
	#!/bin/sh -e

	$(cat "$prerm")

	exit 0
	EOF
	chmod 755 "$target"
}

compat_pkg_write_gentoo_postinst() {
	local target
	target="$1"
	cat >> "$target" <<- EOF
	pkg_postinst() {
	$(cat "$postinst")
	}
	EOF
}

compat_pkg_write_gentoo_prerm() {
	local target
	target="$1"
	cat >> "$target" <<- EOF
	pkg_prerm() {
	$(cat "$prerm")
	}
	EOF
}

liberror() {
	error_invalid_argument "$1" "$2"
}

skipping_pkg_warning() {
	warning_skip_package "$1" "$2"
}

archive_set_error_not_found() {
	error_archive_not_found "$@"
}

archive_guess_type_error() {
	error_archive_type_not_set "$1"
}

archive_print_file_in_use() {
	information_file_in_use "$1"
}

archive_integrity_check_error() {
	error_hashsum_mismatch "$1"
}

select_package_architecture_warning_unavailable() {
	warning_architecture_not_available "$OPTION_ARCHITECTURE"
}

select_package_architecture_error_unknown() {
	error_architecture_not_supported "$OPTION_ARCHITECTURE"
}

select_package_architecture_warning_unsupported() {
	warning_option_not_supported '--architecture'
}

error_no_pkg() {
	error_variable_not_set "$1" '$PKG'
}

set_temp_directories_error_no_size() {
	error_variable_not_set 'set_temp_directories' '$ARCHIVE_SIZE'
}

prepare_package_layout_error_no_list() {
	error_variable_not_set 'prepare_package_layout' '$PACKAGES_LIST'
}

organize_data_error_missing_pkg() {
	error_variable_not_set 'organize_data' '$PKG'
}

icon_path_empty_error() {
	error_variable_not_set 'icons_get_from_path' '$'"$1"
}

set_temp_directories_error_not_enough_space() {
	# shellcheck disable=SC2046
	error_not_enough_free_space $(temporary_directories_list_candidates)
}

archive_extraction_innosetup_error_version() {
	error_innoextract_version_too_old "$1"
}

icon_file_not_found_error() {
	error_icon_file_not_found "$1"
}

missing_pkg_error() {
	error_invalid_argument 'PKG' "$1"
}

pkg_build_print_already_exists() {
	information_package_already_exists "$1"
}

archive_integrity_check_print() {
	information_file_integrity_check "$1"
}

extract_data_from_print() {
	information_archive_data_extraction "$1"
}

pkg_print() {
	information_package_building "$1"
}

# Keep compatibility with 2.10 and older

write_bin() {
	local application
	for application in "$@"; do
		launcher_write_script "$application"
	done
}

write_desktop() {
	local application
	for application in "$@"; do
		launcher_write_desktop "$application"
	done
}

write_desktop_winecfg() {
	launcher_write_desktop 'APP_WINECFG'
}

write_launcher() {
	launchers_write "$@"
}

# Keep compatibility with 2.8 and older

icon_check_file_existence_pre_2_8() {
	# shellcheck disable=SC2039
	local directory file
	directory="$1"
	file="$2"

	if \
		[ -z "${file##* *}" ] || \
		[ ! -f "$directory"/$file ]
	then
		error_icon_file_not_found "$directory/$file"
		return 1
	else
		# get the real file name from its globbed one
		# shellcheck disable=SC2039
		local file_path
		file_path=$(eval printf '%s' "$directory/$file")
		file="${file_path#${directory}/}"
	fi

	printf '%s' "$file"
	return 0
}

# Keep compatibility with 2.7 and older

extract_and_sort_icons_from() {
	icons_get_from_package "$@"
}

extract_icon_from() {
	# Do nothing if the calling script explicitely asked for skipping icons extraction
	[ $SKIP_ICONS -eq 1 ] && return 0

	local destination
	local file
	destination="$PLAYIT_WORKDIR/icons"
	mkdir --parents "$destination"
	for file in "$@"; do
		extension="${file##*.}"
		case "$extension" in
			('exe')
				icon_extract_ico_from_exe "$file" "$destination"
			;;
			(*)
				icon_extract_png_from_file "$file" "$destination"
			;;
		esac
	done
}

get_icon_from_temp_dir() {
	icons_get_from_workdir "$@"
}

move_icons_to() {
	icons_move_to "$@"
}

postinst_icons_linking() {
	icons_linking_postinst "$@"
}

# Keep compatibility with 2.6.0 and older

set_archive() {
	archive_set "$@"
}

set_archive_error_not_found() {
	archive_set_error_not_found "$@"
}

if [ "$(basename "$0")" != 'libplayit2.sh' ] && [ -z "$LIB_ONLY" ]; then

	# Exit immediately on error
	set -o errexit

	# Set input field separator to default value (space, tab, newline)
	unset IFS

	# Unset variables that we do not want to import from the user environment

	unset OPTION_ARCHITECTURE
	unset OPTION_CHECKSUM
	unset OPTION_COMPRESSION
	unset OPTION_PREFIX
	unset OPTION_PACKAGE
	unset SOURCE_ARCHIVE
	unset PLAYIT_WORKDIR
	unset winecfg_desktop
	unset winecfg_launcher

	# Set URLs for error messages

	PLAYIT_GAMES_BUG_TRACKER_URL='https://forge.dotslashplay.it/play.it/games/issues'
	PLAYIT_BUG_TRACKER_URL='https://forge.dotslashplay.it/play.it/scripts/issues'

	# Check library version against script target version

	if [ -z "${target_version:=}" ]; then
		error_missing_target_version
	fi
	VERSION_MAJOR_PROVIDED="${LIBRARY_VERSION%%.*}"
	VERSION_MAJOR_TARGET="${target_version%%.*}"
	VERSION_MINOR_PROVIDED=$(printf '%s' "$LIBRARY_VERSION" | cut --delimiter='.' --fields=2)
	VERSION_MINOR_TARGET=$(printf '%s' "$target_version" | cut --delimiter='.' --fields=2)
	if \
		[ $VERSION_MAJOR_PROVIDED -ne $VERSION_MAJOR_TARGET ] || \
		[ $VERSION_MINOR_PROVIDED -lt $VERSION_MINOR_TARGET ]
	then
		error_incompatible_versions
	fi
	export VERSION_MAJOR_PROVIDED VERSION_MAJOR_TARGET VERSION_MINOR_PROVIDED VERSION_MINOR_TARGET

	# Set allowed values for common options

	# shellcheck disable=SC2034
	ALLOWED_VALUES_ARCHITECTURE='all 32 64 auto'
	# shellcheck disable=SC2034
	ALLOWED_VALUES_CHECKSUM='none md5'
	# shellcheck disable=SC2034
	ALLOWED_VALUES_COMPRESSION_DEB='none gzip xz'
	# shellcheck disable=SC2034
	ALLOWED_VALUES_COMPRESSION_ARCH='none gzip xz bzip2 zstd'
	# shellcheck disable=SC2034
	ALLOWED_VALUES_COMPRESSION_GENTOO='gzip xz bzip2 zstd lz4 lzip lzop'
	# shellcheck disable=SC2034
	ALLOWED_VALUES_COMPRESSION_EGENTOO='none gzip xz bzip2 zstd lzip lzop'
	# shellcheck disable=SC2034
	ALLOWED_VALUES_PACKAGE='arch deb gentoo egentoo'
	# shellcheck disable=SC2034
	ALLOWED_VALUES_ICONS='yes no auto'

	# Set default values for common options

	# shellcheck disable=SC2034
	DEFAULT_OPTION_ARCHITECTURE='all'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_CHECKSUM='md5'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_COMPRESSION_ARCH='zstd'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_COMPRESSION_DEB='none'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_COMPRESSION_GENTOO='bzip2'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_COMPRESSION_EGENTOO='bzip2'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_PREFIX_DEB='/usr/local'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_PREFIX_ARCH='/usr'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_PREFIX_GENTOO='/usr'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_PREFIX_EGENTOO='/usr'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_PACKAGE='deb'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_ICONS='yes'
	# shellcheck disable=SC2034
	DEFAULT_OPTION_OUTPUT_DIR="$PWD"

	# Parse arguments given to the script

	DRY_RUN='0'
	NO_FREE_SPACE_CHECK='0'
	SKIP_ICONS=0
	OVERWRITE_PACKAGES=0
	DEBUG=0

	while [ $# -gt 0 ]; do
		case "$1" in
			('--help')
				help
				exit 0
			;;
			('--architecture='*|\
			 '--architecture'|\
			 '--checksum='*|\
			 '--checksum'|\
			 '--compression='*|\
			 '--compression'|\
			 '--prefix='*|\
			 '--prefix'|\
			 '--package='*|\
			 '--package'|\
			 '--icons='*|\
			 '--icons'|\
			 '--output-dir='*|\
			 '--output-dir')
				if [ "${1%=*}" != "${1#*=}" ]; then
					option="$(printf '%s' "${1%=*}" | sed 's/^--//;s/-/_/g')"
					value="${1#*=}"
				else
					option="$(printf '%s' "$1" | sed 's/^--//;s/-/_/g')"
					value="$2"
					shift 1
				fi
				if [ "$value" = 'help' ]; then
					eval help_$option
					exit 0
				else
					# shellcheck disable=SC2046
					eval OPTION_$(printf '%s' "$option" | sed 's/-/_/g' | tr '[:lower:]' '[:upper:]')=\"$value\"
					# shellcheck disable=SC2046
					export OPTION_$(printf '%s' "$option" | sed 's/-/_/g' | tr '[:lower:]' '[:upper:]')
				fi
				unset option
				unset value
			;;
			('--dry-run')
				DRY_RUN='1'
				export DRY_RUN
			;;
			('--skip-free-space-check')
				NO_FREE_SPACE_CHECK='1'
				export NO_FREE_SPACE_CHECK
			;;
			('--overwrite')
				OVERWRITE_PACKAGES=1
				export OVERWRITE_PACKAGES
			;;
			('--debug'|'--debug='*)
				if [ "${1%=*}" != "${1#*=}" ]; then
					DEBUG="${1#*=}"
				else
					case "$2" in
						([0-9])
							DEBUG="$2"
							shift 1
							;;
						(*)
							DEBUG=1
					esac
				fi
				export DEBUG
			;;
			('--'*)
				error_option_unknown "$1"
			;;
			(*)
				if [ -f "$1" ]; then
					SOURCE_ARCHIVE="$1"
					export SOURCE_ARCHIVE
				else
					error_not_a_file "$1"
				fi
			;;
		esac
		shift 1
	done

	# Try to detect the host distribution if no package format has been explicitely set

	[ "$OPTION_PACKAGE" ] || packages_guess_format 'OPTION_PACKAGE'

	# Check option validity for the package format, since it will be used for the compression method

	check_option_validity 'PACKAGE'

	# Set allowed and default values for compression and prefix depending on the chosen package format

	# shellcheck disable=SC2034
	ALLOWED_VALUES_COMPRESSION="$(get_value "ALLOWED_VALUES_COMPRESSION_$(printf '%s' "$OPTION_PACKAGE" | tr '[:lower:]' '[:upper:]')")"
	# shellcheck disable=SC2034
	DEFAULT_OPTION_COMPRESSION="$(get_value "DEFAULT_OPTION_COMPRESSION_$(printf '%s' "$OPTION_PACKAGE" | tr '[:lower:]' '[:upper:]')")"
	# shellcheck disable=SC2034
	DEFAULT_OPTION_PREFIX="$(get_value "DEFAULT_OPTION_PREFIX_$(printf '%s' "$OPTION_PACKAGE" | tr '[:lower:]' '[:upper:]')")"

	# Set options not already set by script arguments to default values

	for option in 'ARCHITECTURE' 'CHECKSUM' 'COMPRESSION' 'ICONS' 'OUTPUT_DIR' 'PREFIX'; do
		if
			[ -z "$(get_value "OPTION_$option")" ] && \
			[ -n "$(get_value "DEFAULT_OPTION_$option")" ]
		then
			# shellcheck disable=SC2046
			eval OPTION_$option=\"$(get_value "DEFAULT_OPTION_$option")\"
			export OPTION_$option
		fi
	done

	# Check options values validity

	for option in 'CHECKSUM' 'COMPRESSION' 'ICONS'; do
		check_option_validity "$option"
	done

	if [ "$OPTION_ICONS" = 'no' ]; then
		SKIP_ICONS=1
		export SKIP_ICONS
	fi

	case "$DEBUG" in
		([0-9]) ;;
		(*)
			error_option_invalid 'DEBUG' "$DEBUG"
		;;
	esac

	# DEBUG: output all options value
	for option in 'DRY_RUN' 'NO_FREE_SPACE_CHECK' \
		'SKIP_ICONS' 'OVERWRITE_PACKAGES' 'DEBUG'; do
		debug_option_value "$option"
	done
	for option in 'ARCHITECTURE' 'CHECKSUM' 'COMPRESSION' \
		'PREFIX' 'PACKAGE' 'OUTPUT_DIR'; do
		debug_option_value "OPTION_$option"
	done

	# Make sure the output directory exists and is writable

	OPTION_OUTPUT_DIR=$(printf '%s' "$OPTION_OUTPUT_DIR" | sed "s#^~#$HOME#")
	if [ ! -d "$OPTION_OUTPUT_DIR" ]; then
		error_not_a_directory "$OPTION_OUTPUT_DIR"
	fi
	if [ ! -w "$OPTION_OUTPUT_DIR" ]; then
		error_not_writable "$OPTION_OUTPUT_DIR"
	fi
	export OPTION_OUTPUT_DIR

	# Restrict packages list to target architecture

	select_package_architecture

	# Check script dependencies

	check_deps

	# Set package paths

	case $OPTION_PACKAGE in
		('arch'|'gentoo'|'egentoo')
			PATH_BIN="$OPTION_PREFIX/bin"
			PATH_DESK="$DEFAULT_OPTION_PREFIX/share/applications"
			PATH_DOC="$OPTION_PREFIX/share/doc/$GAME_ID"
			PATH_GAME="$OPTION_PREFIX/share/$GAME_ID"
			PATH_ICON_BASE="$DEFAULT_OPTION_PREFIX/share/icons/hicolor"
		;;
		('deb')
			PATH_BIN="$OPTION_PREFIX/games"
			PATH_DESK="$DEFAULT_OPTION_PREFIX/share/applications"
			PATH_DOC="$OPTION_PREFIX/share/doc/$GAME_ID"
			PATH_GAME="$OPTION_PREFIX/share/games/$GAME_ID"
			PATH_ICON_BASE="$DEFAULT_OPTION_PREFIX/share/icons/hicolor"
		;;
		(*)
			error_invalid_argument 'OPTION_PACKAGE' "$0"
		;;
	esac

	# Set main archive

	# shellcheck disable=SC2046
	archive_initialize_required 'SOURCE_ARCHIVE' $(archives_return_list)
	# shellcheck disable=SC2046
	ARCHIVE=$(archive_find_from_candidates 'SOURCE_ARCHIVE' $(archives_return_list))
	export ARCHIVE

	# Set working directories

	# shellcheck disable=SC2046
	set_temp_directories $(packages_get_list)

fi
